@@ -1,7 +1,7 @@
 /*
  * file bomb.c - bombs and explosions
  *
- * $Id: bomb.c,v 1.9 2004/06/28 13:36:16 iskywalker Exp $
+ * $Id: bomb.c,v 1.23 2005/01/12 15:48:22 lodott Exp $
  *
  * Program XBLAST 
  * (C) 1993-1999 by Oliver Vogel (e-mail: m.vogel@ndh.net)
@@ -153,14 +153,17 @@ static void SpreXDir (int lx, int ly, int range, int type,
 static Explosion *bombMaze[MAZE_W][MAZE_H];
 /* list with all bombs and explosions */
 static int numExpl = 0 ;
- Explosion *explList;                
+Explosion *explList;
 static Explosion *explEnd;
- int initialBombDir;
+int initialBombDir;
 static int hauntFactor;
- int curBombTime;
+int curBombTime;
 int defaultBMT;
 static int  specialBMT, evilBMT;
+static int slowMotionBurst=0;
 static int nextNasty;
+static int ceilNasty;
+static int divNextNasty;
 static int gentleNasty;
 static int rangeNasty;
 /* bomb click functions */
@@ -187,7 +190,7 @@ static XBSpecialBombFunc doSpecialBombFunction[NUM_BMT] = {
   SpecialBombNormal,
   SpecialBombNormal,
   /* EPFL SHIT */  /* FUNCTION NAMES */
-  SpecialBombDiagThree,   
+  SpecialBombDiagThree,
   SpecialBombScissor,
   SpecialBombScissor2,
   SpecialBombParallel,
@@ -212,12 +215,14 @@ static DBToData bombClickTable[] = {
   { "clockwise",     (void *) BombClickClockwise },
   { "contact",       (void *) BombClickContact },
   { "initial",       (void *) BombClickInitial },
+  { "none",          (void *) BombClickNone },
+  { "null",          (void *) BombClickNone },
   { "randomdir",     (void *) BombClickRandomdir },
   { "rebound",       (void *) BombClickRebound },
   { "snooker",       (void *) BombClickSnooker },
   { "split",         (void *) BombClickSplit },
   { "thru",          (void *) BombClickThru },
-  { NULL,            (void *) BombClickNone },
+  { NULL, NULL },
 };
 /* conversion table for direction */
 static DBToInt bombDirTable[] = {
@@ -225,13 +230,15 @@ static DBToInt bombDirTable[] = {
   { "left",  GoLeft },
   { "right", GoRight },
   { "up",    GoUp },
-  { NULL,    GoStop }
+  { "stop",  GoStop },
+  { NULL, -1 }
 };
 /* conversion table for fuse times */
 static DBToInt fuseTimeTable[] = {
   { "long",  LONG_FUSE },
   { "short", SHORT_FUSE },
-  { NULL,    BOMB_TIME }
+  { "normal", BOMB_TIME },
+  { NULL,    -1 }
 };
 
 /* conversion table for bomb types */
@@ -243,6 +250,7 @@ static DBToInt bombTypeTable[] = { /* NAME IN LEVEL FILE ALFABETICAL ORDER !!! *
   { "column",         BMTcolumn },
   { "comb",           BMTcomb },
   { "construction",   BMTconstruction }, 
+  { "default",        BMTdefault },
   { "destruction",    BMTdestruction }, 
   { "diagional",      BMTdiagthreebombs }, /* EPFL */
   { "distance",       BMTdistance },
@@ -253,7 +261,8 @@ static DBToInt bombTypeTable[] = { /* NAME IN LEVEL FILE ALFABETICAL ORDER !!! *
   { "grenade",        BMTgrenade },
   { "lucky",          BMTlucky },
   { "mine",           BMTmine },
-  { "napalm",         BMTnapalm }, 
+  { "napalm",         BMTnapalm },
+  { "normal",         BMTnormal },
   { "nuclear",        BMTnuclear },
   { "parallel",       BMTparallel },
   { "parasol",        BMTparasol },
@@ -269,15 +278,17 @@ static DBToInt bombTypeTable[] = { /* NAME IN LEVEL FILE ALFABETICAL ORDER !!! *
   { "scissor2",       BMTscissor2 },
   { "search",         BMTsearch },
   { "short",          BMTshort },
+  { "snipe",          BMTsnipe },
   { "threebombs",     BMTthreebombs }, 
   { "trianglebombs",  BMTtrianglebombs }, 
-  { NULL,             BMTdefault },
+  { NULL,             NUM_BMT },
 };
 /* bomb haunting */
 static DBToInt hauntFactorTable[] = {
   { "fast", HAUNT_FAST },
   { "slow", HAUNT_SLOW },
-  { NULL,   HAUNT_NONE },
+  { "none", HAUNT_NONE },
+  { NULL,   -1 },
 };
 /* bomb extra info */
 static char *bombName[] = {
@@ -324,81 +335,206 @@ static const char *bombDirInfo[MAX_DIR] = {
   "Bombs are going right",
   NULL,
 };
+static char* bmNormalName = "normal";
+ 
+const char* GetBombName(BMBombType type){
+
+  int i;
+  
+  if (type ==BMTnormal )
+    return bmNormalName;
+  for (i = 0; i <NUM_BMT ; ++i)
+    if (bombTypeTable[i].value == type)
+      return bombTypeTable[i].key;
+  return bmNormalName;
+
+
+}
+void SetSlowMotionBurst(int flame){
+  slowMotionBurst=flame;
+}
 
 /*
- * load bombs from level
+ * load bombs from level data
  */
 XBBool
-ConfigLevelBombs (const DBSection *section)
+ParseLevelBombs (const DBSection *section, DBSection *warn)
 {
   void *ptr;
-  const char *s;
-
-  assert (section != NULL);
 
   assert (NULL == explList);
   assert (NULL == explEnd);
   assert (0    == numExpl);
-
+  /* check if section exists */
+  if (NULL == section) {
+    Dbg_Out("LEVEL: bomb section is missing!\n");
+    DB_CreateEntryString(warn,atomMissing,"true");
+    return XBFalse;
+  }
   /* clear list */
   ptr = NULL;
   /* clear lookup table */
   memset (bombMaze, 0, sizeof (bombMaze) );
-  /* set bomb click behavior */
-  if (DB_ConvertEntryData (section, atomBombClick, &ptr, bombClickTable) ) {
-    doBombClick = (XBBombClickFunc) ptr;
-  } else {
+  /* BombClick has default */
+  switch (DB_ConvertEntryData (section, atomBombClick, &ptr, bombClickTable) ) {
+  case DCR_NoSuchEntry:
+    Dbg_Level("default for %s\n",  DB_SectionEntryString(section,atomBombClick));
     doBombClick = BombClickNone;
+    break;
+  case DCR_Failure:
+    Dbg_Out("LEVEL: warning for %s\n", DB_SectionEntryString(section, atomBombClick));
+    doBombClick = BombClickNone;
+    DB_CreateEntryString(warn,atomBombClick, DB_DataToString(bombClickTable, doBombClick));
+    break;
+  default:
+    doBombClick = (XBBombClickFunc) ptr;
+    break;
   }
-  if (DB_ConvertEntryData (section, atomWallClick, &ptr, bombClickTable) ) {
-    doWallClick = (XBBombClickFunc) ptr;
-  } else {
+  /* WallClick has default */
+  switch (DB_ConvertEntryData (section, atomWallClick, &ptr, bombClickTable) ) {
+  case DCR_NoSuchEntry:
+    Dbg_Level("default for %s\n",  DB_SectionEntryString(section,atomWallClick));
     doWallClick = BombClickNone;
+    break;
+  case DCR_Failure:
+    Dbg_Out("LEVEL: warning for %s\n", DB_SectionEntryString(section, atomWallClick));
+    doWallClick = BombClickNone;
+    DB_CreateEntryString(warn,atomWallClick, DB_DataToString(bombClickTable, doWallClick));
+    break;
+  default:
+    doWallClick = (XBBombClickFunc) ptr;
+    break;
   }
-  if (DB_ConvertEntryData (section, atomPlayerClick, &ptr, bombClickTable) ) {
-    doPlayerClick = (XBBombClickFunc) ptr;
-  } else {
+  /* PlayerClick has default */
+  switch (DB_ConvertEntryData (section, atomPlayerClick, &ptr, bombClickTable) ) {
+  case DCR_NoSuchEntry:
+    Dbg_Level("default for %s\n",  DB_SectionEntryString(section,atomPlayerClick));
     doPlayerClick = BombClickNone;
+    break;
+  case DCR_Failure:
+    Dbg_Out("LEVEL: warning for %s\n", DB_SectionEntryString(section, atomPlayerClick));
+    doPlayerClick = BombClickNone;
+    DB_CreateEntryString(warn,atomPlayerClick, DB_DataToString(bombClickTable, doPlayerClick));
+    break;
+  default:
+    doPlayerClick = (XBBombClickFunc) ptr;
+    break;
   }
-  /* general behavior */
-  if (! DB_ConvertEntryInt (section, atomDirection, (int *) &initialBombDir, bombDirTable) ) {
+  /* Direction has default */
+  switch (DB_ConvertEntryInt (section, atomDirection, &initialBombDir, bombDirTable) ) {
+  case DCR_NoSuchEntry:
+    Dbg_Level("default for %s\n",  DB_SectionEntryString(section,atomDirection));
     initialBombDir = GoStop;
+    break;
+  case DCR_Failure:
+    Dbg_Out("LEVEL: warning for %s\n", DB_SectionEntryString(section, atomDirection));
+    initialBombDir = GoStop;
+    DB_CreateEntryString(warn,atomDirection, DB_IntToString(bombDirTable, initialBombDir));
+    break;
+  default:
+    break;
   }
-  if (! DB_ConvertEntryInt (section, atomFuseTime, &curBombTime, fuseTimeTable) ) {
+  /* FuseTime has default */
+  switch (DB_ConvertEntryInt (section, atomFuseTime, &curBombTime, fuseTimeTable) ) {
+  case DCR_NoSuchEntry:
+    Dbg_Level("default for %s\n",  DB_SectionEntryString(section,atomFuseTime));
     curBombTime = BOMB_TIME;
+    break;
+  case DCR_Failure:
+    Dbg_Out("LEVEL: warning for %s\n", DB_SectionEntryString(section, atomFuseTime));
+    curBombTime = BOMB_TIME;
+    DB_CreateEntryString(warn,atomFuseTime, DB_IntToString(fuseTimeTable, curBombTime));
+    break;
+  default:
+    break;
   }
-  if (! DB_ConvertEntryInt (section, atomHaunt, &hauntFactor, hauntFactorTable) ) {
+  /* Haunt has default */
+  switch (DB_ConvertEntryInt (section, atomHaunt, &hauntFactor, hauntFactorTable) ) {
+  case DCR_NoSuchEntry:
+    Dbg_Level("default for %s\n",  DB_SectionEntryString(section,atomHaunt));
     hauntFactor = HAUNT_NONE;
+    break;
+  case DCR_Failure:
+    Dbg_Out("LEVEL: warning for %s\n", DB_SectionEntryString(section, atomHaunt));
+    hauntFactor = HAUNT_NONE;
+    DB_CreateEntryString(warn,atomHaunt, DB_IntToString(hauntFactorTable, hauntFactor));
+    break;
+  default:
+    break;
   }
-  /* bomb types */
-  if (! DB_ConvertEntryInt (section, atomDefault, (int *) &defaultBMT, bombTypeTable) ) {
-    fprintf(stderr,"*** UNKONWN DEFAULT BOMB ***");
+  /* Haunt has default */
+  switch (DB_ConvertEntryInt (section, atomDefault, &defaultBMT, bombTypeTable) ) {
+  case DCR_NoSuchEntry:
+    Dbg_Level("default for %s\n",  DB_SectionEntryString(section,atomDefault));
     defaultBMT = BMTnormal;
+    break;
+  case DCR_Failure:
+    Dbg_Out("LEVEL: warning for %s\n", DB_SectionEntryString(section, atomDefault));
+    defaultBMT = BMTnormal;
+    DB_CreateEntryString(warn,atomDefault, DB_IntToString(bombTypeTable, defaultBMT));
+    break;
+  default:
+    break;
   }
-  fprintf(stderr,"DEFAULT BOMB = %d\n",defaultBMT);
-  if (! DB_ConvertEntryInt (section, atomSpecial, (int *) &specialBMT, bombTypeTable) ) {
-    fprintf(stderr,"*** UNKONWN SPECIAL BOMB ***");
+  /* Special has default */
+  switch (DB_ConvertEntryInt (section, atomSpecial, &specialBMT, bombTypeTable) ) {
+  case DCR_NoSuchEntry:
+    Dbg_Level("default for %s\n",  DB_SectionEntryString(section,atomSpecial));
     specialBMT = BMTnormal;
+    break;
+  case DCR_Failure:
+    Dbg_Out("LEVEL: warning for %s\n", DB_SectionEntryString(section, atomSpecial));
+    specialBMT = BMTnormal;
+    DB_CreateEntryString(warn,atomSpecial, DB_IntToString(bombTypeTable, specialBMT));
+    break;
+  default:
+    break;
   }
-  fprintf(stderr,"SPECIAL BOMB = %d\n",specialBMT);
-  if (! DB_ConvertEntryInt (section, atomHidden, (int *) &evilBMT, bombTypeTable) ) {
+  /* Hidden has default */
+  switch (DB_ConvertEntryInt (section, atomHidden, &evilBMT, bombTypeTable) ) {
+  case DCR_NoSuchEntry:
+    Dbg_Level("default for %s\n",  DB_SectionEntryString(section,atomHidden));
     evilBMT = BMTnormal;
+    break;
+  case DCR_Failure:
+    Dbg_Out("LEVEL: warning for %s\n", DB_SectionEntryString(section, atomHidden));
+    evilBMT = BMTnormal;
+    DB_CreateEntryString(warn,atomHidden, DB_IntToString(bombTypeTable, evilBMT));
+    break;
+  default:
+    break;
   }
-  /* nasty walls .. */
+  /* NastyCeil has default */
+  if (! DB_GetEntryInt (section, atomNastyCeil, &ceilNasty)) {
+    Dbg_Level("default for %s\n",  DB_SectionEntryString(section,atomNastyCeil));
+    ceilNasty = 0;
+  }
+  /* NastyGentle/NastyRange have default */
   if (DB_GetEntryInt (section, atomNastyGentle, &gentleNasty) &&
       DB_GetEntryInt (section, atomNastyRange, &rangeNasty) ) {
-    /* ... activated */
-    nextNasty    = NASTY_INC;
+    /* check for NextNasty */
+    if (!DB_GetEntryInt (section, atomNextNasty, &divNextNasty) || (divNextNasty==0) ) {
+      divNextNasty = 1;
+    }
+    nextNasty    = NASTY_INC/divNextNasty;;
     gentleNasty *= GAME_TIME;
   }  else {
-    /* ... deactivated */
+    Dbg_Level("default nasty data\n");
     nextNasty   = GAME_TIME + NASTY_INC + 1;
     gentleNasty = 0;
     rangeNasty  = 0;
   }
-  /* 
-   * set level info text 
-   */
+  return XBTrue;
+}
+
+/* 
+ * set level info text 
+ */
+void
+ConfigLevelBombs (const DBSection *section)
+{
+  const char *s;
+
   /* bomb types */
   if (NULL != bombName[defaultBMT]) {
     AddLevelInfo ("%s as default", bombName[defaultBMT]);
@@ -477,9 +613,8 @@ ConfigLevelBombs (const DBSection *section)
   if (gentleNasty != 0) {
     AddLevelInfo ("The Walls launch bombs");
   }
-ChoiceDefaultBomb = defaultBMT; 
+  ChoiceDefaultBomb = defaultBMT;
   /* that's all */
-  return XBTrue;
 } /* ConfigBombs */
 
 /*
@@ -1109,63 +1244,64 @@ DoBombs (BMPlayer *ps, int numPlayer) // void changed for search bombs
 	/** **/
 	switch(ptr->dir) {
 	case GoUp:    MoveBombUp (ptr);
+/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ 
 /* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */       if(ptr->jump>0) {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */                 ptr->type=BMTshort;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */                 if (ptr->dy <= -BLOCK_HEIGHT/2) {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */                      bombMaze[ptr->x][ptr->y] = NULL;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */                      ptr->dy += BLOCK_HEIGHT;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */                      ptr->y -= 1;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */                      bombMaze[ptr->x][ptr->y] = ptr;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */                  }
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ 		ptr->dy -= BOMB_VY;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */                 if(ptr->y<=1) { ptr->dy=0; }
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */                 ptr->jump--;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */       } else {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */        break;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */        }
+                 ptr->type=BMTshort;
+                 if (ptr->dy <= -BLOCK_HEIGHT/2) {
+                      bombMaze[ptr->x][ptr->y] = NULL;
+                      ptr->dy += BLOCK_HEIGHT;
+                      ptr->y -= 1;
+                      bombMaze[ptr->x][ptr->y] = ptr;
+                  }
+ 		ptr->dy -= BOMB_VY;
+                 if(ptr->y<=1) { ptr->dy=0; }
+                 ptr->jump--;
+       } else {
+        break;
+        }
  break;
 	case GoDown:  MoveBombDown (ptr);
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */        if(ptr->jump>0) {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */         ptr->type=BMTshort;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */          if (ptr->dy >= BLOCK_HEIGHT/2) {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ 	  bombMaze[ptr->x][ptr->y] = NULL;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */           ptr->dy -= BLOCK_HEIGHT;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */           ptr->y += 1;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */           bombMaze[ptr->x][ptr->y] = ptr;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */           }
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */           ptr->dy += BOMB_VY;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */           if(ptr->y>=MAZE_H-2) { ptr->dy=0; }
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */           ptr->jump--;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */         } else {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */        }
+        if(ptr->jump>0) {
+         ptr->type=BMTshort;
+          if (ptr->dy >= BLOCK_HEIGHT/2) {
+ 	  bombMaze[ptr->x][ptr->y] = NULL;
+           ptr->dy -= BLOCK_HEIGHT;
+           ptr->y += 1;
+           bombMaze[ptr->x][ptr->y] = ptr;
+           }
+           ptr->dy += BOMB_VY;
+           if(ptr->y>=MAZE_H-2) { ptr->dy=0; }
+           ptr->jump--;
+         } else {
+        }
  break;
 	case GoRight: MoveBombRight (ptr); 
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */         if(ptr->jump>0) {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ 	       if (ptr->dx >= BLOCK_WIDTH/2) {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ 		  bombMaze[ptr->x][ptr->y] = NULL;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */                   ptr->dx -= BLOCK_WIDTH;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */                   ptr->x += 1;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */                   bombMaze[ptr->x][ptr->y] = ptr;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ 		}
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ 		ptr->dx += BOMB_VX;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */                 if(ptr->x>=MAZE_W-2) { ptr->dx=0; }
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ 		ptr->jump--;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */         } else {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */         }
+         if(ptr->jump>0) {
+ 	       if (ptr->dx >= BLOCK_WIDTH/2) {
+ 		  bombMaze[ptr->x][ptr->y] = NULL;
+                   ptr->dx -= BLOCK_WIDTH;
+                   ptr->x += 1;
+                   bombMaze[ptr->x][ptr->y] = ptr;
+ 		}
+ 		ptr->dx += BOMB_VX;
+                 if(ptr->x>=MAZE_W-2) { ptr->dx=0; }
+ 		ptr->jump--;
+         } else {
+         }
 break;
 	case GoLeft:  MoveBombLeft (ptr);
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */        if(ptr->jump>0) {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */          if (ptr->dx <= -BLOCK_WIDTH/2) {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */             bombMaze[ptr->x][ptr->y] = NULL;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */             ptr->dx += BLOCK_WIDTH;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */             ptr->x -= 1;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */             bombMaze[ptr->x][ptr->y] = ptr;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */          }
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */          ptr->dx -= BOMB_VX;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */          if(ptr->x<=1) { ptr->dx=0; }
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ 	 ptr->jump--;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */        } else {  
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */         }
+        if(ptr->jump>0) {
+          if (ptr->dx <= -BLOCK_WIDTH/2) {
+             bombMaze[ptr->x][ptr->y] = NULL;
+             ptr->dx += BLOCK_WIDTH;
+             ptr->x -= 1;
+             bombMaze[ptr->x][ptr->y] = ptr;
+          }
+          ptr->dx -= BOMB_VX;
+          if(ptr->x<=1) { ptr->dx=0; }
+ 	 ptr->jump--;
+        } else {  
+         }
  break;
 	default:      break;
 	}
@@ -1173,7 +1309,6 @@ break;
       } else {
 	if(NULL != ptr->player) {
 	//	ptr->player->x = ptr->dx + ptr->x * BLOCK_WIDTH;
-		
 	//	ptr->player->y = ptr->dy + (ptr->y-1) * BLOCK_HEIGHT; 
 	//	move_sprite (ptr->player->sprite, ptr->player->x, ptr->player->y + BASE_Y); 
 	  switch(ptr->player->d_soll) {
@@ -1350,6 +1485,7 @@ DoExplosions (void)
       } else {
 	/* get exploding time */
 	if ( (hilf = ptr->count) == 0) {
+	
 	  /* set any free tiel to burned */
 	  if (CheckMazeFree (ptr->x, ptr->y) ) {
 	    SetMazeBlock (ptr->x, ptr->y, BTBurned);
@@ -1368,24 +1504,32 @@ DoExplosions (void)
       }
     }
     if ((*doSpecialBombFunction[ptr->type])(ptr)) {
-      ptr->count ++;
+      if (slowMotionBurst) {
+	if( ptr->count>0){
+	  ptr->count += ptr->countslower2 % slowMotionBurst==0;
+	} else {
+	  ptr->count++;
+	}
+	ptr->countslower+= ptr->countslower2%slowMotionBurst==0;
+	ptr->countslower2++;
+      } else {
+	ptr->count++;
+      }
       if ( 2 == ptr->count && 
 	   NULL != ptr->sprite ) {
 	DeleteSprite (ptr->sprite);
 	ptr->sprite = NULL;
       }
-    }else{
-
-   /* if(doSpecialBombFunction[ptr->type]==SpecialBombDestruction||
-       doSpecialBombFunction[ptr->type]==SpecialBombConstruction){
-      
-    }*/ptr=NULL;
+    } else {
+      /* bomb deleted you must set explicitly to NULL (dont ask me why) */
+      /* if (doSpecialBombFunction[ptr->type]==SpecialBombDestruction||
+         doSpecialBombFunction[ptr->type]==SpecialBombConstruction){	 
+    }*/
+      ptr=NULL;
     }
-    
-    if(ptr!=NULL){
-    ptr=ptr->next;
-    }
-    else{
+    if (ptr!=NULL) {
+      ptr=ptr->next;
+    } else {
        ptr = nextPtr;
     }
   }
@@ -1790,11 +1934,13 @@ NewExplosion (BMPlayer *player, int x, int y, int range, XBBool remote_controlle
     newExpl->range = range;
   }
   /* bomb counter */
+    newExpl->countslower = -curBombTime;
   if ( (type == BMTblastnow) || 
        (type == BMTfirecracker2) || 
        (type == BMTpyro2) ) {
     /* bombs which explode immediately */
     newExpl->count = 0;
+    newExpl->countslower = 0;
   } else if (remote_controlled) {
     /* remote controled bombs */
     newExpl->count     = -GAME_TIME;
@@ -2113,41 +2259,42 @@ DoSuck (BMPlayer *ps)
  } /* do suck */
 
 
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ void
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ DoJump (BMPlayer *ps)
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */   Explosion *ptr;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */   int x,y,ex,ey;
 /* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ 
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */   for (ptr = explList; ptr != NULL; ptr = ptr->next) {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */     if ((ptr->dir == GoStop))  {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */       x = (ptr->x) * BLOCK_WIDTH;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */       y = (ptr->y-1) * BLOCK_HEIGHT;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */       ex = x - ps->x;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */       ey = y - ps->y;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */       if ((ABS(ex)<BLOCK_WIDTH*2)&&(ABS(ey)<BLOCK_HEIGHT*2)
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */           && ((ex!=0)||(ey!=0))) {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ #ifdef SCORE
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */         ptr->score_flag |= 1<<(ps->id);
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ #endif
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */         ptr->jump=(int)(7*ps->jump_button);
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */         if (ABS(ex)*BLOCK_HEIGHT >= ABS(ey)*BLOCK_WIDTH)  {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */           if (ex<0) {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */             ptr->dir=GoLeft;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */           } else {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */             ptr->dir=GoRight;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */           }
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */         } else {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */           if (ey<0) {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */             ptr->dir=GoUp;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */           } else {
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */             ptr->dir=GoDown;
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */           }
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */         }
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */       }
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */     }
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */   }
-/* Added by Fouf on 09/02/99 22:46:25 */ /* Added by "Belgium Guys" */ }
+void
+DoJump (BMPlayer *ps)
+{
+  Explosion *ptr;
+  int x,y,ex,ey;
+  
+  for (ptr = explList; ptr != NULL; ptr = ptr->next) {
+    if ((ptr->dir == GoStop))  {
+      x = (ptr->x) * BLOCK_WIDTH;
+      y = (ptr->y-1) * BLOCK_HEIGHT;
+      ex = x - ps->x;
+      ey = y - ps->y;
+      if ((ABS(ex)<BLOCK_WIDTH*2)&&(ABS(ey)<BLOCK_HEIGHT*2)
+	  && ((ex!=0)||(ey!=0))) {
+#ifdef SCORE
+	ptr->score_flag |= 1<<(ps->id);
+#endif
+	ptr->jump=(int)(7*ps->jump_button);
+	if (ABS(ex)*BLOCK_HEIGHT >= ABS(ey)*BLOCK_WIDTH)  {
+	  if (ex<0) {
+	    ptr->dir=GoLeft;
+           } else {
+             ptr->dir=GoRight;
+           }
+	} else {
+	  if (ey<0) {
+	    ptr->dir=GoUp;
+	  } else {
+	    ptr->dir=GoDown;
+	  }
+	}
+      }
+    }
+  }
+}
 
 /*
  *
@@ -2163,26 +2310,33 @@ DoNastyWalls (int gameTime)
       nextNasty += NASTY_INC*2;
     }
     if (GameRandomNumber (gentleNasty) < gameTime) {
-      dir = GoUp + GameRandomNumber (4);
-      switch (dir) {
-      case GoUp:
-	x   = GameRandomNumber (MAZE_W-2) + 1;
-	y   = MAZE_H-2;
-	break;
-      case GoLeft:
-	x   = MAZE_W-2;
-	y   = GameRandomNumber (MAZE_H-2) + 1;
-	break;
-      case GoDown:
-	x   = GameRandomNumber (MAZE_W-2) + 1;
-	y   = 1;
-	break;
-      case GoRight:
-	x   = 1;
-	y   = GameRandomNumber (MAZE_H-2) + 1;
-	break;
-      default:
-	return;
+      if(ceilNasty){
+	x = GameRandomNumber (MAZE_W-2) + 1;
+	y = GameRandomNumber (MAZE_H-2) + 1;
+        dir=GoDown;
+        y=1;
+      }else{
+	dir = GoUp + GameRandomNumber (4);
+	switch (dir) {
+	case GoUp:
+	  x   = GameRandomNumber (MAZE_W-2) + 1;
+	  y   = MAZE_H-2;
+	  break;
+	case GoLeft:
+	  x   = MAZE_W-2;
+	  y   = GameRandomNumber (MAZE_H-2) + 1;
+	  break;
+	case GoDown:
+	  x   = GameRandomNumber (MAZE_W-2) + 1;
+	  y   = 1;
+	  break;
+	case GoRight:
+	  x   = 1;
+	  y   = GameRandomNumber (MAZE_H-2) + 1;
+	  break;
+	default:
+	  return;
+	}
       }
       NewExplosion (NULL, x, y, rangeNasty, XBFalse, XBFalse, BMTclose, 0, dir);
     }
@@ -2373,6 +2527,7 @@ SpecialBombNuclear (Explosion *ptr)
 /* Added by x-bresse on 06.04.2000 */     if (c == (-curBombTime)*1/24) {
 /* Added by x-bresse on 06.04.2000 */     /*    SpreadExplosion (x, y, 0, BMTblastnow,0);*/
 /* Added by x-bresse on 06.04.2000 */     DelExplosion(ptr);
+    return XBFalse;
 /* Added by x-bresse on 06.04.2000 */     }
 /* Added by x-bresse on 06.04.2000 */     return XBTrue;
 /* Added by x-bresse on 06.04.2000 */ }
@@ -2424,6 +2579,7 @@ SpecialBombMine (Explosion *ptr)
       SpreadExplosion (x+1, y+1, 0, BMTblastnow,0); 
       SpreadExplosion (x+1, y-1, 0, BMTblastnow,0); 
     }
+    return XBFalse;
   }
 
   //ptr->type = BMTnormal;
@@ -2466,7 +2622,7 @@ SpecialBombMine (Explosion *ptr)
  /* Written by Amilhastre */     }
  /* Written by Amilhastre */     
  /* Written by Amilhastre */     ptr->type = BMTnormal;
- /* Written by Amilhastre */   return XBTrue;
+ /* Written by Amilhastre */   return XBFalse;
  /* Written by Amilhastre */   }
 
 /* bomb_row */
