@@ -73,7 +73,7 @@ extern Explosion *bombMaze[MAZE_W][MAZE_H];
 extern Explosion *explList;
 
 /* Tableau des bombes */
-static Explosion *laby_bomb[MAZE_W][MAZE_H];
+/*static Explosion *laby_bomb[MAZE_W][MAZE_H];*/
 /* Liste des bombes */
 static Explosion bombes_tmp[100];
 /* Tableau des cases dangereuses : temps avant explosion */
@@ -318,7 +318,7 @@ developpe_bombe_tmp (int range, int mazex, int mazey, int type, int count)
 	developpe_bombe_tmp (range/(ABS(i)+1), mazex, mazey+i, BMTnormal, count);
     }
   
-  if (type == BMTgrenade)
+  if (type == BMTgrenade){
     if (range == 1) {
       developpe_bombe_tmp(0, mazex-1, mazey-1, BMTblastnow, count);
       developpe_bombe_tmp(0, mazex+1, mazey-1, BMTblastnow, count);
@@ -331,6 +331,7 @@ developpe_bombe_tmp (int range, int mazex, int mazey, int type, int count)
 	}
       }
     }
+  }
 }
 
 /* Traduit une direction en dx et dy */
@@ -360,19 +361,25 @@ def_score_extra (BMPlayer *ps)
 {
   bonus_extra = BONUS_EXTRA;
   /* La tortue */
-  if ( specialExtraFunc ==SpecialExtraSlow)
-    if (!ps->iniextra_flags)
+  if ( specialExtraFunc ==SpecialExtraSlow){
+    if (!ps->iniextra_flags){
       bonus_extra = -BONUS_SIMPLE / 2;
-  else if(!ps->revextra_flags)
+    }
+    else{ if(!ps->revextra_flags)
       bonus_extra = -BONUS_SIMPLE / 2;
     else bonus_extra = 0; /* Si je suis dj lent, reprendre une tortue ne change rien */
+    }
+  }
   /* Le "bonus" invisible qui fait tourner */
-  if ( specialExtraFunc == SpecialExtraStunOthers)
+  if ( specialExtraFunc == SpecialExtraStunOthers){
     bonus_extra = -2 * BONUS_SIMPLE;
+  }
   /* La seringue */
   if ( specialExtraFunc == SpecialExtraJunkie &&
-      ps->junkie == 0) /* Si j'y ai dj touch, j'ai intrt  en reprendre... */
+      ps->junkie == 0){
+    /* Si j'y ai dj touch, j'ai intrt  en reprendre... */
     bonus_extra = -5 * BONUS_SIMPLE;
+  }
   /* La mort immdiate */
   if ( specialExtraFunc == SpecialExtraPoison)
     {
@@ -390,7 +397,7 @@ score_bonus (int x, int y)
   bonus = GetBlockExtra (x, y);
  
   if (bonus == BTBomb || bonus == BTRange)
-    bonus = BONUS_SIMPLE;
+    bonus = BONUS_SIMPLE+200;
   else if (bonus == BTSpecial||bonus == BTExtra )
     {
       if(specialExtraFunc !=SpecialExtraPoison){
@@ -484,7 +491,7 @@ score_tue (int x, int y, int portee_bombe)
 
       /* Si le joueur a une mauvaise mort (poseuse, lenteur, mini, non-poseuse,
 	 malfunction, teleport, ou *seringue*), je dois l'viter */
-      if ((x == jx) && (y == jy))
+      if ((x == jx) && (y == jy)){
 	if ((ps->illness == IllBomb) ||
 	    (ps->illness == IllSlow) ||
 	    (ps->illness == IllMini) ||
@@ -493,6 +500,7 @@ score_tue (int x, int y, int portee_bombe)
 	    (ps->illness == IllTeleport) ||
 	    (ps->junkie))
 	  tue -= BONUS_SIMPLE;
+      }
       /* sinon, j'essaie toujours de m'en approcher (s'il n'est pas mon partenaire) */
 	else tue += points;
     }
@@ -745,7 +753,7 @@ calc_laby_perf()
 	  p->dx = 0;
 	  p->dy = 0;
 	} else if (doBombClick == BombClickRandomdir) {
-	  p->dir = (BMDirection) (int)((float)rand()/(RAND_MAX+1))*MAX_DIR;//(GameRandomNumber1 (4) + 1);
+	  p->dir =(int)(((float)rand()/(RAND_MAX+1.0))*(4))+1;
 	  p->dx = 0;
 	  p->dy = 0;
 	} else if (doBombClick == BombClickRebound) {
@@ -779,7 +787,7 @@ calc_laby_perf()
 	  p->dx = 0;
 	  p->dy = 0;
 	} else if (doWallClick == BombClickRandomdir) {
-	  p->dir = (BMDirection)(int)((float)rand()/(RAND_MAX+1))*MAX_DIR; 
+	  p->dir = (BMDirection)(int)((float)rand()/(RAND_MAX+1.0))*MAX_DIR; 
 	  p->dx = 0;
 	  p->dy = 0;
 	} else if (doWallClick == BombClickRebound) {
@@ -824,7 +832,7 @@ calc_laby_perf()
 	    p->dx = 0;
 	    p->dy = 0;
 	  } else if (doPlayerClick == BombClickRandomdir) {
-	    p->dir = (BMDirection)(int)((float)rand()/(RAND_MAX+1))*MAX_DIR;// (GameRandomNumber1(4) + 1);
+	    p->dir = (int)(((float)rand()/(RAND_MAX+1.0))*(4))+1;
 	    p->dx = 0;
 	    p->dy = 0;
 	  } else if (doPlayerClick == BombClickRebound) {
@@ -848,7 +856,9 @@ calc_laby_perf()
 	p->dy -= dy * BLOCK_HEIGHT;
 	p->y += dy;
       }
-      
+      if(p->x<0)p->x=0;
+      if(p->y<0)p->y=0;
+
       /* On aime pas tres bien etre sur la trajectoire d'une bombe */
       laby_perf[p->x][p->y] = MIN(laby_perf[p->x][p->y], 50 * LONG_FUSE - 500 + temps * 4);
       
@@ -965,7 +975,7 @@ init_laby_perf_biscotte ()
   for (p = explList, i = 1; p != NULL; p = p->next, i++) {
     bombes_tmp[i] = *p;
     if (p->player)
-      if (p->player->number == num_bot)
+      if (p->player->id == num_bot)
 	bombes_tmp[i].count = 0;
   }
   
@@ -1053,7 +1063,7 @@ souhaite_poser_bombe (BMPlayer *ps)
     return XBFalse;
 
   /* Aucune raison particulire de poser... au hasard prs */
-  if (score == 0 &&(int)(((float)rand()/(RAND_MAX+1))*PROB_POSER)!=0 )// GameRandomNumber1 (PROB_POSER) != 0)
+  if (score == 0 && (int)(((float)rand()/(RAND_MAX+1.0))*(PROB_POSER))  != 0)
     return XBFalse;
   
   /* S'il reste au moins un autre joueur vivant, j'essaie de poser */
@@ -1084,15 +1094,15 @@ static void
 choix_direction (int *choix_dir, int *max_perf)
 {
   int dir=0,dir1=0;
-static int old_dir=0;
- static int init=1;
+  static int old_dir=0;
   *choix_dir = STOP;
   *max_perf = 0;
   for (dir = 0; dir < NB_DIR; dir++)
   {
     /* Ajout d'un petit bonus alatoire si la case est sre */
     if (perf_max[dir] > 10 * LONG_FUSE)
-      perf_max[dir] +=(int)((float)rand()/(RAND_MAX+1))*11; //GameRandomNumber1 (10);
+      perf_max[dir] +=(int)(((float)rand()/(RAND_MAX+1.0))*(10));//
+	//	GameRandomNumber1 (10);
     
     /* Je choisis la destination ayant la meilleure performance */
     // if(dir==old_dir && dir!=0) perf_max[dir]+=50;
@@ -1103,7 +1113,7 @@ static int old_dir=0;
       *choix_dir = dir;
     }
        else if ((perf_max[dir] == *max_perf) &&
-	  ((int)((float)rand()/(RAND_MAX+1))*(5) == dir))
+	  ((int)((float)rand()/(RAND_MAX+1.0))*(5) == dir))
 	  *choix_dir = dir;
   }
   for (dir = 0; dir < NB_DIR; dir++)
@@ -1278,7 +1288,7 @@ actions (BMPlayer *ps)
 	i=1;
 	dx=1;
 	conversion_direction (choix_dir, &dx, &dy);
-	if(!CheckBomb(x-olddx,y-olddy))i=0;
+      	if(!CheckBomb(x-olddx,y-olddy))i=0;
 	/* pas verifier si ma position est libre */
 	
 	if(dx-1!= 0 && dy!=0)
@@ -1314,6 +1324,7 @@ actions (BMPlayer *ps)
 	  vote_poser = XBTrue;
 	}
       }
+      //  if(
     }
   }
 
