@@ -1,7 +1,7 @@
 /*
  * cfg_game.c - storing and loading game setups
  *
- * $Id: cfg_game.c,v 1.4 2004/05/14 10:00:33 alfie Exp $
+ * $Id: cfg_game.c,v 1.22 2005/03/12 14:51:29 lodott Exp $
  *
  * Program XBLAST 
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
@@ -22,9 +22,9 @@
  */
 
 #include "cfg_game.h"
-
 #include "atom.h"
 #include "ini_file.h"
+#include "str_util.h"
 
 /*
  * local variables
@@ -34,6 +34,7 @@ static DBRoot *dbRemote;
 
 static CFGGame defaultGame = {
   {
+    0,         /* ifreclives  */
     1, 	       /* number of lives */
     9, 	       /* number of victories */
     20,        /* frames per seconed */
@@ -41,18 +42,25 @@ static CFGGame defaultGame = {
     XBTrue,    /* random level order */
     XBTrue,    /* random player positions */
     0,         /* level order */
-    15,        /* info wait time */
+    8,         /* info wait time */
     XBFalse,   /* record demo file */
     XBTM_None, /* no team game */
     XBTrue,    /* XBCC not rated */
+    XBFalse,   /* bot */
+    XBFalse,   /* beep */
+    XBFalse,   /* Music */
+    2,         /* number of rec lives (of valid if 1. XBTrue) */
+    1,         /* mask bytes for dgram */
   },
   {
-    0,
-    { ATOM_INVALID, ATOM_INVALID, ATOM_INVALID, ATOM_INVALID, ATOM_INVALID, ATOM_INVALID, }, 
-    { XBPC_None, XBPC_None, XBPC_None, XBPC_None, XBPC_None, XBPC_None, },
-    { XBPH_None, XBPH_None, XBPH_None, XBPH_None, XBPH_None, XBPH_None, },
-    { XBPT_None, XBPT_None, XBPT_None, XBPT_None, XBPT_None, XBPT_None, },
-    { COLOR_INVALID, COLOR_INVALID, COLOR_INVALID, COLOR_INVALID, COLOR_INVALID, COLOR_INVALID, },
+    0,         /* num of players */
+    { ATOM_INVALID, },     /* player name atom */
+    { XBPC_None, },        /* player control */
+    { XBPH_None, },        /* player host */
+    { XBPT_Invalid, },     /* player team */
+    { COLOR_INVALID, },    /* player team color */
+    { 0, },                /* player PID */
+    { 0, },                /* player id */
   },
   {
     "localhost", /* hostname */
@@ -61,8 +69,17 @@ static CFGGame defaultGame = {
     XBTrue,      /* browse LAN */
     XBTrue,      /* allow NAT for clients */
     XBTrue,      /* connect to central */
+    XBFalse,     /* beep if new client */
+    0,           /* Music */
     "unnamed",   /* name of the game */
-  }
+  },
+  {
+    6,           /* max hosts */
+    6,           /* max players */
+    6,           /* num local players */
+    1,           /* max mask bytes */
+    { 0,0,0 },   /* version */
+  },
 };
 /* conversion table for player control (alphanumeric sort by key) */
 static DBToInt convControlTable[XBPC_NUM+1] = {
@@ -105,10 +122,10 @@ static DBToInt convHostTable[XBPH_NUM+1] = {
 };
 /* conversion table for player teams (alphanumeric sort by key) */
 static DBToInt convTeamTable[XBPT_NUM+1] = {
-  { "Blue",   XBPT_Blue,  },
-  { "Green",  XBPT_Green, },
-  { "None",   XBPT_None,  },
-  { "Red",    XBPT_Red,   },
+  { "Blue",      XBPT_Blue,  },
+  { "Green",     XBPT_Green, },
+  { "None",      XBPT_None,  },
+  { "Red",       XBPT_Red,   },
   /* terminator */
   { NULL,     XBPT_NUM,   },
 };
@@ -153,6 +170,8 @@ StoreAnyGameSetup (DBRoot *db, XBAtom atom, const CFGGameSetup *game)
   assert (NULL != section);
   /* other stats */
   DB_CreateEntryInt  (section, atomLives,         game->numLives);
+  DB_CreateEntryInt  (section, atomRecLives,      game->recLives);
+  DB_CreateEntryBool (section, atomIfRecLives,    game->ifRecLives);
   DB_CreateEntryInt  (section, atomWins,          game->numWins);
   DB_CreateEntryInt  (section, atomFrameRate,     game->frameRate);
   DB_CreateEntryBool (section, atomAllLevels,     game->allLevels);
@@ -160,11 +179,12 @@ StoreAnyGameSetup (DBRoot *db, XBAtom atom, const CFGGameSetup *game)
   DB_CreateEntryBool (section, atomRandomPlayers, game->randomPlayers);
   DB_CreateEntryBool (section, atomRecordDemo,    game->recordDemo);
   DB_CreateEntryBool (section, atomRatedGame,     game->rated);
-  DB_CreateEntryBool (section, atomBot,    game->bot);
-
-  DB_CreateEntryInt  (section, atomInfoTime,     game->infoTime); // LRF
-  DB_CreateEntryInt  (section, atomLevelOrder,     game->levelOrder);
-
+  DB_CreateEntryBool (section, atomBot,           game->bot);
+  DB_CreateEntryBool (section, atomBeep,          game->beep);
+  DB_CreateEntryInt  (section, atomMusic,         game->Music);
+  DB_CreateEntryInt  (section, atomInfoTime,      game->infoTime); // LRF
+  DB_CreateEntryInt  (section, atomLevelOrder,    game->levelOrder);
+  DB_CreateEntryInt  (section, atomMaskBytes,     game->maskBytes);
   /* team mode */
   if (NULL != (key = DB_IntToString (convTeamModeTable, (int) game->teamMode) ) ) {
     DB_CreateEntryString (section, atomTeamMode, key);
@@ -183,7 +203,7 @@ StoreGameSetup (CFGType cfgType, XBAtom atom, const CFGGameSetup *game)
 } /* StoreLocalGameSetup */
 
 /*
- *
+ * store GamePlayers data from struct to any database
  */
 static void
 StoreAnyGamePlayers (DBRoot *db, XBAtom atom, const CFGGamePlayers *game)
@@ -229,7 +249,7 @@ StoreAnyGamePlayers (DBRoot *db, XBAtom atom, const CFGGamePlayers *game)
 } /* StoreAnyGamePlayers */
 
 /*
- *
+ * copy GamePlayers data from struct to a config database
  */
 void
 StoreGamePlayers (CFGType cfgType, XBAtom atom, const CFGGamePlayers *game)
@@ -244,7 +264,6 @@ static void
 StoreAnyGameHost (DBRoot *db, XBAtom atom, const CFGGameHost *host)
 {
   DBSection *section;
-  
   /* sanity check */
   assert (db != NULL);
   assert (ATOM_INVALID != atom);
@@ -263,6 +282,7 @@ StoreAnyGameHost (DBRoot *db, XBAtom atom, const CFGGameHost *host)
   DB_CreateEntryBool   (section, atomBrowseLan,    host->browseLan);
   DB_CreateEntryBool   (section, atomAllowNat,     host->allowNat);
   DB_CreateEntryBool   (section, atomCentral,      host->central);
+  DB_CreateEntryBool   (section, atomBeep,         host->beep);
   if (NULL != host->game) {
     DB_CreateEntryString (section, atomGame,      host->game);
   } else {
@@ -280,7 +300,84 @@ StoreGameHost (CFGType cfgType, XBAtom atom, const CFGGameHost *host)
 } /* StoreGameHost */
 
 /*
- *
+ * store version data in any database
+ */
+extern void
+StoreAnyGameVersion (DBRoot *db, XBAtom atom, const XBVersion *ver)
+{
+  DBSection *section;
+  /* sanity check */
+  assert (db != NULL);
+  assert (ATOM_INVALID != atom);
+  assert (NULL != ver);
+  /* create new game section */
+  section = DB_CreateSection (db, atom);
+  assert (NULL != section);
+  /* store version data */
+  DB_CreateEntryInt  (section, atomVersionMajor, (int) ver->major);
+  DB_CreateEntryInt  (section, atomVersionMinor, (int) ver->minor);
+  DB_CreateEntryInt  (section, atomVersionPatch, (int) ver->patch);
+} /* StoreAnyGameVersion */
+
+/*
+ * copy version data database directory
+ */
+void
+StoreGameVersion (CFGType cfgType, XBAtom atom, const XBVersion *ver)
+{
+  StoreAnyGameVersion (GetDB (cfgType), atom, ver);
+} /* StoreGameVersion */
+
+/*
+ * store constants in any database
+ */
+extern void
+StoreAnyGameConst (DBRoot *db, XBAtom atom, const CFGGameConst *con)
+{
+  DBSection *section;
+  /* sanity check */
+  assert (db != NULL);
+  assert (ATOM_INVALID != atom);
+  assert (NULL != con);
+  /* create new game section */
+  section = DB_CreateSection (db, atom);
+  assert (NULL != section);
+  /* store version data */
+  DB_CreateEntryInt  (section, atomMaxHosts, con->maxhosts);
+  DB_CreateEntryInt  (section, atomMaxPlayers, con->maxplayers);
+  DB_CreateEntryInt  (section, atomMaxLocals, con->maxlocals);
+  DB_CreateEntryInt  (section, atomMaxMaskBytes, con->maxbytes);
+  DB_CreateEntryInt  (section, atomVersionMajor, (int) con->version.major);
+  DB_CreateEntryInt  (section, atomVersionMinor, (int) con->version.minor);
+  DB_CreateEntryInt  (section, atomVersionPatch, (int) con->version.patch);
+} /* StoreAnyGameConst */
+
+/*
+ * store constants in config database
+ */
+void
+StoreGameConst (CFGType cfgType, XBAtom atom, const CFGGameConst *con)
+{
+  StoreAnyGameConst (GetDB (cfgType), atom, con);
+} /* StoreGameConst */
+
+/*
+ * store local constants in config database
+ */
+void
+StoreGameConstLocal (CFGType cfgType, XBAtom atom)
+{
+  CFGGameConst con;
+  con.maxhosts = MAX_HOSTS;
+  con.maxplayers = MAX_PLAYER;
+  con.maxlocals = NUM_LOCAL_PLAYER;
+  con.maxbytes = MAX_MASK_BYTES;
+  memcpy(&con.version, &Ver_Local, sizeof(XBVersion));
+  StoreAnyGameConst (GetDB (cfgType), atom, &con);
+} /* StoreGameConstLocal */
+
+/*
+ * retrieve game setup from any database
  */
 XBBool
 RetrieveAnyGameSetup (const DBRoot *db, XBAtom atom, CFGGameSetup *game)
@@ -298,6 +395,8 @@ RetrieveAnyGameSetup (const DBRoot *db, XBAtom atom, CFGGameSetup *game)
   }
   /* copy values */
   (void) DB_GetEntryInt  (section, atomLives,         &game->numLives);
+  (void) DB_GetEntryInt  (section, atomRecLives,      &game->recLives);
+  (void) DB_GetEntryBool (section, atomIfRecLives,    &game->ifRecLives);
   (void) DB_GetEntryInt  (section, atomWins,          &game->numWins);
   (void) DB_GetEntryInt  (section, atomFrameRate,     &game->frameRate);
   (void) DB_GetEntryBool (section, atomAllLevels,     &game->allLevels);
@@ -305,12 +404,14 @@ RetrieveAnyGameSetup (const DBRoot *db, XBAtom atom, CFGGameSetup *game)
   (void) DB_GetEntryBool (section, atomRandomPlayers, &game->randomPlayers);
   (void) DB_GetEntryBool (section, atomRecordDemo,    &game->recordDemo);
   (void) DB_GetEntryBool (section, atomRatedGame,     &game->rated);
-  (void) DB_GetEntryInt (section, atomInfoTime,      &game->infoTime); // LRF
-  (void) DB_GetEntryInt (section, atomLevelOrder,    &game->levelOrder);
-  (void) DB_GetEntryBool (section, atomBot,    &game->bot);
-
+  (void) DB_GetEntryInt  (section, atomInfoTime,      &game->infoTime); // LRF
+  (void) DB_GetEntryInt  (section, atomLevelOrder,    &game->levelOrder);
+  (void) DB_GetEntryBool (section, atomBot,           &game->bot);
+  (void) DB_GetEntryBool (section, atomBeep,          &game->beep);
+  (void) DB_GetEntryInt  (section, atomMusic,         (int *) &game->Music);
+  (void) DB_GetEntryInt  (section, atomMaskBytes,     &game->maskBytes);
   /* team mode */
-  if (! DB_ConvertEntryInt (section, atomTeamMode, (int *) &game->teamMode, convTeamModeTable) ) {
+  if (DCR_Okay != DB_ConvertEntryInt (section, atomTeamMode, (int *) &game->teamMode, convTeamModeTable) ) {
     game->teamMode = XBTM_None;
   }
   /* that's all */
@@ -318,16 +419,37 @@ RetrieveAnyGameSetup (const DBRoot *db, XBAtom atom, CFGGameSetup *game)
 } /* RetrieveGameConfig */
 
 /*
- *
+ * retrieve game setup from config database
  */
 XBBool
 RetrieveGameSetup (CFGType cfgType, XBAtom atom, CFGGameSetup *game)
 {
   return RetrieveAnyGameSetup (GetDB (cfgType), atom, game);
-} /* RetrieveLocalGameSetup */
+} /* RetrieveGameSetup */
 
 /*
- *
+ * fill default player data, call only once when accessing game players
+ */
+static void
+DefaultPlayerData()
+{
+  static int init = 0;
+  int p;
+  if (init == 0) {
+    for (p = 1; p < MAX_PLAYER; p++) {
+      defaultGame.players.player[p] = defaultGame.players.player[0];
+      defaultGame.players.control[p] = defaultGame.players.control[0];
+      defaultGame.players.host[p] = defaultGame.players.host[0];
+      defaultGame.players.team[p] = defaultGame.players.team[0];
+      defaultGame.players.PID[p] = defaultGame.players.PID[0];
+      defaultGame.players.playerID[p] = defaultGame.players.playerID[0];
+    }
+    init = 1;
+  }
+} /* DefaultPlayerData */
+
+/*
+ * copy GamePlayers data from any database to struct
  */
 static XBBool
 RetrieveAnyGamePlayers (const DBRoot *db, XBAtom atom, CFGGamePlayers *game)
@@ -335,10 +457,10 @@ RetrieveAnyGamePlayers (const DBRoot *db, XBAtom atom, CFGGamePlayers *game)
   int i;
   const DBSection *section;
   const char *s;
-
   assert (NULL != db);
   assert (NULL != game);
   /* set defaults */
+  DefaultPlayerData();
   memcpy (game, &defaultGame.players, sizeof (CFGGamePlayers));
   /* find according db section */
   section = DB_GetSection (db, atom);
@@ -349,6 +471,10 @@ RetrieveAnyGamePlayers (const DBRoot *db, XBAtom atom, CFGGamePlayers *game)
   if (! DB_GetEntryInt (section, atomNumPlayers, &game->num) ) {
     game->num = 0;
   }
+  /* check player max */
+  if (game->num > MAX_PLAYER) {
+    return XBFalse;
+  }
   /* get players */
   for (i = 0; i < game->num; i ++) {
     if (DB_GetEntryString (section, atomArrayPlayer0[i], &s) ) {
@@ -356,14 +482,14 @@ RetrieveAnyGamePlayers (const DBRoot *db, XBAtom atom, CFGGamePlayers *game)
     } else {
       game->player[i] = ATOM_INVALID;
     }
-    if (! DB_ConvertEntryInt (section, atomArrayControl0[i], (int *) &game->control[i], convControlTable) ) {
+    if (DCR_Okay != DB_ConvertEntryInt (section, atomArrayControl0[i], (int *) &game->control[i], convControlTable) ) {
       game->control[i] = XBPC_None;
     }
-    if (! DB_ConvertEntryInt (section, atomArrayHost0[i], (int *) &game->host[i], convHostTable) ) {
+    if (DCR_Okay != DB_ConvertEntryInt (section, atomArrayHost0[i], (int *) &game->host[i], convHostTable) ) {
       game->host[i] = XBPH_None;
     }
-    if (! DB_ConvertEntryInt (section, atomArrayTeam0[i], (int *) &game->team[i], convTeamTable) ) {
-      game->team[i] = XBPT_None;
+    if (DCR_Okay != DB_ConvertEntryInt (section, atomArrayTeam0[i], (int *) &game->team[i], convTeamTable) ) {
+      game->team[i] = XBPT_Invalid;
     }
     switch (game->team[i]) {
     case XBPT_Red:   game->teamColor[i] = COLOR_RED;   	 break;
@@ -404,13 +530,14 @@ RetrieveAnyGameHost (const DBRoot *db, XBAtom atom, CFGGameHost *game)
   memcpy (game, &defaultGame.host, sizeof (CFGGameHost) );
   if (DB_GetEntryString (section, atomHost, &s) ) {
     game->name = s;
-  } 
+  }
   (void) DB_GetEntryInt    (section, atomPort,         &game->port);
   (void) DB_GetEntryBool   (section, atomFixedUdpPort, &game->fixedUdpPort);
   (void) DB_GetEntryBool   (section, atomBrowseLan,    &game->browseLan);
   (void) DB_GetEntryBool   (section, atomAllowNat,     &game->allowNat);
   (void) DB_GetEntryBool   (section, atomCentral,      &game->central);
   (void) DB_GetEntryString (section, atomGame,         &game->game);
+  (void) DB_GetEntryBool   (section, atomBeep,         &game->beep);
   return XBTrue;
 } /* RetrieveGameHost */
 
@@ -424,7 +551,93 @@ RetrieveGameHost (CFGType cfgType, XBAtom atom, CFGGameHost *game)
 } /* RetrieveGameHost */
 
 /*
- * save game config at once 
+ * get version data from any database
+ */
+extern XBBool
+RetrieveAnyGameVersion (const DBRoot *db, XBAtom atom, XBVersion *ver)
+{
+  const DBSection *section;
+  int maj, min, pat;
+  assert(NULL != ver);
+  /* find according db section */
+  section = DB_GetSection (db, atom);
+  if (NULL == section) {
+    return XBFalse;
+  }
+  Version_Clear(ver);
+  /* now extract version data */
+  if (! DB_GetEntryInt (section, atomVersionMajor, &maj) ) {
+    return XBFalse;
+  }
+  if (! DB_GetEntryInt (section, atomVersionMinor, &min) ) {
+    return XBFalse;
+  }
+  if (! DB_GetEntryInt (section, atomVersionPatch, &pat) ) {
+    return XBFalse;
+  }
+  ver->major = maj & 0xFF;
+  ver->minor = min & 0xFF;
+  ver->patch = pat & 0xFF;
+  return XBTrue;
+} /* RetrieveAnyGameVersion */
+
+/*
+ * get version data from config database
+ */
+XBBool
+RetrieveGameVersion (CFGType cfgType, XBAtom atom, XBVersion *ver)
+{
+  return RetrieveAnyGameVersion (GetDB (cfgType), atom, ver);
+} /* RetrieveGameVersion */
+
+/*
+ * get game constants any database
+ */
+extern XBBool
+RetrieveAnyGameConst (const DBRoot *db, XBAtom atom, CFGGameConst *con)
+{
+  const DBSection *section;
+  int maj, min, pat;
+  assert(NULL != con);
+  /* set default */
+  memcpy(con, &defaultGame.constants, sizeof(CFGGameConst));
+  /* find according db section */
+  section = DB_GetSection (db, atom);
+  if (NULL == section) {
+    return XBFalse;
+  }
+  /* extract constants */
+  (void) DB_GetEntryInt (section, atomMaxHosts, &con->maxhosts);
+  (void) DB_GetEntryInt (section, atomMaxPlayers, &con->maxplayers);
+  (void) DB_GetEntryInt (section, atomMaxLocals, &con->maxlocals);
+  (void) DB_GetEntryInt (section, atomMaxMaskBytes, &con->maxbytes);
+  /* now extract version data */
+  if (! DB_GetEntryInt (section, atomVersionMajor, &maj) ) {
+    return XBTrue;
+  }
+  if (! DB_GetEntryInt (section, atomVersionMinor, &min) ) {
+    return XBTrue;
+  }
+  if (! DB_GetEntryInt (section, atomVersionPatch, &pat) ) {
+    return XBTrue;
+  }
+  con->version.major = maj & 0xFF;
+  con->version.minor = min & 0xFF;
+  con->version.patch = pat & 0xFF;
+  return XBTrue;
+} /* RetrieveAnyGameConst */
+
+/*
+ * get game constants from config database
+ */
+XBBool
+RetrieveGameConst (CFGType cfgType, XBAtom atom, CFGGameConst *con)
+{
+  return RetrieveAnyGameConst (GetDB (cfgType), atom, con);
+} /* RetrieveGameConst */
+
+/*
+ * save game config at once
  */
 void
 StoreGame (CFGType cfgType, XBAtom atom, const CFGGame *cfg)
@@ -434,12 +647,13 @@ StoreGame (CFGType cfgType, XBAtom atom, const CFGGame *cfg)
   StoreGameSetup   (cfgType, atom, &cfg->setup);
   StoreGamePlayers (cfgType, atom, &cfg->players);
   StoreGameHost    (cfgType, atom, &cfg->host);
+  StoreGameConst   (cfgType, atom, &cfg->constants);
 } /* StoreGame */
 
 /*
- * load game config at once
+ * retrieve game config at once
  */
-XBBool 
+XBBool
 RetrieveGame (CFGType cfgType, XBAtom atom, CFGGame *cfg)
 {
   XBBool result = XBTrue;
@@ -455,12 +669,12 @@ RetrieveGame (CFGType cfgType, XBAtom atom, CFGGame *cfg)
   if (! RetrieveGameHost (cfgType, atom, &cfg->host)) {
     result = XBFalse;
   }
+  (void) RetrieveGameConst (cfgType, atom, &cfg->constants);
   return result;
 } /* RetrieveGame */
 
-
 /*
- * 
+ * load game config from file
  */
 void
 LoadGameConfig (void)
@@ -468,13 +682,16 @@ LoadGameConfig (void)
   /* remote database */
   dbRemote = DB_Create (DT_Config, atomRemoteGame);
   assert (dbRemote != NULL);
-  /* local daatabase */
+  /* local database */
   dbLocal = DB_Create (DT_Config, atomGame);
   assert (dbLocal != NULL);
+  Dbg_Config("loading local game config\n");
   if (DB_Load (dbLocal) ) {
     return;
   }
+  Dbg_Config("failed to load local game config, setting defaults\n");
   /* set default values */
+  DefaultPlayerData();
   StoreGame (CT_Local, atomLocal,  &defaultGame);
   StoreGame (CT_Local, atomServer, &defaultGame);
   StoreGame (CT_Local, atomClient, &defaultGame);
@@ -483,25 +700,27 @@ LoadGameConfig (void)
 } /* LoadGameConfig */
 
 /*
- *
+ * save game config to file
  */
 void
 SaveGameConfig (void)
 {
   assert (dbLocal != NULL);
   if (DB_Changed (dbLocal) ) {
+    Dbg_Config("saving local game config\n");
     DB_Store (dbLocal);
   }
 #ifdef DEBUG
   assert (dbRemote != NULL);
   if (DB_Changed (dbRemote) ) {
+    Dbg_Config("saving remote game config\n");
     DB_Store (dbRemote);
   }
 #endif
 } /* SaveGameConfig */
 
 /*
- *
+ * remove game configs
  */
 void
 FinishGameConfig (void)
@@ -514,6 +733,7 @@ FinishGameConfig (void)
     DB_Delete (dbRemote);
     dbRemote = NULL;
   }
+  Dbg_Config("game config cleared!\n");
 } /* FinishGameConfig */
 
 /*
@@ -596,7 +816,7 @@ GetHostName (CFGType cfgType, XBAtom atom)
 } /* GetRemoteHostName */
 
  /*
-  *
+  * push a single ip into ip history database
   */
 void
 StoreIpHistory (CFGGameHost *host, XBAtom atom)
@@ -607,61 +827,57 @@ StoreIpHistory (CFGGameHost *host, XBAtom atom)
   /* sanity check */
   assert (ATOM_INVALID != atom);
   assert (NULL != host);
-  section = (DBSection *)DB_GetSection ((DBRoot *)GetDB (CT_Local), atom);
-  /* create new game section */
+  section = (DBSection *) DB_GetSection (GetDB (CT_Local), atom);
+  /* get old history data */
   assert (NULL != section);
-  RetrieveIpHistory(hist,atom);
-   for(i=9;i>0;i--){
-    hist[i].name=malloc(strlen( hist[i-1].name));
-    strcpy((char*)hist[i].name, hist[i-1].name);
-    hist[i].game=malloc(strlen( hist[i-1].game));
-    strcpy((char*)hist[i].game, hist[i-1].game);
-  hist[i].port=hist[i-1].port;
+  RetrieveIpHistory(hist, atom);
+  /* push back server data */
+  for (i=9; i>0; i--) {
+    hist[i].name = hist[i-1].name;
+    hist[i].port = hist[i-1].port;
   }
-  hist[0].port=host->port; 
-    hist[0].game=malloc(strlen( host->game));
-    strcpy((char*)hist[0].game, host->game);
-    hist[0].name=malloc(strlen( host->name));
-    strcpy((char*)hist[0].name, host->name);
-
-  /* other stats */
-
-  for(i=0;i<10;i++){
-   DB_CreateEntryInt    (section, 
-			atomPorthis[i],
-			hist[i].port);
-  if (NULL != hist[i].name) {
-    DB_CreateEntryString (section, atomGamehis[i],      hist[i].name);
-  } else {
-    DB_DeleteEntry (section, atomGamehis[i]);
+  /* store latest host data */
+  hist[0].port = host->port;
+  hist[0].name = host->name;
+  /* store back to database, backwards (important for pointer validity!) */
+  for (i=9; i>=0; i--) {
+    if (NULL != hist[i].name) {
+      DB_CreateEntryString (section, atomGamehis[i], hist[i].name);
+      DB_CreateEntryInt    (section, atomPorthis[i], hist[i].port);
+    } else {
+      DB_DeleteEntry (section, atomGamehis[i]);
+      DB_DeleteEntry (section, atomPorthis[i]);
+    }
   }
-  }
-} /* StoreGameHost */
+  RetrieveIpHistory(hist, atom);
+} /* StoreIpHistory */
 
-XBBool RetrieveIpHistory
-(CFGGameHost game[10], XBAtom atom)
+/*
+ * retrieve all ip history data
+ */
+XBBool
+RetrieveIpHistory (CFGGameHost host[10], XBAtom atom)
 {
-const DBSection *section;
-  const char *s;
- int i=0;
-section = DB_GetSection (GetDB (CT_Local), atom);
-
+  const DBSection *section;
+  int i = 0;
+  section = DB_GetSection (GetDB (CT_Local), atom);
+  /* check if there is section with ip history data */
   if (NULL == section) {
     return XBFalse;
   }
-  for (i=0; i<10;i++){
-  /* get host data */
-  memcpy (&(game[i]), &defaultGame.host, sizeof (CFGGameHost) );
-
-   if (DB_GetEntryString (section, atomHost, &s) ) {
-     game[i].name=  malloc(strlen(s));
-      sprintf((char*)game[i].name,"%s",s);
-  } 
-  (void) DB_GetEntryInt    (section, atomPorthis[i],         &game[i].port);
-  (void) DB_GetEntryString (section, atomGamehis[i],         &game[i].name);
-}
+  /* retrieve data */
+  for (i=0; i<10; i++) {
+    /* first default data for entry */
+    host[i].name = NULL;
+    host[i].game = NULL;
+    host[i].port = 0;
+    /* now update from database */
+    (void) DB_GetEntryString (section, atomGamehis[i], &host[i].name);
+    (void) DB_GetEntryInt    (section, atomPorthis[i], &host[i].port);
+  }
   return XBTrue;
-}
+} /* RetrieveIpHistory */
+
 /*
  * end of file cfg_game.c
  */
