@@ -1,9 +1,9 @@
 /*
  * cfg_level.c - managing level data
  *
- * $Id: cfg_level.c,v 1.4 2004/05/14 10:00:33 alfie Exp $
+ * $Id: cfg_level.c,v 1.19 2005/03/12 17:37:31 iskywalker Exp $
  *
- * Program XBLAST 
+ * Program XBLAST
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -12,7 +12,7 @@
  * any later version
  *
  * This program is distributed in the hope that it will be entertaining,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
  * Public License for more details.
  *
@@ -22,11 +22,12 @@
  */
 
 #include "cfg_level.h"
-
+#include "level.h"
 #include "atom.h"
 #include "str_util.h"
 #include "util.h"
 #include "random.h"
+#include "version.h"
 
 /*
  * level info structure
@@ -71,7 +72,7 @@ typedef struct {
 static DBRoot        *dbLevel         = NULL;
 static LevelTable    *sortedTable     = NULL;
 static LevelTable    *shuffledTable   = NULL;
-static LevelTable    *timeTable       = NULL; // Better level randomness
+static LevelTable    *timeTable       = NULL; /* Better level randomness */
 static const char    *gameModeString  = "R23456STDL";
 static unsigned       gameMode        = 0;
 static XBBool         randomOrder     = XBFalse;
@@ -81,8 +82,12 @@ static int            indexSorted     = 0;
 static int            indexShuffled   = 0;
 static int            indexTime       = 0;
 
-static DBRoot        *levelRemote;
-static DBRoot        *levelLocal;
+static DBRoot        *levelRemote     = NULL;
+static DBRoot        *levelLocal      = NULL;
+
+/******************
+ * local routines *
+ ******************/
 
 /*
  * additional info for new levels files
@@ -97,85 +102,37 @@ InsertLevel (DBSection *section)
 } /* InsertLevel */
 
 /*
- *
+ * get game mode string for current level, given section
  */
-void
-LoadLevelConfig (void)
+static unsigned
+GetLevelGameMode (XBAtom atom)
 {
-#ifdef DEBUG
-  Dbg_StartClock ();
-#endif
-  /* create and load level database */
-  dbLevel = DB_Create (DT_Config, atomLevel);
-  assert (dbLevel != NULL);
-  if (DB_LoadDir (dbLevel, GAME_DATADIR"/level", "xal", DT_Level, atomTime, atomInfo, InsertLevel)) {
-    DB_Store (dbLevel);
+  const DBSection *section;
+  const char      *s;
+  int              i;
+  unsigned         gameMode = 0;
+  if (NULL == (section = DB_GetSection (dbLevel, atom) ) ) {
+    return 0;
   }
-#ifdef DEBUG
-  Dbg_Out ("init levels: %lu msec\n", Dbg_FinishClock ());
-#endif
-} /* LoadLevelConfig */
+  if (! DB_GetEntryString (section, atomGameMode, &s) ) {
+    return 0;
+  }
+  for (i = 0; s[i] != 0 && gameModeString[i] != 0; i ++) {
+    if (s[i] == gameModeString[i]) {
+      gameMode |= (1 << i);
+    }
+  }
+  return gameMode;
+} /* GetLevelGameMode */
 
 /*
- *
- */
-void
-SaveLevelConfig (void)
-{
-  assert (dbLevel != NULL);
-  if (DB_Changed (dbLevel) ) {
-    DB_Store (dbLevel);
-  }
-} /* SaveLevelConfig */
-
-/*
- *
- */
-void
-FinishLevelConfig (void)
-{
-  DB_Delete (dbLevel);
-  if (NULL != sortedTable) {
-    free (sortedTable);
-  }
-  if (NULL != shuffledTable) {
-    free (shuffledTable);
-  }
-  if (NULL != levelRemote) {
-    DB_Delete (levelRemote);
-  }
-  if (NULL != levelLocal) {
-    DB_Delete (levelLocal);
-  }
-} /* FinishLevelConfig */
-
-/*
- * number of levels in database
- */
-int
-GetNumLevels (void)
-{
-  return DB_NumSections (dbLevel);
-} /* GetNumLevels */
-
-/*
- * get atom for level
- */
-XBAtom
-GetLevelAtom (int index)
-{
-  return DB_IndexSection (dbLevel, index);
-} /* GetLevelAtom */
-
-/*
- * get string of level
+ * get a level/atom entry from current level
  */
 static const char *
 GetLevelStringByAtom (XBAtom level, XBAtom atom)
 {
   const char      *s;
   const DBSection *section;
-  
   assert (dbLevel != NULL);
   section = DB_GetSection (dbLevel, level);
   if (NULL == section) {
@@ -188,60 +145,12 @@ GetLevelStringByAtom (XBAtom level, XBAtom atom)
 } /* GetLevelString */
 
 /*
- * get name of level
- */
-const char *
-GetLevelNameByAtom (XBAtom level)
-{
-  return GetLevelStringByAtom (level, atomName);
-} /* GetLevelName */
-
-/*
- * is level selected
- */
-XBBool
-GetLevelSelected (XBAtom level)
-{
-  XBBool selected;
-
-  const DBSection *section;
-  
-  assert (dbLevel != NULL);
-  section = DB_GetSection (dbLevel, level);
-  if (NULL == section) {
-    return XBFalse;
-  }
-  if (! DB_GetEntryBool (section, atomSelect, &selected) ) {
-    return XBTrue;
-  }
-  return selected;
-} /* GetLevelSelected */
-
-/*
- * Update Level in Database
- */
-void
-StoreLevelSelected (XBAtom atom, XBBool select)
-{
-  DBSection *section;
-
-  assert (NULL != dbLevel);
-
-  section = DB_CreateSection (dbLevel, atom);
-  assert (section != NULL);
-  /* we only change data not stored in file */
-  Dbg_Out("Level select %s = %d\n",GetLevelNameByAtom(atom),select);
-  (void) DB_CreateEntryBool (section, atomSelect,  select);
-} /* StoreLevelSelection */
-
-/*
- * get shuffle index for level
+ * get shuffle index for current level, section atom
  */
 static XBBool
 GetLevelShuffle (XBAtom level, int *pShuffle)
 {
   const DBSection *section;
-  
   /* sanity checks */
   assert (pShuffle != NULL);
   assert (dbLevel  != NULL);
@@ -257,15 +166,13 @@ GetLevelShuffle (XBAtom level, int *pShuffle)
 } /* GetLevelShuffle */
 
 /*
- * Update Level in Database
+ * store shuffle index in current level, section atom
  */
 static void
 StoreLevelShuffle (XBAtom atom, int shuffle)
 {
   DBSection *section;
-
   assert (NULL != dbLevel);
-
   section = DB_CreateSection (dbLevel, atom);
   assert (section != NULL);
   /* we only change data not stored in file */
@@ -273,13 +180,12 @@ StoreLevelShuffle (XBAtom atom, int shuffle)
 } /* StoreLevelShuffle */
 
 /*
- * get shuffle index for level
+ * get time from current level, section level
  */
 static XBBool
 GetLevelTime (XBAtom level, time_t *lpTime)
 {
   const DBSection *section;
-  
   /* sanity checks */
   assert (lpTime != NULL);
   assert (dbLevel  != NULL);
@@ -295,15 +201,13 @@ GetLevelTime (XBAtom level, time_t *lpTime)
 } /* GetLevelTime */
 
 /*
- * Update Level in Database
+ * store time in current level, section level
  */
 static void
 StoreLevelTime (XBAtom atom, int lpTime)
 {
   DBSection *section;
-
   assert (NULL != dbLevel);
-
   section = DB_CreateSection (dbLevel, atom);
   assert (section != NULL);
   /* we only change data not stored in file */
@@ -311,52 +215,25 @@ StoreLevelTime (XBAtom atom, int lpTime)
 } /* StoreLevelTime */
 
 /*
- *
- */
-unsigned 
-GetLevelGameMode (XBAtom atom)
-{
-  const DBSection *section;
-  const char      *s;
-  int              i;
-  unsigned         gameMode = 0;
-
-  if (NULL == (section = DB_GetSection (dbLevel, atom) ) ) {
-    return 0;
-  }
-  if (! DB_GetEntryString (section, atomGameMode, &s) ) {
-    return 0;
-  }
-  for (i = 0; s[i] != 0 && gameModeString[i] != 0; i ++) {
-    if (s[i] == gameModeString[i]) {
-      gameMode |= (1 << i);
-    } 
-  }
-  return gameMode;
-} /* GetLevelGameMode */
-
-/*
- * set last level for specific level order
+ * set flag in current level, given section/entry
  */
 static void
 SetLevelWasLast (XBAtom level, XBAtom entry, XBBool flag)
 {
   DBSection *section;
-
   section = DB_CreateSection (dbLevel, level);
   assert (section != NULL);
   (void) DB_CreateEntryBool (section, entry, flag);
 } /* SetWasLastLevel */
 
 /*
- * get last level for specific level order
+ * get flag in current level, given section/entry
  */
 static XBBool
 GetLevelWasLast (XBAtom level, XBAtom entry)
 {
   const DBSection *section;
   XBBool result;
-
   section = DB_GetSection (dbLevel, level);
   assert (section != NULL);
   if (! DB_GetEntryBool (section, entry, &result) ) {
@@ -366,7 +243,505 @@ GetLevelWasLast (XBAtom level, XBAtom entry)
 } /* GetWasLastLevel */
 
 /*
- * load a level from file
+ * compare level names by name
+ */
+static int
+CompareSorted (const void *a, const void *b)
+{
+  return strcmp ( ( (LevelTable *) a)->key.name, ( (LevelTable *) b)->key.name);
+} /* CompareSorted */
+
+/*
+ * compare levels by shuffle value
+ */
+static int
+CompareShuffle (const void *a, const void *b)
+{
+  return ( (LevelTable *) a)->key.value - ( (LevelTable *) b)->key.value;
+} /* CompareShuffle */
+
+/*
+ * compare levels by time
+ */
+static int
+CompareTime (const void *a, const void *b)
+{
+  return ( (LevelTable *) a)->key.lastplayed - ( (LevelTable *) b)->key.lastplayed;
+} /* CompareTime */
+
+/*
+ * return level index after sorting by name
+ */
+static int
+GetLastIndexSorted (void)
+{
+  int              i,numLevels;
+  XBAtom           atom;
+  /* allocate table if necessary */
+  numLevels = GetNumLevels ();
+  if (NULL == sortedTable) {
+    sortedTable = calloc (numLevels, sizeof (LevelTable) );
+    assert (sortedTable != NULL);
+  }
+  /* enter level names */
+  for (i = 0; i < numLevels; i ++) {
+    atom = GetLevelAtom (i);
+    sortedTable[i].level    = atom;
+    sortedTable[i].key.name = GetLevelNameByAtom (atom);
+  }
+  /* sort the table */
+  qsort (sortedTable, numLevels,  sizeof (LevelTable), CompareSorted);
+  /* find first level marked as sorted*/
+  for (i = 0; i < numLevels; i ++) {
+    if (GetLevelWasLast (sortedTable[i].level, atomLevelSorted) ) {
+      Dbg_Level ("last level (sorted)   = %s\n", GetLevelNameByAtom (sortedTable[i].level) );
+      return i;
+    }
+  }
+  return numLevels;
+} /* GetLastIndexSorted */
+
+/*
+ * return level index after sorting by value
+ */
+static int
+GetLastIndexShuffled (void)
+{
+  int i, miss, max, swap, save, numLevels;
+  /* allocate shuffledTable if necessary */
+  numLevels = GetNumLevels ();
+  if (NULL == shuffledTable) {
+    shuffledTable = calloc (numLevels, sizeof (LevelTable) );
+    assert (shuffledTable != NULL);
+  }
+  /* build table, count missing values, get next value */
+  miss = 0;
+  max = 0;
+  for (i = 0; i < numLevels; i ++) {
+    shuffledTable[i].level = GetLevelAtom (i);
+    assert (ATOM_INVALID != shuffledTable[i].level);
+    if (!GetLevelShuffle (shuffledTable[i].level, &shuffledTable[i].key.value) ) {
+      shuffledTable[i].key.value = -1;
+      miss++;
+    }
+    if (shuffledTable[i].key.value > max) {
+      max = shuffledTable[i].key.value;
+    }
+  }
+  Dbg_Level("%u missing shuffle values, %u is current max\n", miss, max);
+  /* assign new values to missing */
+  for (i = 0; i < numLevels; i ++) {
+    if (shuffledTable[i].key.value < 0) {
+      shuffledTable[i].key.value = -max;
+      max++;
+    }
+  }
+  Dbg_Level("update max value\n", max);
+  /* shuffle missing values */
+  for (i=0; i<numLevels; i++) {
+    if (shuffledTable[i].key.value < 0) {
+      swap = OtherRandomNumber(numLevels);
+      save = -shuffledTable[i].key.value;
+      shuffledTable[i].key.value = abs(shuffledTable[swap].key.value);
+      shuffledTable[swap].key.value = save;
+    }
+  }
+  /* update shuffle values in database */
+  for (i = 0; i < numLevels; i ++) {
+    StoreLevelShuffle(shuffledTable[i].level, shuffledTable[i].key.value);
+  }
+  /* sort the table by value */
+  qsort (shuffledTable, numLevels,  sizeof (LevelTable), CompareShuffle);
+  /* get last level played from database */
+  for (i = 0; i < numLevels; i ++) {
+    if (GetLevelWasLast (shuffledTable[i].level, atomLevelShuffled) ) {
+      Dbg_Level ("last level (shuffled) = %s\n", GetLevelNameByAtom (shuffledTable[i].level) );
+      return i;
+    }
+  }
+  return numLevels;
+} /* GetLastIndexShuffled */
+
+/*
+ * return last level after sorting by time
+ */
+static int
+GetLastIndexTime (void)
+{
+  int i, numLevels;
+  /* allocate timeTable if necessary */
+  numLevels = GetNumLevels ();
+  if (NULL == timeTable) {
+    timeTable = calloc (numLevels, sizeof (LevelTable) );
+    assert (timeTable != NULL);
+  }
+  /* build table */
+  for (i = 0; i < numLevels; i ++) {
+    timeTable[i].level = GetLevelAtom (i);
+    assert (ATOM_INVALID != timeTable[i].level);
+    if (!GetLevelTime (timeTable[i].level, &timeTable[i].key.lastplayed) ) {
+      timeTable[i].key.lastplayed = 0;
+      StoreLevelTime(timeTable[i].level, timeTable[i].key.lastplayed);
+    }
+  }
+  /* sort table */
+  qsort (timeTable, numLevels,  sizeof (LevelTable), CompareTime);
+#ifdef DEBUG_LEVELTIME
+  for (i=0; i<numLevels; i++) {
+    Dbg_Out("Lastplayed time of level %s = %s\n",GetLevelNameByAtom(timeTable[i].level),DateString(timeTable[i].key.lastplayed));
+  }
+#endif
+  return numLevels;
+} /* GetLastIndexTime */
+
+/*
+ * reset sorted level
+ */
+static void
+ResortLevels (void)
+{
+  indexSorted = 0;
+} /* ResortLevels */
+
+/*
+ * reset level time
+ */
+static void
+ReTimeLevels (void)
+{
+  indexTime = 0;
+} /* ReTimeLevels */
+
+/*
+ * recreate shuffle table
+ */
+static void
+ReshuffleLevels (void)
+{
+  int i,j,k, numLevels;
+
+  Dbg_Level ("reshuffling levels\n");
+  numLevels = GetNumLevels ();
+  for (i=0; i< numLevels; i++) {
+    shuffledTable[i].key.value=i;
+  }
+  for (i = numLevels-1; i > 0 ; i--) {
+    j=OtherRandomNumber(i+1);
+    k=shuffledTable[i].key.value;
+    shuffledTable[i].key.value=shuffledTable[j].key.value;
+    shuffledTable[j].key.value=k;
+  }
+  for (i=0; i< numLevels; i++) {
+    StoreLevelShuffle(shuffledTable[i].level, shuffledTable[i].key.value);
+  }
+  SaveLevelConfig();
+  qsort (shuffledTable, numLevels,  sizeof (LevelTable), CompareShuffle);
+  indexShuffled = 0;
+} /* ReshuffleLevels */
+
+/*
+ * get next level from table
+ */
+static XBAtom
+GetNextLevelFromTable (const LevelTable *table, int *pIndex, XBAtom entry, void (*pFunc) (void))
+{
+  int    numLevels;
+  XBAtom atom;
+
+  assert (pIndex != NULL);
+  assert (pFunc  != NULL);
+  numLevels = GetNumLevels ();
+  /* unmark last level */
+  if (*pIndex < numLevels) {
+    SetLevelWasLast (table[*pIndex].level, entry, XBFalse);
+  }
+  /* find next level to play */
+  do {
+    *pIndex = *pIndex + 1;
+    if (*pIndex >= numLevels) {
+      (*pFunc) ();
+      *pIndex=0;
+    }
+    assert (table != NULL);
+    /* get level */
+    atom = table[*pIndex].level;
+    assert (ATOM_INVALID != atom);
+    /* check game mode */
+  } while ( gameMode != (gameMode & GetLevelGameMode (atom) ) ||
+	    (! allLevels &&
+	     ! GetLevelSelected (atom) ) );
+  /* mark as last level */
+  SetLevelWasLast (atom, entry, XBTrue);
+  StoreLevelTime(timeTable[*pIndex].level, time(NULL));
+  /* that's all */
+  return atom;
+} /* GetNextLevelFromTable */
+
+/*
+ * get next level from table, variant
+ */
+static XBAtom
+GetNextLevelFromTable2 (LevelTable *table, int *pIndex, XBAtom entry, void (*pFunc) (void))
+{
+  int    numLevels;
+  int    i,l,cnt;
+  time_t t0,t1;
+  XBAtom atom=ATOM_INVALID;
+
+  assert (pIndex != NULL);
+  assert (pFunc  != NULL);
+  numLevels = GetNumLevels ();
+  /* find time interval for selected levels matching gamemode*/
+  t0=0;
+  t1=0;
+  cnt=0;
+  for (i = 0; i < numLevels; i++) {
+    atom = table[i].level;
+    assert (ATOM_INVALID != atom);
+    if ( gameMode == (gameMode & GetLevelGameMode (atom) ) && ( allLevels ||  GetLevelSelected (atom) ) ) {
+      if (cnt==0) {
+	t0=table[i].key.lastplayed;
+	t1=t0;
+      } else {
+	if (table[i].key.lastplayed < t0) {
+	  t0 = table[i].key.lastplayed;
+	}
+	if (table[i].key.lastplayed > t1) {
+	  t1 = table[i].key.lastplayed;
+	}
+      }
+      cnt++;
+    }
+  }
+  Dbg_Level("There are %i selected levels between [%s",cnt,DateString(t0));
+  Dbg_Level(", %s]\n",DateString(t1));
+  assert (cnt > 0);
+  /* look at oldest tenth of time interval */
+  t1=t0+(t1-t0)/10;
+  Dbg_Level("Finding those between [%s",DateString(t0));
+  Dbg_Level(", %s]\n",DateString(t1));
+  cnt=0;
+  for (i = 0; i<numLevels; i++) {
+    atom = table[i].level;
+    assert (ATOM_INVALID != atom);
+    if( gameMode == (gameMode & GetLevelGameMode (atom) ) && ( allLevels ||  GetLevelSelected (atom) ) ) {
+      if (table[i].key.lastplayed <= t1) {
+	cnt++;
+      }
+    }
+  }
+  Dbg_Level("There were %i levels found between [%s",cnt,DateString(t0));
+  Dbg_Level(", %s]\n",DateString(t1));
+  assert (cnt > 0);
+  /* choose a random level of those */
+  l=GameRandomNumber(cnt)+1;
+  Dbg_Level("The %ith of %i levels was chosen\n",l,cnt);
+  /* find it in table */
+  i = 0;
+  while ( (i<numLevels) && (l>0)) {
+    atom = table[i].level;
+    assert (ATOM_INVALID != atom);
+    if ( gameMode == (gameMode & GetLevelGameMode (atom) ) && ( allLevels ||  GetLevelSelected (atom) ) && (table[i].key.lastplayed <= t1) ) {
+      l--;
+    }
+    i++;
+  }
+  assert(l==0);
+  /* update level time */
+  *pIndex = i-1;
+  t0=table[*pIndex].key.lastplayed;
+  Dbg_Level("Lastplayed time of level %s to %s\n",GetLevelNameByAtom(timeTable[*pIndex].level),DateString(t0));
+  t0=time(NULL);
+  table[*pIndex].key.lastplayed=t0;
+  Dbg_Level("Set lastplayed time of level %s to %s\n",GetLevelNameByAtom(timeTable[*pIndex].level),DateString(t0));
+  StoreLevelTime(timeTable[*pIndex].level, timeTable[*pIndex].key.lastplayed);
+  DB_Store(dbLevel);
+  /* that's all */
+  return atom;
+} /* GetNextLevelFromTable2 */
+
+#if 0
+/*
+ * convert level section to iob
+ */
+static XBTeleIOB
+SectionToIOB (XBAtom atom)
+{
+  if (GUI_CompareAtoms (atom, atomInfo) ) {
+    return IOB_INFO;
+  } else if (GUI_CompareAtoms (atom, atomPlayer) ) {
+    return IOB_PLAYER;
+  } else if (GUI_CompareAtoms (atom, atomShrink) ) {
+    return IOB_SHRINK;
+  } else if (GUI_CompareAtoms (atom, atomScrambleDraw) ) {
+    return IOB_SCRAMBLE_DRAW;
+  } else if (GUI_CompareAtoms (atom, atomScrambleDel) ) {
+    return IOB_SCRAMBLE_DEL;
+  } else if (GUI_CompareAtoms (atom, atomFunc) ) {
+    return IOB_FUNC;
+  } else if (GUI_CompareAtoms (atom, atomBombs) ) {
+    return IOB_BOMBS;
+  } else if (GUI_CompareAtoms (atom, atomGraphics) ) {
+    return IOB_GRAPHICS;
+  } else if (GUI_CompareAtoms (atom, atomMap) ) {
+    return IOB_MAP;
+  }
+  return IOB_INVALID;
+} /* SectionToIOB */
+#endif
+
+/*
+ * convert iob to level section
+ */
+static XBAtom
+IOBToSection (XBTeleIOB iob)
+{
+  switch (iob) {
+  case IOB_INFO:          return atomInfo;
+  case IOB_PLAYER:        return atomPlayer;
+  case IOB_SHRINK:        return atomShrink;
+  case IOB_SCRAMBLE_DRAW: return atomScrambleDraw;
+  case IOB_SCRAMBLE_DEL:  return atomScrambleDel;
+  case IOB_FUNC:          return atomFunc;
+  case IOB_BOMBS:         return atomBombs;
+  case IOB_GRAPHICS:      return atomGraphics;
+  case IOB_MAP:           return atomMap;
+  default:                return ATOM_INVALID;
+  }
+} /* IOBToSection */
+
+/*******************
+ * shared routines *
+ *******************/
+
+/*
+ * load all level info into database
+ */
+void
+LoadLevelConfig (void)
+{
+#ifdef DEBUG
+  Dbg_StartClock ();
+#endif
+  /* create empty level database */
+  dbLevel = DB_Create (DT_Config, atomLevel);
+  assert (dbLevel != NULL);
+  /* scan search paths */
+  Dbg_Config("loading list of levels\n");
+  if (DB_LoadDir (dbLevel, GAME_DATADIR"/level", "xal", DT_Level, atomTime, atomInfo, InsertLevel, XBTrue) ) {
+    SaveLevelConfig();
+  } else {
+    Dbg_Config("no changes found!\n");
+  }
+  Dbg_Out("init levels: %lu msec\n", Dbg_FinishClock ());
+} /* LoadLevelConfig */
+
+/*
+ * save level info from database to files
+ */
+void
+SaveLevelConfig (void)
+{
+  assert (dbLevel != NULL);
+  if (DB_Changed (dbLevel) ) {
+    Dbg_Config("saving level list!\n");
+    DB_Store (dbLevel);
+  }
+} /* SaveLevelConfig */
+
+/*
+ * clear local level databases
+ */
+void
+FinishLevelConfig (void)
+{
+  DB_Delete (dbLevel);
+  if (NULL != sortedTable) {
+    free (sortedTable);
+    sortedTable = NULL;
+  }
+  if (NULL != shuffledTable) {
+    free (shuffledTable);
+    shuffledTable = NULL;
+  }
+  if (NULL != timeTable) {
+    free (timeTable);
+    timeTable = NULL;
+  }
+  if (NULL != levelRemote) {
+    DB_Delete (levelRemote);
+    levelRemote = NULL;
+  }
+  if (NULL != levelLocal) {
+    DB_Delete (levelLocal);
+    levelLocal = NULL;
+  }
+  Dbg_Config("level config clear!\n");
+} /* FinishLevelConfig */
+
+/*
+ * number of level info's in database
+ */
+int
+GetNumLevels (void)
+{
+  return DB_NumSections (dbLevel);
+} /* GetNumLevels */
+
+/*
+ * get atom for level section with given index
+ */
+XBAtom
+GetLevelAtom (int index)
+{
+  return DB_IndexSection (dbLevel, index);
+} /* GetLevelAtom */
+
+/*
+ * get name of level from level atom
+ */
+const char *
+GetLevelNameByAtom (XBAtom level)
+{
+  return GetLevelStringByAtom (level, atomName);
+} /* GetLevelName */
+
+/*
+ * return selection flag for current level, given section
+ */
+XBBool
+GetLevelSelected (XBAtom level)
+{
+  XBBool selected;
+  const DBSection *section;
+  assert (dbLevel != NULL);
+  section = DB_GetSection (dbLevel, level);
+  if (NULL == section) {
+    return XBFalse;
+  }
+  if (! DB_GetEntryBool (section, atomSelect, &selected) ) {
+    return XBTrue;
+  }
+  return selected;
+} /* GetLevelSelected */
+
+/*
+ * set selection flag for given level, given section
+ */
+void
+StoreLevelSelected (XBAtom atom, XBBool select)
+{
+  DBSection *section;
+  assert (NULL != dbLevel);
+  section = DB_CreateSection (dbLevel, atom);
+  assert (section != NULL);
+  /* we only change data not stored in file */
+  Dbg_Level("Level select %s = %d\n",GetLevelNameByAtom(atom),select);
+  (void) DB_CreateEntryBool (section, atomSelect,  select);
+} /* StoreLevelSelection */
+
+/*
+ * load an actual level from file
  */
 const DBRoot *
 LoadLevelFile (XBAtom atom)
@@ -391,160 +766,10 @@ LoadLevelFile (XBAtom atom)
 } /* LoadLevelFile */
 
 /*
- * comprare shuffles by key
- */
-static int
-CompareSorted (const void *a, const void *b)
-{
-  return strcmp ( ( (LevelTable *) a)->key.name, ( (LevelTable *) b)->key.name);
-} /* CompareSorted */
-
-/*
- * comprare shuffles by key
- */
-static int
-CompareShuffle (const void *a, const void *b)
-{
-  return ( (LevelTable *) a)->key.value - ( (LevelTable *) b)->key.value;
-} /* CompareShuffle */
-
-/*
- * comprare shuffles by time
- */
-static int
-CompareTime (const void *a, const void *b)
-{
-  return ( (LevelTable *) a)->key.lastplayed - ( (LevelTable *) b)->key.lastplayed;
-} /* CompareShuffle */
-
-/*
- *
- */
-static int
-GetLastIndexSorted (void) 
-{
-  int              i,numLevels;
-  XBAtom           atom;
-
-  numLevels = GetNumLevels ();
-  /* build shuffle table */
-  if (NULL == sortedTable) {
-    sortedTable = calloc (numLevels, sizeof (LevelTable) );
-    assert (sortedTable != NULL);
-  }
-  for (i = 0; i < numLevels; i ++) {
-    atom = GetLevelAtom (i);
-    sortedTable[i].level    = atom;
-    sortedTable[i].key.name = GetLevelNameByAtom (atom);
-  }
-  qsort (sortedTable, numLevels,  sizeof (LevelTable), CompareSorted);
-  /* get last level from database */
-  for (i = 0; i < numLevels; i ++) {
-    if (GetLevelWasLast (sortedTable[i].level, atomLevelSorted) ) {
-      Dbg_Out ("last level (sorted)   = %s\n", GetLevelNameByAtom (sortedTable[i].level) );
-      return i;
-    }
-  }
-  return numLevels;
-} /* GetLastLevelIndex */
-
-/*
- *
- */
-static int
-GetLastIndexShuffled (void) 
-{
-  int i,j,k, numLevels;
-
-  Dbg_Out("Shuffling levels for the first time\n");
-  numLevels = GetNumLevels ();
-  /* build shuffle table */
-  if (NULL == shuffledTable) {
-    shuffledTable = calloc (numLevels, sizeof (LevelTable) );
-    assert (shuffledTable != NULL);
-  }
-  j=0;
-  k=0;
-  for (i = 0; i < numLevels; i ++) {
-    shuffledTable[i].level = GetLevelAtom (i);
-    assert (ATOM_INVALID != shuffledTable[i].level);    
-    if (!GetLevelShuffle (shuffledTable[i].level, &shuffledTable[i].key.value) ) { 
-      shuffledTable[i].key.value = -1;
-      j++;
-    }
-    if(shuffledTable[i].key.value > k) k = shuffledTable[i].key.value++;
-  }
-  for (i = 0; i < numLevels; i ++) {
-    if(shuffledTable[i].key.value < 0) {
-      shuffledTable[i].key.value = -k;
-      k++;
-    }
-  }
-
-  Dbg_Out("%i new keys for levels\n",j);
-  for(i=0;i<numLevels;i++) {
-    if(shuffledTable[i].key.value < 0) {
-      j=OtherRandomNumber(numLevels);
-      k=abs(shuffledTable[i].key.value);
-      shuffledTable[i].key.value=abs(shuffledTable[j].key.value);
-      shuffledTable[j].key.value=k;
-    }
-  }
-
-  for (i = 0; i < numLevels; i ++) {
-    StoreLevelShuffle(shuffledTable[i].level, shuffledTable[i].key.value);
-  }
-
-  qsort (shuffledTable, numLevels,  sizeof (LevelTable), CompareShuffle);
-  /* get last level from database */
-  for (i = 0; i < numLevels; i ++) {
-    if (GetLevelWasLast (shuffledTable[i].level, atomLevelShuffled) ) {
-      Dbg_Out ("last level (shuffled) = %s\n", GetLevelNameByAtom (shuffledTable[i].level) );
-      return i;
-    }
-  }
-  return numLevels;
-} /* GetLastLevelIndexShuffled */
-/*
- *
- */
-static int
-GetLastIndexTime (void) 
-{
-  int i, numLevels;
-
-  Dbg_Out("Timing levels for the first time\n");
-  numLevels = GetNumLevels ();
-  /* build shuffle table */
-  if (NULL == timeTable) {
-    timeTable = calloc (numLevels, sizeof (LevelTable) );
-    assert (timeTable != NULL);
-  }
-  for (i = 0; i < numLevels; i ++) {
-    timeTable[i].level = GetLevelAtom (i);
-    assert (ATOM_INVALID != timeTable[i].level);
-    if (!GetLevelTime (timeTable[i].level, &timeTable[i].key.lastplayed) ) { 
-      timeTable[i].key.lastplayed = 0;
-      StoreLevelTime(timeTable[i].level, timeTable[i].key.lastplayed);
-    }
-  }
-
-  /*
-  qsort (timeTable, numLevels,  sizeof (LevelTable), CompareTime);
-  i=0;
-  do {
-    Dbg_Out("Lastplayed time of level %s = %s\n",GetLevelNameByAtom(timeTable[i].level),DateString(timeTable[i].key.lastplayed));
-    i++;
-    } while(i < numLevels);  */
-
-  return numLevels;
-} /* GetLastLevelIndexTime */
-
-/*
- *
+ * store level setup locally
  */
 XBBool
-InitLevels (const CFGGame *cfgGame) 
+InitLevels (const CFGGame *cfgGame)
 {
   int    i;
   int    numSelected;
@@ -594,256 +819,35 @@ InitLevels (const CFGGame *cfgGame)
     }
     numSelected ++;
   }
-  Dbg_Out ("%d levels selected\n", numSelected);
+  Dbg_Level("%d levels selected\n", numSelected);
 
   /* last levels played ... */
   indexSorted   = GetLastIndexSorted ();
   indexShuffled = GetLastIndexShuffled ();
   indexTime     = GetLastIndexTime ();
   ReshuffleLevels ();
-  
+
   return (numSelected != 0);
 } /* InitLevels */
 
 /*
- * reset sorted level
- */
-static void
-ResortLevels (void)
-{
-  indexSorted = 0;
-} /* ResortLevels */
-
-/*
- * reset sorted level
- */
-static void
-ReTimeLevels (void)
-{
-  indexTime = 0;
-} /* ReTimeLevels */
-
-/*
- * recreate shuffle table
- */
-static void
-ReshuffleLevels (void)
-{
-  int i,j,k, numLevels;
-
-  Dbg_Out ("shuffling levels\n");
-  
-  numLevels = GetNumLevels ();
-  for (i=0; i< numLevels; i++) {
-    shuffledTable[i].key.value=i;
-  }
-  for (i = numLevels-1; i > 0 ; i--) {
-    j=OtherRandomNumber(i+1);
-    k=shuffledTable[i].key.value;
-    shuffledTable[i].key.value=shuffledTable[j].key.value;
-    shuffledTable[j].key.value=k;
-  }
-  for (i=0; i< numLevels; i++) {
-    StoreLevelShuffle(shuffledTable[i].level, shuffledTable[i].key.value);
-  }
-  SaveLevelConfig();
-  qsort (shuffledTable, numLevels,  sizeof (LevelTable), CompareShuffle);
-  indexShuffled = 0;
-} /* ShuffleLevels */
-
-/*
- * get next level from table
- */
-static XBAtom
-GetNextLevelFromTable (const LevelTable *table, int *pIndex, XBAtom entry, void (*pFunc) (void))
-{
-  int    numLevels;
-  XBAtom atom;
-  
-  assert (pIndex != NULL);
-  assert (pFunc  != NULL);
-  numLevels = GetNumLevels ();
-  /* unmark last level */
-  if (*pIndex < numLevels) {
-    SetLevelWasLast (table[*pIndex].level, entry, XBFalse);
-  }
-  /* find next level to play */
-  do {
-    *pIndex = *pIndex + 1;
-    if (*pIndex >= numLevels) {
-      (*pFunc) ();
-      *pIndex=0;
-    }
-    assert (table != NULL);
-    /* get level */
-    atom = table[*pIndex].level;
-    assert (ATOM_INVALID != atom);
-    /* check game mode */
-  } while ( gameMode != (gameMode & GetLevelGameMode (atom) ) ||
-	    (! allLevels && 
-	     ! GetLevelSelected (atom) ) );
-  /* mark as last level */
-  SetLevelWasLast (atom, entry, XBTrue);
-  StoreLevelTime(timeTable[*pIndex].level, time(NULL));
-  /* that's all */
-  return atom;
-} /* GetNextLevelFromTable */
-
-static XBAtom
-GetNextLevelFromTable2 (LevelTable *table, int *pIndex, XBAtom entry, void (*pFunc) (void))
-{
-  int    numLevels;
-  int    i,ll;
-  time_t t0,t1;
-  XBAtom atom;
-  
-  assert (pIndex != NULL);
-  assert (pFunc  != NULL);
-  numLevels = GetNumLevels ();
-  /* find next level to play */
-  t0=0;
-  t1=0;
-  ll=0;
-  *pIndex=0;
-  do {
-    atom = table[*pIndex].level;
-    assert (ATOM_INVALID != atom);
-    // Dbg_Out("Time of level %s = %s]\n",GetLevelNameByAtom(timeTable[*pIndex].level),DateString(table[*pIndex].key.lastplayed));
-    if( gameMode == (gameMode & GetLevelGameMode (atom) ) && ( allLevels ||  GetLevelSelected (atom) ) ) {	
-      if(ll==0) {
-	t0=table[*pIndex].key.lastplayed;
-	t1=t0;
-	ll++;
-      } else {
-	ll++;
-	if(table[*pIndex].key.lastplayed < t0) t0 = table[*pIndex].key.lastplayed;
-	if(table[*pIndex].key.lastplayed > t1) t1 = table[*pIndex].key.lastplayed;
-      }
-    }
-    *pIndex = *pIndex + 1;
-  } while(*pIndex < numLevels);
-  Dbg_Out("There are %i selected levels between [%s",ll,DateString(t0));
-  Dbg_Out(", %s]\n",DateString(t1));
-  assert (ll > 0);
-  t1=t0+(t1-t0)/10;
-  Dbg_Out("Finding those between [%s",DateString(t0));
-  Dbg_Out(", %s]\n",DateString(t1));
-  ll=0;
-  *pIndex=0;
-  do {
-    atom = table[*pIndex].level;
-    assert (ATOM_INVALID != atom);
-    if( gameMode == (gameMode & GetLevelGameMode (atom) ) && ( allLevels ||  GetLevelSelected (atom) ) ) {	
-      if(table[*pIndex].key.lastplayed <= t1) ll++;
-    }
-    *pIndex = *pIndex + 1;
-  } while(*pIndex < numLevels);  
-  Dbg_Out("There were %i levels found between [%s",ll,DateString(t0));
-  Dbg_Out(", %s]\n",DateString(t1));
-  assert (ll > 0);
-  i=GameRandomNumber(ll)+1;
-  Dbg_Out("The %ith Level was chosen\n",i);
-  *pIndex=0;
-  ll=1;
-  do {
-    atom = table[*pIndex].level;
-    assert (ATOM_INVALID != atom);
-    if( gameMode == (gameMode & GetLevelGameMode (atom) ) && ( allLevels ||  GetLevelSelected (atom) ) ) {	
-      if(table[*pIndex].key.lastplayed <= t1) {;
-        if(i==ll) break;
-	ll++;
-      }
-    }
-    *pIndex = *pIndex + 1;
-  } while(*pIndex < numLevels);
-  t0=table[*pIndex].key.lastplayed;
-  Dbg_Out("Lastplayed time of level %s to %s\n",GetLevelNameByAtom(timeTable[*pIndex].level),DateString(t0));
-  t0=time(NULL);
-  table[*pIndex].key.lastplayed=t0;
-  Dbg_Out("Set lastplayed time of level %s to %s\n",GetLevelNameByAtom(timeTable[*pIndex].level),DateString(t0));
-  StoreLevelTime(timeTable[*pIndex].level, timeTable[*pIndex].key.lastplayed);
-  DB_Store(dbLevel);
-
-  /* that's all */
-  return atom;
-} /* GetNextLevelFromTable */
-
-/*
- * get next level to plays
+ * get next level to play
  */
 XBAtom
 GetNextLevel (void)
 {
-  /* Old way
-  Dbg_Out ("get next level %s\n", randomOrder ? "shuffled" : "sorted");
-  if (randomOrder) {
-    return GetNextLevelFromTable (shuffledTable, &indexShuffled, atomLevelShuffled, ReshuffleLevels);
-  } else {
-    return GetNextLevelFromTable (sortedTable,   &indexSorted,   atomLevelSorted,   ResortLevels);
-  }*/
-  // New way
   if (levelOrder==2) { // LRF
-    Dbg_Out ("get next level: shuffled\n");
+    Dbg_Level ("get next level: shuffled\n");
     return GetNextLevelFromTable (shuffledTable, &indexShuffled, atomLevelShuffled, ReshuffleLevels);
   } else if (levelOrder==1) {
-    Dbg_Out ("get next level: alfabet\n");
+    Dbg_Level ("get next level: alfabet\n");
     return GetNextLevelFromTable (sortedTable,   &indexSorted,   atomLevelSorted,   ResortLevels);
   } else {
-    Dbg_Out ("get next level: time based\n");
-    return GetNextLevelFromTable2 (timeTable,   &indexTime,   atomLastPlayed,  ReTimeLevels);    
+    Dbg_Level ("get next level: time based\n");
+    return GetNextLevelFromTable2 (timeTable,   &indexTime,   atomLastPlayed,  ReTimeLevels);
   }
 } /* GetNextLevel */
 
-#if 0
-/*
- * convert level section to iob 
- */
-static XBTeleIOB
-SectionToIOB (XBAtom atom)
-{
-  if (GUI_CompareAtoms (atom, atomInfo) ) {
-    return IOB_INFO;
-  } else if (GUI_CompareAtoms (atom, atomPlayer) ) {
-    return IOB_PLAYER;
-  } else if (GUI_CompareAtoms (atom, atomShrink) ) {
-    return IOB_SHRINK;
-  } else if (GUI_CompareAtoms (atom, atomScrambleDraw) ) {
-    return IOB_SCRAMBLE_DRAW;
-  } else if (GUI_CompareAtoms (atom, atomScrambleDel) ) {
-    return IOB_SCRAMBLE_DEL;
-  } else if (GUI_CompareAtoms (atom, atomFunc) ) {
-    return IOB_FUNC;
-  } else if (GUI_CompareAtoms (atom, atomBombs) ) {
-    return IOB_BOMBS;
-  } else if (GUI_CompareAtoms (atom, atomGraphics) ) {
-    return IOB_GRAPHICS;
-  } else if (GUI_CompareAtoms (atom, atomMap) ) {
-    return IOB_MAP;
-  }
-  return IOB_INVALID;
-} /* SectionToIOB */
-#endif
-
-/*
- * convert iob to level section
- */
-static XBAtom
-IOBToSection (XBTeleIOB iob)
-{
-  switch (iob) {
-  case IOB_INFO:          return atomInfo;
-  case IOB_PLAYER:        return atomPlayer;
-  case IOB_SHRINK:        return atomShrink;
-  case IOB_SCRAMBLE_DRAW: return atomScrambleDraw;
-  case IOB_SCRAMBLE_DEL:  return atomScrambleDel;
-  case IOB_FUNC:          return atomFunc;
-  case IOB_BOMBS:         return atomBombs;
-  case IOB_GRAPHICS:      return atomGraphics;
-  case IOB_MAP:           return atomMap;
-  default:                return ATOM_INVALID;
-  }
-} /* IOBToSection */
-
 /*
  * send level data to host
  */
@@ -856,7 +860,6 @@ SendLevelConfig (XBSndQueue *queue, XBTeleCOT cot, const DBRoot *level)
   XBTelegram      *tele;
   size_t           len;
   char             buffer[256];
-
   /* sanity check */
   assert (queue != NULL);
   assert (level != NULL);
@@ -910,7 +913,6 @@ AddToRemoteLevelConfig (unsigned iob, const char *data)
 {
   XBAtom atom;
   DBSection *section;
-
   assert (levelRemote != NULL);
   if (IOB_NAME == iob) {
     /* level names received */
@@ -929,6 +931,67 @@ AddToRemoteLevelConfig (unsigned iob, const char *data)
   }
 } /* AddToRemoteLevelConfig */
 
+/*
+ * check local level for version (server)
+ */
+XBBool
+CheckLocalLevelConfig() {
+  assert (levelLocal != NULL);
+  return CheckAnyLevelConfig(levelLocal);
+} /* CheckLocalLevelConfig */
+
+/*
+ * check remote level for version (client)
+ */
+XBBool
+CheckRemoteLevelConfig() {
+  assert (levelRemote != NULL);
+  return CheckAnyLevelConfig(levelRemote);
+} /* CheckRemoteLevelConfig */
+
+/*
+ * check if received level is okay
+ * accept all levels without warnings
+ * warnings:
+ * - unknown features
+ * - incompatible server/client versions for some feature
+ */
+XBBool
+CheckAnyLevelConfig(const DBRoot *level) {
+  if (!ParseLevel(level)) {
+    return XBFalse;
+  }
+  if (GetWarningCount() > 0) {
+    return XBFalse;
+  }
+  return XBTrue;
+} /* CheckAnyLevelConfig */
+
+/*
+ * check all levels in database
+ */
+XBBool
+CheckAllLevelConfigs()
+{
+  XBAtom atom;
+  const DBRoot *lvl;
+  indexTime     = GetLastIndexTime ();
+  indexSorted = GetLastIndexSorted();
+  indexSorted = 0;
+  do {
+    Dbg_Out("-------- %i ---------\n", indexSorted);
+    atom = GetNextLevelFromTable (sortedTable,   &indexSorted,   atomLevelSorted,   ResortLevels);
+    Dbg_Out("+++LEVEL+++ current = %s(%s)!\n", GUI_AtomToString(atom), GetLevelNameByAtom(atom) );
+    lvl = LoadLevelFile(atom);
+    if (! ParseLevel(lvl) ) {
+      Dbg_Out("+++LEVEL+++ Parse failed!\n");
+      DB_Dump(lvl);
+      return XBFalse;
+    }
+  } while (indexSorted > 0);
+  return XBTrue;
+} /* CheckAllLevelConfigs */
+
 /*
  * retrieve remote level config
  */
