@@ -1,7 +1,7 @@
 /*
  * cfg_main.h - managing configuration files 
  *
- * $Id: cfg_main.h,v 1.3 2004/05/14 10:00:33 alfie Exp $
+ * $Id: cfg_main.h,v 1.4 2004/08/07 18:27:44 lodott Exp $
  * 
  * Program XBLAST 
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
@@ -23,6 +23,8 @@
 #ifndef XBLAST_CFG_MAIN_H
 #define XBLAST_CFG_MAIN_H
 
+#include "xblast.h"
+
 /*
  * global types
  */
@@ -39,11 +41,10 @@ typedef enum {
  */
 extern void InitConfig (void);
 extern void SaveConfig (void);
+extern XBBool CheckConfig (void);
 extern void FinishConfig (void);
 
 #endif
 /*
  * end of file cfg_main.h
  */
-
-
