@@ -1,9 +1,9 @@
 /*
  * cfg_player.c - player configuration data
- * 
- * $Id: cfg_player.c,v 1.3 2004/05/14 10:00:33 alfie Exp $
  *
- * Program XBLAST 
+ * $Id: cfg_player.c,v 1.9 2005/03/12 14:11:10 lodott Exp $
+ *
+ * Program XBLAST
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -12,7 +12,7 @@
  * any later version
  *
  * This program is distributed in the hope that it will be entertaining,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
  * Public License for more details.
  *
@@ -31,37 +31,37 @@
  * global variables
  */
 static const char *defaultShape[NUM_DEFAULT_PLAYERS] = {
-  "normal", "normal", "normal", "fat", "girl", "tall", 
+  "normal", "normal", "normal", "fat", "girl", "tall",
 };
 static CFGPlayerGraphics defaultPlayerGraphics[NUM_DEFAULT_PLAYERS] = {
   {
     ATOM_INVALID,
-    COLOR_TURQUOISE, COLOR_LIGHT_SALMON, COLOR_WHITE, COLOR_GRAY_75, 
+    COLOR_TURQUOISE, COLOR_LIGHT_SALMON, COLOR_WHITE, COLOR_GRAY_75,
     COLOR_GRAY_25, COLOR_ROYAL_BLUE, COLOR_WHITE,
   },
   {
     ATOM_INVALID,
-    COLOR_MIDNIGHT_BLUE, COLOR_LIGHT_SALMON, COLOR_NAVY_BLUE, COLOR_RED, 
+    COLOR_MIDNIGHT_BLUE, COLOR_LIGHT_SALMON, COLOR_NAVY_BLUE, COLOR_RED,
     COLOR_ROYAL_BLUE, COLOR_GOLD, COLOR_WHITE,
   },
   {
     ATOM_INVALID,
-    COLOR_RED, COLOR_LIGHT_SALMON, COLOR_RED, COLOR_FOREST_GREEN, 
+    COLOR_RED, COLOR_LIGHT_SALMON, COLOR_RED, COLOR_FOREST_GREEN,
     COLOR_INDIAN_RED, COLOR_DARK_SEA_GREEN, COLOR_WHITE,
   },
   {
     ATOM_INVALID,
-    COLOR_YELLOW, COLOR_LIGHT_SALMON, COLOR_SPRING_GREEN, COLOR_ORANGE_RED, 
+    COLOR_YELLOW, COLOR_LIGHT_SALMON, COLOR_SPRING_GREEN, COLOR_ORANGE_RED,
     COLOR_LIGHT_YELLOW, COLOR_ROYAL_BLUE, COLOR_WHITE,
   },
   {
     ATOM_INVALID,
-    COLOR_DEEP_PINK, COLOR_LIGHT_SALMON, COLOR_ORCHID, COLOR_SPRING_GREEN, 
+    COLOR_DEEP_PINK, COLOR_LIGHT_SALMON, COLOR_ORCHID, COLOR_SPRING_GREEN,
     COLOR_ROYAL_BLUE, COLOR_DEEP_PINK, COLOR_WHITE,
   },
   {
     ATOM_INVALID,
-    COLOR_BLACK, COLOR_TAN, COLOR_BLACK, COLOR_ORANGE, 
+    COLOR_BLACK, COLOR_TAN, COLOR_BLACK, COLOR_ORANGE,
     COLOR_BLACK, COLOR_ORANGE, COLOR_WHITE,
   },
 };
@@ -70,7 +70,7 @@ static const char *administratorShape = "normal";
 static CFGPlayerGraphics administratorPlayerGraphics =
   {
     ATOM_INVALID,
-    COLOR_TURQUOISE, COLOR_LIGHT_SALMON, COLOR_WHITE, COLOR_GRAY_75, 
+    COLOR_TURQUOISE, COLOR_LIGHT_SALMON, COLOR_WHITE, COLOR_GRAY_75,
     COLOR_GRAY_25, COLOR_ROYAL_BLUE, COLOR_WHITE,
   };
 
@@ -174,7 +174,7 @@ StoreAnyPlayerName (DBRoot *db, XBAtom atom, const char *name)
 
   assert (name != NULL);
   assert (ATOM_INVALID != atom);
-  
+
   section = DB_CreateSection (db, atom);
   assert (section != NULL);
   /* graphics */
@@ -193,15 +193,15 @@ StorePlayerName (CFGType cfgType, XBAtom atom, const char *name)
 /*
  *
  */
-static void 
+static void
 StoreAnyPlayerGraphics (DBRoot *db, XBAtom atom, const CFGPlayerGraphics *gfx)
 {
   DBSection *section;
-  
+
   assert (db != NULL);
   assert (gfx != NULL);
   assert (ATOM_INVALID != atom);
-  
+
   section = DB_CreateSection (db, atom);
   assert (section != NULL);
   /* graphics */
@@ -217,20 +217,20 @@ StoreAnyPlayerGraphics (DBRoot *db, XBAtom atom, const CFGPlayerGraphics *gfx)
 /*
  *
  */
-void 
+void
 StorePlayerGraphics (CFGType cfgType, XBAtom atom, const CFGPlayerGraphics *gfx)
 {
   StoreAnyPlayerGraphics (GetDB (cfgType), atom, gfx);
 } /* StoreLocalPlayerGraphics */
 
 /*
- * 
+ *
  */
 static XBBool
 RetrieveAnyPlayerGraphics (const DBRoot *db, XBAtom atom, XBColor teamColor, CFGPlayerGraphics *gfx)
 {
   const DBSection *section;
-  
+
   assert (db != NULL);
   assert (gfx != NULL);
   /* set to defaults */
@@ -249,11 +249,14 @@ RetrieveAnyPlayerGraphics (const DBRoot *db, XBAtom atom, XBColor teamColor, CFG
   (void) DB_GetEntryColor  (section, atomBackpack,  &gfx->backpack);
   /* set white color */
   gfx->white = COLOR_WHITE;
+  /* save original colors */
+  gfx->bodySave = gfx->body;
+  gfx->handsFeetSave = gfx->handsFeet;
   /* set team colors */
   if (teamColor != COLOR_INVALID) {
-    // KOEN
-    //gfx->helmet    = teamColor;
-    gfx->body      = teamColor;  
+    /* KOEN */
+    /* gfx->helmet    = teamColor; */
+    gfx->body      = teamColor;
     gfx->handsFeet = teamColor;
   }
   /* that's all */
@@ -302,6 +305,8 @@ StoreAnyPlayerMessages (DBRoot *db, XBAtom atom, const CFGPlayerMessages *msg)
   StoreMessage (section, atomMsgLoseLife,  msg->msgLoseLife);
   StoreMessage (section, atomMsgLoseLevel, msg->msgLoseLevel);
   StoreMessage (section, atomMsgGloat,     msg->msgGloat);
+  StoreMessage (section, atomMsgLaola,     msg->msgLaola);
+  StoreMessage (section, atomMsgLoser,     msg->msgLoser);
   StoreMessage (section, atomMsgWelcome,   msg->msgWelcome);
 } /* StorePlayerMessages */
 
@@ -316,7 +321,7 @@ StorePlayerMessages (CFGType cfgType, XBAtom atom, const CFGPlayerMessages *msg)
 
 
 /*
- * 
+ *
  */
 static XBBool
 RetrieveAnyPlayerMessages (const DBRoot *db, XBAtom atom, CFGPlayerMessages *msg)
@@ -337,6 +342,8 @@ RetrieveAnyPlayerMessages (const DBRoot *db, XBAtom atom, CFGPlayerMessages *msg
   (void) DB_GetEntryString (section, atomMsgLoseLife,  &msg->msgLoseLife);
   (void) DB_GetEntryString (section, atomMsgLoseLevel, &msg->msgLoseLevel);
   (void) DB_GetEntryString (section, atomMsgGloat,     &msg->msgGloat);
+  (void) DB_GetEntryString (section, atomMsgLaola,     &msg->msgLaola);
+  (void) DB_GetEntryString (section, atomMsgLoser,     &msg->msgLoser);
   (void) DB_GetEntryString (section, atomMsgWelcome,   &msg->msgWelcome);
   /* that's all */
   return XBTrue;
@@ -380,13 +387,13 @@ StorePlayerMisc (CFGType cfgType, XBAtom atom, const CFGPlayerMisc *misc)
 } /* StoreLocalPlayerMisc */
 
 /*
- * retrive misc player data 
+ * retrive misc player data
  */
 static XBBool
 RetrieveAnyPlayerMisc (const DBRoot *db, XBAtom atom, CFGPlayerMisc *misc)
 {
   const DBSection *section;
-  
+
   assert (db != NULL);
   assert (misc != NULL);
   /* set default values */
@@ -404,7 +411,7 @@ RetrieveAnyPlayerMisc (const DBRoot *db, XBAtom atom, CFGPlayerMisc *misc)
 } /* RetrievePlayerMisc */
 
 /*
- * retrive misc player data 
+ * retrive misc player data
  */
 XBBool
 RetrievePlayerMisc (CFGType cfgType, XBAtom atom, CFGPlayerMisc *misc)
@@ -413,7 +420,7 @@ RetrievePlayerMisc (CFGType cfgType, XBAtom atom, CFGPlayerMisc *misc)
 } /* RetrieveLocalPlayerMisc */
 
 /*
- * save player config at once 
+ * save player config at once
  */
 void
 StorePlayer (CFGType cfgType, XBAtom atom, const CFGPlayer *cfg)
@@ -430,7 +437,7 @@ StorePlayer (CFGType cfgType, XBAtom atom, const CFGPlayer *cfg)
 /*
  * load player config at once
  */
-XBBool 
+XBBool
 RetrievePlayer (CFGType cfgType, XBAtom atom, XBColor color, CFGPlayer *cfg)
 {
   XBBool result = XBTrue;
@@ -457,7 +464,7 @@ RetrievePlayer (CFGType cfgType, XBAtom atom, XBColor color, CFGPlayer *cfg)
 /*
  * load player config at once
  */
-XBBool 
+XBBool
 RetrievePlayerEx (CFGType cfgType, XBAtom atom, CFGPlayerEx *cfg)
 {
   XBBool result = XBTrue;
@@ -484,7 +491,7 @@ RetrievePlayerEx (CFGType cfgType, XBAtom atom, CFGPlayerEx *cfg)
 } /* RetrievePlayerEx */
 
 /*
- * XBCC save player EX config at once 
+ * XBCC save player EX config at once
  */
 void
 StorePlayerEx (CFGType cfgType, XBAtom atom, const CFGPlayerEx *cfg)
@@ -501,14 +508,14 @@ StorePlayerEx (CFGType cfgType, XBAtom atom, const CFGPlayerEx *cfg)
 
 
 /*
- * init and load player config 
+ * init and load player config
  */
 void
 LoadPlayerConfig (void)
 {
   int    i;
   XBAtom atom;
-    
+
   /* set default values */
   newGfx.shape = GUI_StringToAtom ("normal");
   /* initialize remote databse */
@@ -517,9 +524,11 @@ LoadPlayerConfig (void)
   /* initialiaze config database */
   dbLocal = DB_Create (DT_Config, atomPlayer);
   assert (dbLocal != NULL);
+  Dbg_Config("loading local player configs\n");
   if (DB_Load (dbLocal)) {
     return;
   }
+  Dbg_Config("failed to load player config, setting defaults\n");
   /* set some useful default values */
   for (i = 0; i < NUM_DEFAULT_PLAYERS; i ++) {
     atom = GUI_StringToAtom (defaultName[i]);
@@ -527,7 +536,7 @@ LoadPlayerConfig (void)
     StorePlayerGraphics (CT_Local, atom, DefaultPlayerGraphics (i));
     StorePlayerMisc     (CT_Local, atom, &newMisc);
     StorePlayerMessages (CT_Local, atom, &newMsg);
-    StorePlayerID       (CT_Local, atom, &newID);    
+    StorePlayerID       (CT_Local, atom, &newID);
   }
   DB_Store (dbLocal);
 } /* InitPlayerConfig */
@@ -537,14 +546,16 @@ LoadPlayerConfig (void)
  */
 void
 SavePlayerConfig (void)
-{ 
+{
   assert (dbLocal != NULL);
   if (DB_Changed (dbLocal) ) {
+    Dbg_Config("saving local player configs\n");
     DB_Store (dbLocal);
   }
 #ifdef DEBUG
   assert (dbRemote != NULL);
   if (DB_Changed (dbRemote) ) {
+    Dbg_Config("saving remote player configs\n");
     DB_Store (dbRemote);
   }
 #endif
@@ -564,6 +575,7 @@ FinishPlayerConfig (void)
     DB_Delete (dbRemote);
     dbRemote = NULL;
   }
+  Dbg_Config("player configs cleared\n");
 } /* FinishPlayerConfig */
 
 /*
@@ -574,7 +586,7 @@ LoadPlayerCentral (XBBool amCentral)
 {
   XBAtom atom;
   isCentral = amCentral;
-    
+
   /* set default values */
   newGfx.shape = GUI_StringToAtom ("normal");
   if(amCentral) {
@@ -606,7 +618,7 @@ LoadPlayerCentral (XBBool amCentral)
  */
 void
 SavePlayerCentral (void)
-{ 
+{
   assert (dbCentral != NULL);
   if (DB_Changed (dbCentral) ) {
     DB_Store (dbCentral);
@@ -680,19 +692,39 @@ GetPlayerName (CFGType cfgType, XBAtom atom)
   return GetAnyPlayerName (GetDB (cfgType), atom);
 } /* GetLocalPlayerName */
 
+/*
+ * search player names for newplayer name
+ */
+int
+FindDoubleName(CFGType cfgType, XBAtom newplayer)
+{
+  XBAtom      atom;
+  const char *cmp;
+  int j;
+  cmp =  GetPlayerName(cfgType,newplayer);
+  for (j=0; j<GetNumPlayerConfigs(cfgType); j++) {
+    atom = GetPlayerAtom(cfgType,j);
+    if (atom != newplayer) {
+      if (0 == strcmp(cmp,GetPlayerName(cfgType,atom))) {
+	return(j);
+      }
+    }
+  }
+  return(-1);
+} /* FindDoubleName */
 
-/* 
+/*
  * XBCC
  */
-static void 
+static void
 StoreAnyPlayerID (DBRoot *db, XBAtom atom, const CFGPlayerID *id)
 {
   DBSection *section;
-  
+
   assert (db != NULL);
   assert (id != NULL);
   assert (ATOM_INVALID != atom);
-  
+
   section = DB_CreateSection (db, atom);
   assert (section != NULL);
   /* graphics */
@@ -703,14 +735,14 @@ StoreAnyPlayerID (DBRoot *db, XBAtom atom, const CFGPlayerID *id)
 /*
  *
  */
-void 
+void
 StorePlayerID (CFGType cfgType, XBAtom atom, const CFGPlayerID *id)
 {
   StoreAnyPlayerID (GetDB (cfgType), atom, id);
 } /* StoreLocalPlayerGraphics */
 
 /*
- * 
+ *
  */
 static XBBool
 RetrieveAnyPlayerID (const DBRoot *db, XBAtom atom, CFGPlayerID *id)
@@ -726,6 +758,8 @@ RetrieveAnyPlayerID (const DBRoot *db, XBAtom atom, CFGPlayerID *id)
   if (NULL == section) {
     return XBFalse;
   }
+  /* set default PID to invalid */
+  id->PID = -1;
   (void) DB_GetEntryString (section, atomPass,  &id->pass);
   (void) DB_GetEntryInt    (section, atomPID ,  &id->PID);
   /* that's all */
@@ -741,18 +775,18 @@ RetrievePlayerID (CFGType cfgType, XBAtom atom, CFGPlayerID *id)
   return RetrieveAnyPlayerID (GetDB (cfgType), atom, id);
 } /* RetrievePlayerMessages*/
 
-/* 
+/*
  * XBCC rating
  */
-static void 
+static void
 StoreAnyPlayerRating (DBRoot *db, XBAtom atom, const CFGPlayerRating *rating)
 {
   DBSection *section;
-  
+
   assert (db != NULL);
   assert (rating != NULL);
   assert (ATOM_INVALID != atom);
-  
+
   section = DB_CreateSection (db, atom);
   assert (section != NULL);
   /* graphics */
@@ -767,14 +801,14 @@ StoreAnyPlayerRating (DBRoot *db, XBAtom atom, const CFGPlayerRating *rating)
 /*
  *
  */
-void 
+void
 StorePlayerRating (CFGType cfgType, XBAtom atom, const CFGPlayerRating *rating)
 {
   StoreAnyPlayerRating (GetDB (cfgType), atom, rating);
 } /* StoreLocalPlayerRating */
 
 /*
- * 
+ *
  */
 static XBBool
 RetrieveAnyPlayerRating (const DBRoot *db, XBAtom atom, CFGPlayerRating *rating)
@@ -824,7 +858,7 @@ DeletePlayerConfig (CFGType cfgType, XBAtom atom)
  * create new player config
  */
 static XBAtom
-CreatePlayerConfig (CFGType cfgType, const char *name, const CFGPlayerGraphics *cfgGfx, 
+CreatePlayerConfig (CFGType cfgType, const char *name, const CFGPlayerGraphics *cfgGfx,
 		    const CFGPlayerMessages *cfgMsg, const CFGPlayerMisc *cfgMisc, const CFGPlayerID *cfgID)
 {
   XBAtom  atom;
@@ -896,7 +930,7 @@ RenamePlayerConfig (CFGType cfgType, XBAtom atom, const char *name)
 /*
  * compare to graphics sets
  */
-XBBool 
+XBBool
 ComparePlayerGraphics (const CFGPlayerGraphics *a, const CFGPlayerGraphics *b)
 {
   assert (a != NULL);
@@ -917,10 +951,10 @@ static XBBool
 SendAnyPlayerConfig (const DBRoot *db, XBSndQueue *queue, XBTeleCOT cot, XBTeleIOB iob, XBAtom atom, XBBool toCentral)
 {
   const  DBSection *section;
-  int         	    i,k,l;
-  size_t      	    len;
-  XBTelegram 	   *tele;
-  char        	    tmp[256];
+  int	    i,k,l;
+  size_t	    len;
+  XBTelegram	   *tele;
+  char	    tmp[256];
   char              pass[256];
 
   assert (db != NULL);
@@ -979,21 +1013,20 @@ AddToPlayerConfig (CFGType cfgType, XBAtom atom, const char *line)
   (void) DB_ParseEntry (section, line);
 } /* AddToPlayerConfig */
 
-
 /*
  * XBST Store game results
  */
-void 
+void
 StoreAnyGameResult (DBRoot *db, XBAtom atom, int k, int *regPl, int *PID, int *Score) // XBST
 {
   DBSection *section;
   int i,j;
-  
+
   assert (db != NULL);
   assert (PID != NULL);
   assert (Score != NULL);
   assert (ATOM_INVALID != atom);
-  
+
   section = DB_CreateSection (db, atom);
   assert (section != NULL);
   for(i=0;i<k;i++) {
@@ -1001,26 +1034,40 @@ StoreAnyGameResult (DBRoot *db, XBAtom atom, int k, int *regPl, int *PID, int *S
     (void) DB_CreateEntryGameResult(section, GUI_IntToAtom(PID[j]), abs(Score[j]));
   }
 } /* StorePlayerRating */
- 
+
 /*
- *
+ * store game result in database
  */
 void
 StoreGameResult (CFGType cfgType, XBAtom atom, int k, int *regPl, int *PID, int *Score) { // XBST
   StoreAnyGameResult(GetDB (cfgType), atom, k,regPl,PID,Score);
 } /* StoreGameResult */
 
-void 
+/*
+ * store game result, append to file and delete in database
+ */
+void
+AppendGameResult (CFGType cfgType, XBAtom fname, XBAtom atom, int k, int *regPl, int *PID, int *Score)
+{ /*  XBST */
+  DBRoot *res;
+  res = DB_Create (cfgType, fname);
+  assert (res != NULL);
+  StoreAnyGameResult(res, atom, k,regPl,PID,Score);
+  DB_Append(res);
+  DB_Delete(res);
+} /* AppendGameResult */
+
+void
 StoreAnyTimePlayerRating (DBRoot *db, XBAtom atom, int k, int *regPl, int *PID, float *rating) // XBST
 {
   DBSection *section;
   int i,j;
-  
+
   assert (db != NULL);
   assert (PID != NULL);
   assert (rating != NULL);
   assert (ATOM_INVALID != atom);
-  
+
   section = DB_CreateSection (db, atom);
   assert (section != NULL);
   for(i=0;i<k;i++) {
@@ -1028,16 +1075,29 @@ StoreAnyTimePlayerRating (DBRoot *db, XBAtom atom, int k, int *regPl, int *PID,
     (void) DB_CreateEntryFloat(section, GUI_IntToAtom(PID[j]), rating[j]);
   }
 } /* StorePlayerRating */
- 
+
 /*
- *
+ * store timed player rating to database
  */
 void
 StoreTimePlayerRating (CFGType cfgType, XBAtom atom, int k, int *regPl, int *PID, float *rating) { // XBST
   StoreAnyTimePlayerRating (GetDB (cfgType), atom, k,regPl,PID,rating);
 } /* StoreGameResult */
 
+/*
+ * store timed player rating, append to file and delete in database
+ */
+void
+AppendTimePlayerRating (CFGType cfgType, XBAtom fname, XBAtom atom, int k, int *regPl, int *PID, float *rating)
+{ /*  XBST */
+  DBRoot *res;
+  res = DB_Create (cfgType, fname);
+  assert (res != NULL);
+  StoreAnyTimePlayerRating (res, atom, k,regPl,PID,rating);
+  DB_Append(res);
+  DB_Delete(res);
+} /* StoreGameResult */
+
 /*
  * end of file cfg_player.c
  */
-
