@@ -1,9 +1,9 @@
 /*
- * file com_central.h - server answers to broadcasts by clients
+ * file com_central.h - central answers to browse from clients
  *
- * $Id: com_central.h,v 1.3 2004/05/14 10:00:33 alfie Exp $
+ * $Id: com_central.h,v 1.4 2005/01/23 14:19:59 lodott Exp $
  *
- * Program XBLAST 
+ * Program XBLAST
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
  * Added by Koen De Raedt for central support
  *
@@ -13,7 +13,7 @@
  * any later version
  *
  * This program is distributed in the hope that it will be entertaining,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
  * Public License for more details.
  *
@@ -34,9 +34,9 @@
 /*
  * global prototypes
  */
-extern int com_GetOpenGames();
-extern void Central_ClearOldGames();
-extern XBComm* Central_CreateComm (unsigned short centralPort);
+extern int C2B_GetOpenGames();
+extern void C2B_ClearOldGames();
+extern XBComm* C2B_CreateComm (unsigned short centralPort);
 
 #endif
 /*
