@@ -1,7 +1,7 @@
 /*
  * file com_listen.h - listen for client to connect
  *
- * $Id: com_listen.h,v 1.3 2004/05/14 10:00:33 alfie Exp $
+ * $Id: com_listen.h,v 1.4 2004/11/05 16:43:17 lodott Exp $
  *
  * Program XBLAST 
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
@@ -34,7 +34,7 @@
  * global prototypes
  */
 extern XBComm *CommCreateListen (const CFGGameHost *, XBBool central);
-extern void CommDeleteListen (XBComm *);
+extern void CommFinishListen (XBComm *);
 
 #endif
 /*
