@@ -1,7 +1,7 @@
 /*
  * file com_newgame.h - client newGamees for local network game
  *
- * $Id: com_newgame.h,v 1.4 2004/07/07 10:24:20 iskywalker Exp $
+ * $Id: com_newgame.h,v 1.6 2004/10/30 20:11:40 lodott Exp $
  *
  * Program XBLAST 
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
@@ -33,9 +33,7 @@
  * global prototypes
  */
 extern XBComm *NewGame_CreateComm (const char *deviceAddress, unsigned short port, const char *broadcastAddress, const CFGGameHost *cfg, const CFGGameSetup *setup);
-/* GAMEONFIX */
-extern void    NewGame_Send (XBComm *qComm, const struct timeval *tv, const char *busy);
-/* GAMEONFIX */
+extern void    NewGame_Send (XBComm *qComm, const struct timeval *tv, int num, const char *score);
 extern void    NewGame_Close (XBComm *comm, const struct timeval *tv);
 #endif
 /*
