@@ -1,7 +1,7 @@
 /*
  * file com_query.h - client queryes for local network game
  *
- * $Id: com_query.h,v 1.3 2004/05/14 10:00:33 alfie Exp $
+ * $Id: com_query.h,v 1.7 2005/01/21 22:22:26 lodott Exp $
  *
  * Program XBLAST 
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
@@ -26,13 +26,15 @@
 #include "com.h"
 
 #include "common.h"
+#include "xblast.h"
 
 /*
  * global prototypes
  */
-extern XBComm *Query_CreateComm (const char *deviceAddress, unsigned short port, const char *broadcastAddress);
+extern XBComm *Query_CreateComm (unsigned id, const char *local, const char *remote, unsigned short port, XBBool broadcast);
 extern void    Query_Send (XBComm *qComm, const struct timeval *tv);
-extern void    Query_NewGame (XBComm *qComm, const struct timeval *tv); // XBCC 
+extern void    Query_NewGame (XBComm *qComm, const struct timeval *tv);
+extern XBBool  Query_isDeleted (XBComm *qComm);
 
 #endif
 /*
