@@ -1,7 +1,7 @@
 /*
  * file com_to_server.c - client's communication with server
  *
- * $Id: com_to_server.h,v 1.3 2004/05/14 10:00:33 alfie Exp $
+ * $Id: com_to_server.h,v 1.10 2004/10/19 21:19:44 lodott Exp $
  *
  * Program XBLAST 
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
@@ -24,17 +24,38 @@
 #define XBLAST_COM_TO_SERVER_C
 
 #include "com_base.h"
+#include "com_stream.h"
 #include "cfg_game.h"
 #include "network.h"
+#include "chat.h"
 
 /*
  * global prototypes
  */
+
+/* constructor */
 extern XBComm *C2S_CreateComm (const CFGGameHost *);
+
+/* get local data */
 extern const char *C2S_ServerName (XBComm *);
 extern const char *C2S_ClientName (XBComm *);
+
+/* set local data */
+extern void C2S_MarkEOF(XBComm *comm, XBBool flag);
+
+/* queue data */
 extern void C2S_SendDgramPort (XBComm *, unsigned short);
+extern void C2S_GameDataNotAvailable(XBComm *comm);
+extern XBBool C2S_SendGameConfig (XBComm *comm, CFGType cfgType, XBAtom atom);
+extern void C2S_PlayerDataNotAvailable(XBComm *comm, unsigned id);
+extern XBBool C2S_SendPlayerConfig (XBComm *comm, CFGType cfgType, XBAtom atom, unsigned player, XBBool how);
+extern void C2S_SendHostState (XBComm *comm, unsigned state);
+extern void C2S_SendHostStateReq (XBComm *comm, unsigned host, unsigned state);
+extern void C2S_SendTeamStateReq (XBComm *comm, unsigned host, unsigned player, unsigned state);
+extern void C2S_SendChat (XBComm *comm, XBChat *chat);
 extern void C2S_Sync (XBComm *, XBNetworkEvent);
+extern void C2S_LevelCheck (XBComm *comm, XBBool rej);
+extern void C2S_SendWinner (XBComm *comm, unsigned team);
 
 #endif
 /*
