@@ -1,7 +1,7 @@
 /*
  * func.c - function pointers for special extras
  *
- * $Id: func.c,v 1.6 2004/06/30 16:06:00 iskywalker Exp $
+ * $Id: func.c,v 1.15 2005/03/10 17:53:25 lodott Exp $
  *
  * Program XBLAST 
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
@@ -43,16 +43,18 @@ void (*specialKeyFunc) (BMPlayer *);
  */
 typedef void (*SpecialExtraFunc) (BMPlayer *);
 typedef void (*SpecialKeyFunc) (BMPlayer *);
+static char* seVoidName = "No Special Extra";
+static char* skVoidName = "No Special Key";
 
 /* 
  * void functions 
  */
-static void 
+void 
 SpecialExtraVoid (BMPlayer *ps)
 {
 }
 
-static void 
+void 
 SpecialKeyVoid (BMPlayer *ps)
 {
 }
@@ -139,6 +141,29 @@ SpecialKeyChoice (BMPlayer *ps)
       }
     }
 }
+/* Written by VVL */ 
+static void
+SpecialKeyThrough (BMPlayer *ps)
+{
+  if ((ps->throughCount > 0) && (ps->lives > 0)) {
+    if( ps->through){
+      ps->through=0;
+    }
+    else{
+    ps->through=EXTRA_THROUGH_TIME;
+    ps->throughCount--;
+    }
+  }
+}
+/**/
+/* Walk through bomb Mode */
+/**/
+static void
+SpecialExtraThrough (BMPlayer *ps)
+{
+  ps->throughCount+=1;
+}
+  
 
 /* 
  * Teleport 
@@ -359,16 +384,16 @@ SpecialExtraHolyGrail (BMPlayer *ps)
 } /* SpecialExtraHolyGrail */
 
 
-/* Added by "Belgium Guys" */ /**/
-/* Added by "Belgium Guys" */ /* steal extra */
-/* Added by "Belgium Guys" */ /**/
-/* Added by "Belgium Guys" */ static void
-/* Added by "Belgium Guys" */ SpecialExtraSteal (BMPlayer *ps)
-     /* Added by "Belgium Guys" */ {
-  /* Added by "Belgium Guys" */ 
-  /* Added by "Belgium Guys" */   ps->bombs += StealBombsOtherPlayers(ps->team);
-  /* Added by "Belgium Guys" */   ps->range += StealRangeOtherPlayers(ps->team);
-  /* Added by "Belgium Guys" */ }
+/* Added by "Belgium Guys" */ 
+/* steal extra */
+/**/
+static void
+SpecialExtraSteal (BMPlayer *ps)
+{
+  
+  ps->bombs += StealBombsOtherPlayers(ps->team);
+  ps->range += StealRangeOtherPlayers(ps->team);
+}
 /*
  * multiple extra 
  */
@@ -418,26 +443,27 @@ SpecialExtraMultiple (BMPlayer *ps)
 
 /*Skywalker epfl */
 
-  /* Written by VVL */ /**/
+/* Written by VVL */
 /* Electrify */
-/* Written by VVL */ void
-/* Written by VVL */ SpecialExtraElectrify (BMPlayer *ps)
-     /* Written by VVL */ {
-  /* Written by VVL */   ps->electrify=EXTRA_ELECTRIFY_COUNT;
-  /* Written by VVL */ }
-/* Written by VVL */ void
-/* Written by VVL */ SpecialKeyElectrify (BMPlayer *ps)
-     /* Written by VVL */ {
-  /* Written by VVL */   if ((ps->lives > 0) && (ps->electrify)) {
-    /* Written by VVL */      if (ElectrifyOtherPlayers(ps->id)) {
-      /* Written by VVL */        /* Success ... another player is hit! :-) */
-      /* Written by VVL */        ps->electrify=0;
-      /* Written by VVL */      } else {
-	/* Written by VVL */        /* Failure ... just try again! :-( */
-	/* Written by VVL */        ps->electrify--;
-	/* Written by VVL */      }
-    /* Written by VVL */   }
- /* Written by VVL */ }
+void
+SpecialExtraElectrify (BMPlayer *ps)
+{
+  ps->electrify=EXTRA_ELECTRIFY_COUNT;
+}
+void
+SpecialKeyElectrify (BMPlayer *ps)
+{
+  if ((ps->lives > 0) && (ps->electrify)) {
+    if (ElectrifyOtherPlayers(ps->id)) {
+      /* Success ... another player is hit! :-) */
+      ps->electrify=0;
+    } else {
+      /* Failure ... just try again! :-( */
+      ps->electrify--;
+    }
+  }
+}
+
 static void
 SpecialExtraFarter (BMPlayer *ps)
 {
@@ -496,7 +522,7 @@ SpecialExtraSwapPosition (BMPlayer *ps)
 /* */
 
 /*
- * cloakung device
+ * cloaking device
  */
 static void
 SpecialExtraCloak (BMPlayer *ps)
@@ -554,7 +580,6 @@ SpecialKeyMorph (BMPlayer *ps)
   }
 } /* SpecialKeyMorph */
 
-
 void 
 SpecialExtraStop (BMPlayer *ps)
 {
@@ -571,37 +596,31 @@ SpecialKeyStop (BMPlayer *ps)
   }
 }
 
-/* Written by Amilhastre */ static   void
-/* Written by Amilhastre */ SpecialExtraEvilGrail (BMPlayer *ps)
-     /* Written by Amilhastre */ {
-  /* Written by Amilhastre */     ps->evilill = ILLDEATHTIME;
-  /* Written by Amilhastre */ }
+/* Written by Amilhastre */
+static   void
+SpecialExtraEvilGrail (BMPlayer *ps)
+{
+  ps->evilill = ILLDEATHTIME;
+}
 
-/* Written by Amilhastre */ /* phantom extra */
-/* Written by Amilhastre */ /**/ static
-/* Written by Amilhastre */ void
-/* Written by Amilhastre */ SpecialExtraPhantom (BMPlayer *ps)
-     /* Written by Amilhastre */ {
-  /* Written by Amilhastre */   if (!ps->phantom)
-    /* Written by Amilhastre */     ps->phantom = 3*EXTRA_INVISIBLE;
-  /* Written by Amilhastre */   else
-    /* Written by Amilhastre */     ps->phantom += EXTRA_INVISIBLE;
-  /* Written by Amilhastre */ }
-/* Written by Amilhastre */ 
-/* Written by Amilhastre */ 
-/* Written by Amilhastre */ 
-/* Written by Amilhastre */ /**/
-/* Written by Amilhastre */ /* revive extra */
-/* Written by Amilhastre */ /**/  static
-/* Written by Amilhastre */ void
-/* Written by Amilhastre */ 
+/* phantom extra */
+/* Written by Amilhastre */
+static void
+SpecialExtraPhantom (BMPlayer *ps)
+{
+  if (!ps->phantom)
+         ps->phantom = 3*EXTRA_INVISIBLE;
+  else
+    ps->phantom += EXTRA_INVISIBLE;
+}
+ 
+/* revive extra */
+/* Written by Amilhastre */
+static void
 SpecialExtraRevive (BMPlayer *ps)
-     /* Written by Amilhastre */ {
-  /* Written by Amilhastre */     ps->revive++;
-  /* Written by Amilhastre */ }
-/* Written by Amilhastre */ 
-/* Written by Amilhastre */ 
-
+{
+  ps->revive++;
+}
 
 /* ** Skywalker **
  * sniping extra (player moves the bomb)
@@ -630,7 +649,6 @@ SpecialKeySnipe (BMPlayer *ps)
       }
     }
   }
- 
   
 } /* SpecialKeySnipe */
 
@@ -661,8 +679,8 @@ static DBToData extraTable[] = {
   { "choicebombtype",     (void *) SpecialExtraChoice },        /* skywalker */  
   { "cloak",              (void *) SpecialExtraCloak }, 
   { "daleif",             (void *) SpecialExtraDaleif }, 
-  {"electrify",           (void *) SpecialExtraElectrify },
-  {  "evilgrail",         (void *) SpecialExtraEvilGrail},
+  { "electrify",          (void *) SpecialExtraElectrify },
+  { "evilgrail",          (void *) SpecialExtraEvilGrail},
   { "farter",             (void *) SpecialExtraFarter},        /* skywalker */     
   { "frogger",            (void *) SpecialExtraFrogger },      /* skywalker */     
   { "ghost",              (void *) SpecialExtraGhost },      /* skywalker */     
@@ -685,20 +703,22 @@ static DBToData extraTable[] = {
   { "snipe",              (void *) SpecialExtraSnipe },          /* skywalker */
   { "specialBomb",        (void *) SpecialExtraSpecialBomb }, 
   { "speed",              (void *) SpecialExtraSpeed },     
-  { "speed2",              (void *) SpecialExtraSpeed2 },    
+  { "speed2",             (void *) SpecialExtraSpeed2 },    
   { "steal",              (void *) SpecialExtraSteal },      
   { "stop",               (void *) SpecialExtraStop}, 
   { "stunOthers",         (void *) SpecialExtraStunOthers },  
   { "sucker",             (void *) SpecialExtraSuck }, 
   { "swapcolor",          (void *) SpecialExtraSwapColor },    /* skywalker */   
   { "swapposition",       (void *) SpecialExtraSwapPosition },      /* skywalker */
-  { "teleport",           (void *) SpecialExtraTeleport },    
+  { "teleport",           (void *) SpecialExtraTeleport }, 
+  { "through",            (void *) SpecialExtraThrough },    
+  { "void",	          (void *) SpecialExtraVoid },    
   { NULL, NULL }
 };
 /* extra key functions */
 static DBToData keyTable[] = {
   { "air",	          (void *) SpecialKeyAir },       
-  { "bfarter",            (void *)  SpecialKeyBfarter},
+  { "bfarter",            (void *) SpecialKeyBfarter},
   { "choicebombtype",     (void *) SpecialKeyChoice },        /* skywalker */   
   { "cloak",	          (void *) SpecialKeyCloak },  
   { "electrify",          (void *) SpecialKeyElectrify },
@@ -711,32 +731,73 @@ static DBToData keyTable[] = {
   { "specialBomb",        (void *) SpecialKeySpecialBomb }, 
   { "stop",               (void *) SpecialKeyStop},
   { "sucker",             (void *) SpecialKeySuck }, 
-  { "teleport",	          (void *) SpecialKeyTeleport },    
+  { "teleport",	          (void *) SpecialKeyTeleport },
+  { "through",	          (void *) SpecialKeyThrough }, 
+  { "void",	          (void *) SpecialKeyVoid },    
   { NULL, NULL }
 };
 
 /*
- * parse section [FUNC]
+ * parse func section of level data
  */
 XBBool
-ConfigLevelFunc (const DBSection *section)
+ParseLevelFunc (const DBSection *section, DBSection *warn)
 {
   void *ptr;
-
-  /* special extra func */
-  if (DB_ConvertEntryData (section, atomExtra, &ptr, extraTable) ) {
-    assert (ptr !=  NULL);
-    specialExtraFunc = (SpecialExtraFunc) ptr;
-  } else {
+  /* check if section exists */
+  if (NULL == section) {
+    Dbg_Out("LEVEL: func section is missing!\n");
+    DB_CreateEntryString(warn,atomMissing,"true");
+    return XBFalse;
+  }
+  /* Extra has default */
+  switch (DB_ConvertEntryData (section, atomExtra, &ptr, extraTable) ) {
+  case DCR_NoSuchEntry:
+    Dbg_Level("default for %s\n",  DB_SectionEntryString(section,atomExtra));
     specialExtraFunc = SpecialExtraVoid;
+    break;
+  case DCR_Failure:
+    Dbg_Out("LEVEL: warning for %s\n", DB_SectionEntryString(section, atomExtra));
+    specialExtraFunc = SpecialExtraVoid;
+    DB_CreateEntryString(warn,atomExtra, DB_DataToString(extraTable, specialExtraFunc));
+    break;
+  default:
+    assert (ptr != NULL);
+    specialExtraFunc = (SpecialExtraFunc) ptr;
+    break;
   }
-  /* special key func */
-  if (DB_ConvertEntryData (section, atomKey, &ptr, keyTable) ) {
-    assert (ptr !=  NULL);
-    specialKeyFunc = (SpecialKeyFunc) ptr;
-  } else {
+  /* check for swap extras, buggy for < 2.10.2 */
+  if (! Version_AtLeast(VERSION_JOINT, &Ver_2_10_2)) {
+    if (specialExtraFunc == SpecialExtraSwapColor) {
+      /* add warning and reject level */
+      DB_CreateEntryString(warn,atomExtra, DB_DataToString(extraTable, specialExtraFunc));
+      return XBFalse;
+    }
+  }
+  /* Key has default */
+  switch (DB_ConvertEntryData (section, atomKey, &ptr, keyTable) ) {
+  case DCR_NoSuchEntry:
+    Dbg_Level("default for %s\n",  DB_SectionEntryString(section,atomKey));
     specialKeyFunc = SpecialKeyVoid;
+    break;
+  case DCR_Failure:
+    Dbg_Out("LEVEL: warning for %s\n", DB_SectionEntryString(section, atomKey));
+    specialKeyFunc = SpecialKeyVoid;
+    DB_CreateEntryString(warn,atomExtra, DB_DataToString(keyTable, specialKeyFunc));
+    break;
+  default:
+    assert (ptr != NULL);
+    specialKeyFunc = (SpecialKeyFunc) ptr;
+    break;
   }
+  return XBTrue;
+}
+/*
+ * config section [FUNC]
+ */
+void
+ConfigLevelFunc (const DBSection *section)
+{
   /* set extra info */
   if (specialExtraFunc == SpecialExtraAir) {
     AddExtraInfo ("Airpump as an extra");
@@ -808,8 +869,9 @@ ConfigLevelFunc (const DBSection *section)
     AddExtraInfo ("Stunner as an extra");
   } else if (specialExtraFunc == SpecialExtraTeleport) {
     AddExtraInfo ("Teleporter as an extra");
+  } else if (specialExtraFunc == SpecialExtraThrough) {
+    AddExtraInfo ("Through as an extra");
   }
-  return XBTrue;
 } /* ConfigLevelFunc */
 
 /*
@@ -820,6 +882,72 @@ HasSpecialBombs (void)
 {
   return (specialExtraFunc == SpecialExtraSpecialBomb);
 } /* HasSpecialBombs */
+/*
+ * return the name of the extra type
+ */
+const char*
+GetExtraNameFunc(void *type) {
+  int i;
+  if (type ==SpecialExtraVoid )
+    return seVoidName;
+  for (i = 0;extraTable[i].value!=NULL ; ++i)
+    if (extraTable[i].value == type)
+      return extraTable[i].key;
+  return seVoidName;
+}
+const char*
+GetExtraNameInt(int type) {
+  int i;
+  if (type ==0 )
+    return seVoidName;
+  for (i = 0;extraTable[i].value!=NULL ; ++i)
+    if (i == type)
+      return extraTable[i].key;
+  return seVoidName;
+}
+/*
+ * return the name of the key type
+ */
+const char*
+GetKeyNameFunc(void *type) {
+  int i;
+  if (type ==SpecialKeyVoid )
+    return skVoidName;
+  for (i = 0;keyTable[i].value!=NULL ; ++i)
+    if (keyTable[i].value == type)
+      return keyTable[i].key;
+  return skVoidName;
+}
+const char*
+GetKeyNameInt(int type) {
+  int i;
+  if (type ==0 )
+    return skVoidName;
+  for (i = 0;keyTable[i].value!=NULL ; ++i)
+    if (i == type)
+      return keyTable[i].key;
+  return skVoidName;
+}
+
+/*
+ * return the number of the key type
+ */
+int
+GetNumberOfKeys( void  ) {
+  int i;
+  for (i = 0;keyTable[i].value!=NULL ; ++i);
+  return i;
+}
+
+/*
+ * return the number of the extra type
+ */
+int
+GetNumberOfExtras( void ) {
+  int i;
+  for (i = 0;extraTable[i].value!=NULL ; ++i);
+  return i;
+}
 
 /*
  * end of file func.c
