@@ -1,7 +1,7 @@
 /*
  * func.h - special ingame function (pointers)
  *
- * $Id: func.h,v 1.4 2004/06/30 16:06:00 iskywalker Exp $
+ * $Id: func.h,v 1.7 2004/11/29 14:44:49 lodott Exp $
  *
  * Program XBLAST 
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
@@ -39,8 +39,17 @@ extern  void SpecialExtraSlow (BMPlayer *ps);
 extern  void SpecialExtraStunOthers (BMPlayer *ps);
 extern  void SpecialExtraPoison (BMPlayer *ps);
 extern  void SpecialExtraGhost (BMPlayer *ps);
-extern XBBool ConfigLevelFunc (const DBSection *section);
+extern XBBool ParseLevelFunc (const DBSection *section, DBSection *);
+extern void ConfigLevelFunc (const DBSection *section);
 extern XBBool HasSpecialBombs (void);
+extern const char *GetKeyNameFunc(void *type);
+extern const char *GetExtraNameFunc(void *type);
+extern const char *GetKeyNameInt(int type);
+extern const char *GetExtraNameInt(int type);
+extern int GetNumberOfKeys( void  );
+extern int GetNumberOfExtras( void  );
+extern void SpecialExtraVoid (BMPlayer *ps);
+extern void SpecialKeyVoid (BMPlayer *ps);
 #endif
 /*
  * end of file func.h
