@@ -1,9 +1,9 @@
 /*
  * file game.c - play the game
  *
- * $Id: game.c,v 1.7 2004/06/26 03:31:39 iskywalker Exp $
+ * $Id: game.c,v 1.35 2005/02/26 18:10:35 lodott Exp $
  *
- * Program XBLAST 
+ * Program XBLAST
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -12,7 +12,7 @@
  * any later version
  *
  * This program is distributed in the hope that it will be entertaining,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
  * Public License for more details.
  *
@@ -50,15 +50,22 @@
 #ifdef DEBUG_RANDOM
 #endif
 
+#define TEAM_NOWINNER 255
+#define TEAM_LOCALASYNC 254
+
 /*
  * local variables
  */
+
 /* player input */
 static PlayerAction *actionTable[NUM_XBE];
 static XBBool        useStopKey[NUM_XBE];
+static BMPlayer     *localPlayer[NUM_XBE];
+
 static int           turnX[MAX_PLAYER];
 static int           turnY[MAX_PLAYER];
-static int           numofplayers;
+static int           numOfPlayers;
+
 /* sounds to play */
 static SND_Id levelSounds[] = {
   SND_BAD,
@@ -100,7 +107,14 @@ static SND_Id levelSounds[] = {
   SND_NEWCLOAK,
   SND_BOMBMORPH,
   SND_APPL,
+  SND_APPL2,
   SND_WON,
+  SND_STEP1,
+  SND_STEP2,
+  SND_STEP3,
+  SND_STEP4,
+  SND_STEP5,
+  SND_STEP6,
   SND_MAX
 };
 
@@ -138,56 +152,52 @@ static void
 InitPlayerInput (XBPlayerHost hostType, const CFGGamePlayers *gamePlayers, const CFGPlayer *cfgPlayer, PlayerAction *playerAction)
 {
   int           i;
-
+  XBEventCode   code;
+  XBBool stopkey;
   assert (NULL != gamePlayers);
   assert (NULL != cfgPlayer);
   /* clear data */
   memset (actionTable, 0, sizeof (actionTable));
   memset (useStopKey, 0, sizeof (useStopKey));
+  memset (localPlayer, 0, sizeof (localPlayer));
   /* set controls for each player */
   for (i = 0; i < gamePlayers->num; i ++) {
     assert (ATOM_INVALID != gamePlayers->player[i]);
-    /* connect player to event type */
-    switch (gamePlayers->control[i]) {
-    case XBPC_RightKeyboard: 
-      if (hostType == gamePlayers->host[i]) {
-	actionTable[XBE_KEYB_1] = &playerAction[i]; 
-	useStopKey[XBE_KEYB_1]  = cfgPlayer[i].misc.useStopKey;
+    /* local player? */
+    if (hostType == gamePlayers->host[i]) {
+      /* default stopkey value, joysticks override */
+      stopkey = cfgPlayer[i].misc.useStopKey;
+      /* assign event codes for local */
+      switch (gamePlayers->control[i]) {
+      case XBPC_RightKeyboard:
+	code = XBE_KEYB_1;
+	break;
+      case XBPC_LeftKeyboard:
+	code = XBE_KEYB_2;
+	break;
+      case XBPC_Joystick1:
+	code = XBE_JOYST_1;
+	stopkey = XBTrue;
+	break;
+      case XBPC_Joystick2:
+	code = XBE_JOYST_2;
+	stopkey = XBTrue;
+	break;
+      default:
+	fprintf(stderr, "player %u (local) with unsupported control %u\n", i, gamePlayers->control[i]);
+	code = XBE_NONE;
+	break;
       }
-      turnX[i] = cfgPlayer[i].misc.turnStepKeyboard * BASE_X;
-      turnY[i] = cfgPlayer[i].misc.turnStepKeyboard * BASE_Y;
-      break;
-
-    case XBPC_LeftKeyboard:  
-      if (hostType == gamePlayers->host[i]) {
-	actionTable[XBE_KEYB_2] = &playerAction[i]; 
-	useStopKey[XBE_KEYB_2]  = cfgPlayer[i].misc.useStopKey;
+      /* set values, if event code was assigned */
+      if (code != XBE_NONE) {
+	actionTable[code] = &playerAction[i];
+	useStopKey[code]  = stopkey;
+	localPlayer[code] = player_stat+i;
       }
-      turnX[i] = cfgPlayer[i].misc.turnStepKeyboard * BASE_X;
-      turnY[i] = cfgPlayer[i].misc.turnStepKeyboard * BASE_Y;
-      break;
-
-    case XBPC_Joystick1:     
-      if (hostType == gamePlayers->host[i]) {
-	actionTable[XBE_JOYST_1] = &playerAction[i];
-	useStopKey[XBE_JOYST_1]  = XBTrue;
-      }
-      turnX[i] = cfgPlayer[i].misc.turnStepJoystick * BASE_X;
-      turnY[i] = cfgPlayer[i].misc.turnStepJoystick * BASE_Y;
-      break;
-
-    case XBPC_Joystick2:     
-      if (hostType == gamePlayers->host[i]) {
-	actionTable[XBE_JOYST_2] = &playerAction[i]; 
-	useStopKey[XBE_JOYST_2]  = XBTrue;
-      }
-      turnX[i] = cfgPlayer[i].misc.turnStepJoystick * BASE_X;
-      turnY[i] = cfgPlayer[i].misc.turnStepJoystick * BASE_Y;
-      break;
-
-    default:                 
-      break;
     }
+    /* assign turn step values for all players */
+    turnX[i] = cfgPlayer[i].misc.turnStepKeyboard * BASE_X;
+    turnY[i] = cfgPlayer[i].misc.turnStepKeyboard * BASE_Y;
   }
 } /* InitPlayerInput */
 
@@ -198,23 +208,23 @@ void
 ClearPlayerAction (PlayerAction *playerAction)
 {
   int player;
-  
+
   for (player = 0; player < MAX_PLAYER; player ++) {
     playerAction[player].player  = player;
     playerAction[player].dir     = GoDefault;
     playerAction[player].bomb    = XBFalse;
     playerAction[player].special = XBFalse;
     playerAction[player].pause   = XBFalse;
+    playerAction[player].suicide = XBFalse;
     playerAction[player].abort   = ABORT_NONE;
-    /* Skywalker */
     playerAction[player].laola = XBFalse;
     playerAction[player].looser = XBFalse;
-    /* */
+    playerAction[player].bot = XBFalse;
   }
 } /* ClearPlayerAction */
 
 /*
- *
+ * update game display
  */
 void
 GameUpdateWindow (void)
@@ -238,70 +248,98 @@ GameUpdateWindow (void)
 } /* GameUpdateWindow */
 
 /*
- * Evaluate events for game
+ * evaluate events for game until wait event
  */
 static  XBBool
 GameEvent (XBEventCode waitCode, XBEventData *data)
 {
   XBEventCode code;
   BMPlayer         *ps;
-  int counter=0;
+  static unsigned int target=0, init=0;
 
   assert (data != NULL);
   /* wait for specific event */
+  if (init == 0) {
+    target = numOfPlayers;
+    init = 1;
+  }
+  GUI_SetMouseMode (XBTrue);
   while (waitCode != (code = GUI_WaitEvent (data) ) ) {
+    /* return if escape */
     if (code == XBE_XBLAST) {
       return XBTrue;
     }
-    if (NULL != actionTable[code]) {
-      switch (data->value) {
-	/* simple ones */
-      case XBGK_GO_UP:        actionTable[code]->dir     = GoUp;       	 break;
-      case XBGK_GO_DOWN:      actionTable[code]->dir     = GoDown;     	 break;
-      case XBGK_GO_LEFT:      actionTable[code]->dir     = GoLeft;     	 break;
-      case XBGK_GO_RIGHT:     actionTable[code]->dir     = GoRight;    	 break;
-      case XBGK_BOMB:         actionTable[code]->bomb    = XBTrue;     	 break;
-      case XBGK_SPECIAL:      actionTable[code]->special = XBTrue;     	 break;
-      case XBGK_PAUSE:        actionTable[code]->pause   = XBTrue;       break; 
-      case XBGK_ABORT:        actionTable[code]->abort   = ABORT_TRUE; 	 break;
-      case XBGK_ABORT_CANCEL: actionTable[code]->abort   = ABORT_CANCEL; break;
-	/* more complex ones */ 
-/* Skywalker */
-      case XBGK_LAOLA: actionTable[code]->laola = XBTrue;     	 
-	SetMessage("OOOlllAAaaooOOLllaa",XBFalse);break;
-      case XBGK_LOOSER:actionTable[code]->looser = XBTrue;     	 break;
-      case XBGK_BOT:	
-
-	for (ps = player_stat,counter=1; ps < player_stat + numofplayers; ps ++,counter++) {
-	  if (ps->local) {
-	    break;
-	  fprintf(stderr," bot activated \n");
-
+    ps = localPlayer[code];
+    /* test chat event */
+    if (! Chat_Event(code, *data) && NULL != ps) {
+      /* check for player action */
+      if (NULL != actionTable[code]) {
+	switch (data->value) {
+	case XBGK_GO_UP:
+	  Player_ActivateBot(ps, XBFalse);
+	  actionTable[code]->dir     = GoUp;
+	  break;
+	case XBGK_GO_DOWN:
+	  Player_ActivateBot(ps, XBFalse);
+	  actionTable[code]->dir     = GoDown;
+	  break;
+	case XBGK_GO_LEFT:
+	  Player_ActivateBot(ps, XBFalse);
+	  actionTable[code]->dir     = GoLeft;
+	  break;
+	case XBGK_GO_RIGHT:
+	  Player_ActivateBot(ps, XBFalse);
+	  actionTable[code]->dir     = GoRight;
+	  break;
+	case XBGK_BOMB:
+	  Player_ActivateBot(ps, XBFalse);
+	  actionTable[code]->bomb    = XBTrue;
+	  break;
+	case XBGK_SPECIAL:
+	  Player_ActivateBot(ps, XBFalse);
+	  actionTable[code]->special = XBTrue;
+	  break;
+	case XBGK_PAUSE:
+	  Player_ActivateBot(ps, XBFalse);
+	  actionTable[code]->pause   = XBTrue;
+	  break;
+	case XBGK_ABORT:
+	  Player_ActivateBot(ps, XBFalse);
+	  actionTable[code]->abort   = ABORT_TRUE;
+	  break;
+	case XBGK_ABORT_CANCEL:
+	  Player_ActivateBot(ps, XBFalse);
+	  actionTable[code]->abort   = ABORT_CANCEL;
+	  break;
+	case XBGK_LAOLA:
+	  Player_ActivateBot(ps, XBFalse);
+	  actionTable[code]->laola = XBTrue;
+	  SetMsgLaola(ps->id);
+	  break;
+	case XBGK_LOOSER:
+	  Player_ActivateBot(ps, XBFalse);
+	  actionTable[code]->looser = XBTrue;
+	  SetMsgLoser(ps->id);
+	  break;
+	case XBGK_BOT:
+	  Player_ActivateBot(ps,! ps->bot);
+	  break;
+	case XBGK_STOP_ALL:
+	  Player_ActivateBot(ps, XBFalse);
+	  if (useStopKey[code]) {
+	    actionTable[code]->dir = GoStop;
 	  }
-	}
-	if(	ps->bot == XBTrue){
-	  SetMessage("Bot Deactivated",XBFalse);
-	  fprintf(stderr," bot activated \n");
-	  ps->bot = XBFalse;     		  
-	}
-	else{
-	  ps->bot = XBTrue;     	
-	}
+	  break;
+	case XBGK_STOP_UP:
+	case XBGK_STOP_DOWN:
+	case XBGK_STOP_LEFT:
+	case XBGK_STOP_RIGHT:
+	  Player_ActivateBot(ps, XBFalse);
+	  if (! useStopKey[code]) {
+	    actionTable[code]->dir = GoStop;
+	  }
 	  break;
-    /* */
-      case XBGK_STOP_ALL: 
-	if (useStopKey[code]) {
-	  actionTable[code]->dir = GoStop;  
 	}
-	break;
-      case XBGK_STOP_UP: 
-      case XBGK_STOP_DOWN: 
-      case XBGK_STOP_LEFT: 
-      case XBGK_STOP_RIGHT: 
-	if (! useStopKey[code]) {
-	  actionTable[code]->dir = GoStop;  
-	}
-	break;
       }
     }
   }
@@ -334,8 +372,8 @@ PauseEvalAction (int numPlayer, PlayerAction *playerAction, int pauseStatus)
 
   for (player = 0; player < numPlayer; player ++) {
     /* check if player has deactivated pause mode */
-    if (player == pauseStatus && 
-	player_stat[player].lives > 0 && 
+    if (player == pauseStatus &&
+	player_stat[player].lives > 0 &&
 	playerAction[player].pause) {
       return XBTrue;
     }
@@ -345,45 +383,51 @@ PauseEvalAction (int numPlayer, PlayerAction *playerAction, int pauseStatus)
 
 /*
  * Evaluate player action in game
- */ 
+ */
 int
 GameEvalAction (int numPlayer, PlayerAction *playerAction)
 {
   int    player;
   int    result  = -1;
   XBBool checkAbort = XBFalse;
-  
+
   for (player=0; player < numPlayer; player++) {
     BMPlayer     *ps = player_stat + player;
     PlayerAction *pa = playerAction + player;
-    /* toggle pause mode */
-    if (ps->lives > 0 && 
-	pa->pause) {
-      result = player;
-    }
+    /* if player lives */
     if (ps->lives > 0) {
-    /* drop bomb if needed */
-    if (pa->bomb) {
-      DropBomb (ps, BMTdefault);
-    }
-    /* execute special key function */
-    if (pa->special) {
-      (*specialKeyFunc) (ps);
-    }
-    /* Skywalker */
-    if(pa->laola){
-      ps->laola=6;
- SetMessage("OOOlllAAaaooOOLllaa",XBFalse);
-    }
-    if(pa->bot){
-      ps->bot=XBTrue;
-    }
-    if(pa->looser){
-SetMessage("Hooo NOOoo !!",XBFalse);
-      ps->looser=10;
-    }
-    /* */
-    /* try to abort game if needed */
+      /* suicide, unless already dying */
+      if (pa->suicide && ! ps->dying) {
+	ps->dying = DEAD_TIME;
+	ps->lives = 1;
+      }
+      /* toggle pause mode */
+      if (pa->pause) {
+	result = player;
+      }
+      /* drop bomb if needed */
+      if (pa->bomb) {
+	DropBomb (ps, BMTdefault);
+      }
+      /* execute special key function */
+      if (pa->special) {
+	(*specialKeyFunc) (ps);
+      }
+      /* do laola */
+      if(pa->laola){
+	ps->laola=6;
+	SetMsgLaola(player);
+      }
+      /* activate bot */
+      if (pa->bot){
+	ps->bot=XBTrue;
+      }
+      /* do the loser */
+      if (pa->looser){
+	SetMsgLoser(player);
+	ps->looser=10;
+      }
+      /* try to abort game if needed */
       if (pa->abort == ABORT_CANCEL) {
 	if (ps->abort != ABORT_NONE) {
 	  SetMessage (p_string[player].abortcancel, XBFalse);
@@ -394,70 +438,70 @@ SetMessage("Hooo NOOoo !!",XBFalse);
 	ps->abort = ABORT_TRUE;
 	checkAbort = XBTrue;
       }
-    }
-    /* get direction */
-    if (pa->dir != GoDefault) {
-	/* reverse direction  if player is ill */
-      if (ps->illness == IllReverse) {
-	switch (pa->dir) {
-	case GoUp:    pa->dir = GoDown;  break;
-	case GoDown:  pa->dir = GoUp;    break;
-	case GoLeft:  pa->dir = GoRight; break;
-	case GoRight: pa->dir = GoLeft;  break;
-	default:      break;
-	}
-      } 
-      if (ps->illness == IllReverse2) {
-	switch (pa->dir) {
-	case GoDown:  pa->dir = GoLeft;    break;
-	case GoUp:    pa->dir = GoRight;   break;
-	case GoLeft:  pa->dir = GoUp;	   break;
-	case GoRight: pa->dir = GoDown;    break;
-	default:      break;
-	    }
+      /* get direction */
+      if (pa->dir != GoDefault) {
+	/* reverse direction  if needed */
+	if (ps->illness == IllReverse) {
+	  switch (pa->dir) {
+	  case GoUp:    pa->dir = GoDown;  break;
+	  case GoDown:  pa->dir = GoUp;    break;
+	  case GoLeft:  pa->dir = GoRight; break;
+	  case GoRight: pa->dir = GoLeft;  break;
+	  default:      break;
 	  }
-      /* set new course */
-      ps->d_soll = pa->dir;
-      /* look if player reverts course */
-      switch (ps->d_ist) {
-      case GoUp:
-	if ( ( ( GoRight == ps->d_soll || 
-		 GoLeft  == ps->d_soll ) &&
-	       ps->y % BLOCK_HEIGHT >= BLOCK_HEIGHT - turnY[player] ) ||
-	     GoDown == ps->d_soll) {
-	  ps->d_ist = GoDown;
 	}
-	break;	
-      case GoDown:
-	if ( ( ( GoRight == ps->d_soll || 
-		 GoLeft  == ps->d_soll ) &&
-	       ps->y % BLOCK_HEIGHT <= turnY[player]) ||
-	     GoUp == ps->d_soll) {
-	  ps->d_ist = GoUp;
+	/* reverse2 direction, if needed */
+	if (ps->illness == IllReverse2) {
+	  switch (pa->dir) {
+	  case GoDown:  pa->dir = GoLeft;    break;
+	  case GoUp:    pa->dir = GoRight;   break;
+	  case GoLeft:  pa->dir = GoUp;	   break;
+	  case GoRight: pa->dir = GoDown;    break;
+	  default:      break;
+	  }
 	}
-	break;	
-      case GoRight:
-	if ( ( ( GoUp   == ps->d_soll || 
-		 GoDown == ps->d_soll ) &&
-	       ps->x % BLOCK_WIDTH <= turnX[player]) ||
-	     GoLeft == ps->d_soll ) {
-	  ps->d_ist = GoLeft;
+	/* set new course */
+	ps->d_soll = pa->dir;
+	/* check if player changes course */
+	switch (ps->d_ist) {
+	case GoUp:
+	  if ( ( ( GoRight == ps->d_soll || GoLeft  == ps->d_soll ) &&
+		 ps->y % BLOCK_HEIGHT >= BLOCK_HEIGHT - turnY[player] ) ||
+	       GoDown == ps->d_soll) {
+	    ps->d_ist = GoDown;
+	  }
+	  break;
+	case GoDown:
+	  if ( ( ( GoRight == ps->d_soll ||
+		   GoLeft  == ps->d_soll ) &&
+		 ps->y % BLOCK_HEIGHT <= turnY[player]) ||
+	       GoUp == ps->d_soll) {
+	    ps->d_ist = GoUp;
+	  }
+	  break;
+	case GoRight:
+	  if ( ( ( GoUp   == ps->d_soll ||
+		   GoDown == ps->d_soll ) &&
+		 ps->x % BLOCK_WIDTH <= turnX[player]) ||
+	       GoLeft == ps->d_soll ) {
+	    ps->d_ist = GoLeft;
+	  }
+	  break;
+	case GoLeft:
+	  if ( ( ( GoUp   == ps->d_soll ||
+		   GoDown == ps->d_soll ) &&
+		 ps->x % BLOCK_WIDTH >= BLOCK_WIDTH - turnX[player] ) ||
+	       GoRight == ps->d_soll ) {
+	    ps->d_ist = GoRight;
+	  }
+	  break;
+	default:
+	  break;
 	}
-	break;	
-      case GoLeft:
-	if ( ( ( GoUp   == ps->d_soll || 
-		 GoDown == ps->d_soll ) &&
-	       ps->x % BLOCK_WIDTH >= BLOCK_WIDTH - turnX[player] ) ||
-	     GoRight == ps->d_soll ) {
-	  ps->d_ist = GoRight;
-	}
-	break;	
-      default:
-	break;
       }
     }
-    
   }
+  /* abort check if one player requested */
   if (checkAbort) {
     XBBool doAbort = XBTrue;
     for (player = 0; player < numPlayer; player ++) {
@@ -499,15 +543,14 @@ LevelBegin (const char *name)
   GUI_FlushPixmap (XBFalse);
 } /* LevelBegin */
 
-/* 
- * after one level 
+/*
+ * after one level
  */
-const char *  
+const char *
 LevelResult (int gameTime, int *lastTeam, int numPlayers, const DBRoot *level, XBBool store)
 {
   int         i,j;
   XBBool      won[MAX_PLAYER];
-  XBBool      activeTeams = XBFalse;
   XBBool      outOfTime   = XBFalse;
   XBBool      drawGame    = XBFalse;
   int         numWinners  = 0;
@@ -518,81 +561,88 @@ LevelResult (int gameTime, int *lastTeam, int numPlayers, const DBRoot *level, X
 
   /* sanity check */
   assert (lastTeam != NULL);
-  /* any player(s) left */
-  *lastTeam = MAX_PLAYER;
+  *lastTeam = TEAM_NOWINNER;
+  /* check each player */
   for (i = 0; i < numPlayers; i ++) {
-  if(!store){
+    /* let dying players die */
     if (player_stat[i].dying) {
+      player_stat[i].dying = 0;
       player_stat[i].lives --;
     }
-  }
-    /* any one still alive */
+    /* still alive, then local winner */
     won[i] = (player_stat[i].lives > 0);
+    Dbg_Game(" player %i, team %i -> %s\n",i,player_stat[i].team, won[i] ? "won" : "lost");
+    /* update winner counts */
     if (won[i]) {
-      activeTeams = XBTrue;
-      /* increment victories */
-      *lastTeam   = player_stat[i].team;
-      //KOEN
-      //player_stat[i].victories ++;
-    }
-    /* count number of players in winner team */
-      if (*lastTeam == player_stat[i].team) {
-	numWinners ++;
-      } else {
-	numLoosers ++;
-      }
-    }
-  /* Sync Sky */
-    for (i = 0; i < numPlayers; i ++) {
-      if(*lastTeam == player_stat[i].team) {
-	if(store){
-	player_stat[i].victories ++;
-	}
-	won[i]=XBTrue;
-      }
-    } 
-  /* determine message  */
-  msg="Test";
-  if (gameTime >= (GAME_TIME-1)) {
-    /* This game was out of time */
-    msg       = "Out of Time";
-    outOfTime = XBTrue;
-  } else if (! activeTeams) {
-    /* is it a draw game */
-    msg       = "Draw Game";
-    drawGame  = XBTrue;
-  } else {
-    assert (numWinners > 0);
-    /* one player/team has won */
-    if(numWinners==1) { // one player
-      for (i = 0; i < numPlayers; i ++) {
-	if(won[i]) {
-	  Dbg_Out("Only one player won: %i %s\n", i, p_string[i].name);
-	  msg = p_string[i].winlevel; 
+      numWinners++;
+      switch (*lastTeam) {
+      case TEAM_NOWINNER: *lastTeam = player_stat[i].team; break;
+      case TEAM_LOCALASYNC: break;
+      default:
+	if (*lastTeam != player_stat[i].team) {
+	  *lastTeam = TEAM_LOCALASYNC;
 	}
       }
     } else {
-      j=OtherRandomNumber(numWinners)+1;
-      Dbg_Out("random %i\n", j);
+      numLoosers ++;
+    }
+  }
+  Dbg_Game("num of winners = %i\n",numWinners);
+  /* determine message  */
+  msg="Test";
+  if (gameTime >= (GAME_TIME-1)) {
+    Dbg_Game("game timed out\n");
+    msg       = "Out of Time";
+    outOfTime = XBTrue;
+  } else if (*lastTeam == TEAM_NOWINNER) {
+    Dbg_Game("draw game\n");
+    msg       = "Draw Game";
+    drawGame  = XBTrue;
+  } else if (*lastTeam == TEAM_LOCALASYNC) {
+    Dbg_Game("game asynced locally\n");
+    msg       = "Local async";
+  } else {
+    assert (numWinners > 0);
+    /* update player victories, if store requested */
+    if (store) {
+      for (i = 0; i < numPlayers; i ++) {
+	if (*lastTeam == player_stat[i].team) {
+	  player_stat[i].victories ++;
+	}
+      }
+    }
+    Dbg_Game("winner team = %u\n", *lastTeam);
+    if (numWinners == 1) {
+      /* one player/team has won */
+      for (i = 0; i < numPlayers; i ++) {
+	if (won[i]) {
+	  Dbg_Game("Only player %i won: %s\n", i, p_string[i].name);
+	  msg = p_string[i].winlevel;
+	}
+      }
+    } else {
+      /* multiple winner, random win msg */
+      j = OtherRandomNumber(numWinners)+1;
       for (i = 0; j>0 && i < numPlayers; i ++) {
-	if(won[i]) {
+	if (won[i]) {
 	  j--;
-	  if(j==0) { 
-	    Dbg_Out("Win msg of %i\n", i);
-	    msg = p_string[i].winlevel; 
+	  if (j==0) {
+	    Dbg_Game("random win msg, player %i = %s\n", i,msg);
+	    msg = p_string[i].winlevel;
 	  }
 	}
-      }  
+      }
     }
     pointWinner = numLoosers / numWinners;
     pointLooser = -1;
   }
-  /* log result for draws and out of time */
+  /* store level statistics if requested */
   if (store) {
     /* log player result */
     for (i = 0; i < numPlayers; i ++) {
       StoreLevelStat (DB_Atom (level), GUI_StringToAtom (p_string[i].name), won[i], won[i] ? pointWinner : pointLooser);
     }
+    /* log result for draws and out of time */
     StoreLevelStat (DB_Atom (level), atomOutOfTime, outOfTime, 0.0);
     StoreLevelStat (DB_Atom (level), atomDrawGame,  drawGame,  0.0);
   }
@@ -601,10 +651,10 @@ LevelResult (int gameTime, int *lastTeam, int numPlayers, const DBRoot *level, X
 } /* LevelResult */
 
 #if 0
-/* 
- * after one level 
+/*
+ * after one level
  */
-void  
+void
 LevelEnd (int numPlayers, int lastTeam, const char *msg, XBBool timeOut)
 {
   BMPlayer         *ps;
@@ -646,7 +696,7 @@ LevelEnd (int numPlayers, int lastTeam, const char *msg, XBBool timeOut)
 	}
       }
       /* check for timeout */
-      if (timeOut && 
+      if (timeOut &&
 	  count > 1000*TIMEOUT_LEVEL_END / FRAME_TIME) {
 	done = XBTrue;
       }
@@ -654,7 +704,7 @@ LevelEnd (int numPlayers, int lastTeam, const char *msg, XBBool timeOut)
       count ++;
       break;
     case XBE_MENU:
-      if (eData.value == XBMK_SELECT || 
+      if (eData.value == XBMK_SELECT ||
 	  eData.value == XBMK_DEFAULT) {
 	done = XBTrue;
       }
@@ -664,7 +714,7 @@ LevelEnd (int numPlayers, int lastTeam, const char *msg, XBBool timeOut)
     case XBE_MOUSE_3:
       done = XBTrue;
       break;
-    default: 
+    default:
       break;
     }
   }
@@ -677,7 +727,7 @@ LevelEnd (int numPlayers, int lastTeam, const char *msg, XBBool timeOut)
 #endif
 
 /*
- *
+ * play a turn
  */
 void
 GameTurn (int gameTime, int numPlayer, int *numActive)
@@ -705,12 +755,10 @@ GameTurn (int gameTime, int numPlayer, int *numActive)
   IgniteBombs ();
   /* stun players hit by other bombs */
   StunPlayers (player_stat, numPlayer);
-
   /* check if and Virus or Junkie infects another player */
   InfectOtherPlayers(numActive);
-
   /*evil grail */
- DoEvilIll();
+  DoEvilIll();
   /* do junkie countdown  */
   DoJunkie ();
   /* check if player was hit by any explosions */
@@ -727,32 +775,38 @@ GameTurn (int gameTime, int numPlayer, int *numActive)
 } /* GameTurn */
 
 /*
- *
+ * return number of players in game
+ */
+int
+GetNumOfPlayers(){
+  return numOfPlayers;
+} /* GetNumOfPlayers */
+
+/*
+ * initialize game data
  */
 XBBool
-InitGame (XBPlayerHost hostType, CFGType cfgType, const CFGGame *cfgGame, PlayerAction *playerAction) 
+InitGame (XBPlayerHost hostType, CFGType cfgType, const CFGGame *cfgGame, PlayerAction *playerAction)
 {
   int       i;
   CFGPlayer cfgPlayer[MAX_PLAYER];
-  
   /* sanity check */
   assert (XBPH_None != hostType);
   assert (NULL      != cfgGame);
   assert (NULL      != playerAction);
-  numofplayers=cfgGame->players.num;
+  numOfPlayers=cfgGame->players.num;
   /* get player graphics */
   for (i = 0; i < cfgGame->players.num; i ++) {
-    if (! RetrievePlayer (cfgType, 
-			  cfgGame->players.player[i], 
-			  cfgGame->players.teamColor[i], 
+    if (! RetrievePlayer (cfgType,
+			  cfgGame->players.player[i],
+			  cfgGame->players.teamColor[i],
 			  &cfgPlayer[i]) ) {
       return XBFalse;
     }
   }
   /* check for demo recording */
   if (cfgGame->setup.recordDemo) {
-    //    DemoInitGame (XBPH_Local, cfgGame);
-    DemoInitGame (cfgType, cfgGame); // DEMOFIX
+    DemoInitGame (cfgType, cfgGame);
   }
   /* init player graphics */
   if (! InitPlayerSprites (cfgGame->players.num, cfgPlayer) ) {
@@ -773,14 +827,13 @@ InitGame (XBPlayerHost hostType, CFGType cfgType, const CFGGame *cfgGame, Player
 } /* InitGame */
 
 /*
- *
+ * clean up game data
  */
 void
 FinishGame (const CFGGame *cfgGame)
 {
   /* sanity check */
   assert (NULL != cfgGame);
-
   /* shutdown demo recorded if needed */
   if (cfgGame->setup.recordDemo) {
     DemoFinishGame ();
@@ -793,4 +846,4 @@ FinishGame (const CFGGame *cfgGame)
 
 /*
  * end of file game.c
- */ 
+ */
