@@ -1,9 +1,9 @@
 /*
  * file info.h - display info at level start
  *
- * $Id: info.h,v 1.3 2004/05/14 10:00:33 alfie Exp $
+ * $Id: info.h,v 1.6 2004/12/04 06:01:13 lodott Exp $
  *
- * Program XBLAST 
+ * Program XBLAST
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -12,7 +12,7 @@
  * any later version
  *
  * This program is distributed in the hope that it will be entertaining,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
  * Public License for more details.
  *
@@ -23,10 +23,14 @@
 #ifndef _INFO_H
 #define _INFO_H
 
+#include "xblast.h"
+#include "ini_file.h"
+
 /*
  * prototypes
- */ 
+ */
 extern void ResetInfo (void);
+extern XBBool ParseLevelInfo (const DBSection *section, DBSection *warn);
 extern void ClearInfo (void);
 extern void AddPlayerInfo (const char *fmt, ...);
 extern void AddLevelInfo (const char *fmt, ...);
@@ -34,7 +38,7 @@ extern void AddExtraInfo (const char *fmt, ...);
 extern const char **GetPlayerInfo (int *pNum);
 extern const char **GetLevelInfo (int *pNum);
 extern const char **GetExtraInfo (int *pNum);
-
+extern unsigned GetGameModeInfo();
 #endif
 /*
  * end of file info.h
