@@ -1,9 +1,9 @@
 /*
  * File intro.c - misc. intros and inbetween screens
  *
- * $Id: intro.c,v 1.4 2004/06/26 19:09:29 iskywalker Exp $
+ * $Id: intro.c,v 1.14 2005/02/26 18:42:09 lodott Exp $
  *
- * Program XBLAST 
+ * Program XBLAST
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -12,7 +12,7 @@
  * any later version
  *
  * This program is distributed in the hope that it will be entertaining,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
  * Public License for more details.
  *
@@ -33,7 +33,7 @@
 #include "level.h"
 #include "status.h"
 #include "random.h"
-
+#include "chat.h"
 #include "snd.h"
 /*
  * local constants
@@ -71,13 +71,15 @@ typedef struct  {
  * local variables
  */
 static BMPosition scorePos[MAX_PLAYER];
+#ifndef SMPF
 static BMPosition namePos[MAX_PLAYER];
+#endif
 /* audience */
 static int        numAudience = 0;
 static BMAudience audience[MAX_AUDIENCE];
 static const int  audienceSpriteAnime[] = {
-  SpriteStopDown, 
-  SpriteLooser, SpriteLooser1, SpriteLooser2, 
+  SpriteStopDown,
+  SpriteLooser, SpriteLooser1, SpriteLooser2,
   SpriteWinner, SpriteWinner2, SpriteWinner3,
   MAX_ANIME
 };
@@ -88,13 +90,12 @@ static int     winnerNum;
 static Sprite *winnerSprite[MAX_PLAYER];
 
 /*
- * 
+ * draw intro text box
  */
 static void
 DrawIntroTextBoxes (const IntroTextBox *box)
 {
   assert (box != NULL);
-
   while (box->text != NULL) {
     GUI_DrawTextbox (box->text, box->flags, &box->rect);
     box ++;
@@ -102,7 +103,7 @@ DrawIntroTextBoxes (const IntroTextBox *box)
 } /* DrawIntroTextBoxes */
 
 /*
- *
+ * fade screen
  */
 void
 DoFade (XBFadeMode fadeMode, int maxLine)
@@ -114,6 +115,7 @@ DoFade (XBFadeMode fadeMode, int maxLine)
   GUI_InitFade (fadeMode, maxLine);
   /* do it */
   while (GUI_DoFade ()) {
+    /* fade finished, wait for timer event */
     while (XBE_TIMER != GUI_WaitEvent (&eData) ) continue;
   }
   GUI_SetTimer (0, XBTrue);
@@ -123,8 +125,9 @@ DoFade (XBFadeMode fadeMode, int maxLine)
  * wait for user input, return XBFalse if user wants to quit
  */
 static XBBool
-DoWait (int sec, XBBool countdown, PFV func) 
+DoWait (int sec, XBBool countdown, PFV func)
 {
+  XBEventCode code;
   XBEventData data;
   int    count = sec * 20;
 
@@ -133,7 +136,8 @@ DoWait (int sec, XBBool countdown, PFV func)
   GUI_SetTimer (50, XBTrue);
   /* wait for events */
   for (;;) {
-    switch (GUI_WaitEvent (&data)) {
+    code = GUI_WaitEvent (&data);
+    switch (code) {
       /* timeout and countdown handling */
     case XBE_TIMER:
       /* exec user function */
@@ -168,7 +172,7 @@ DoWait (int sec, XBBool countdown, PFV func)
 	return XBTrue;
       case XBMK_ABORT:   /* escape */
 	return XBFalse;
-      default: 
+      default:
 	break;
       }
       break;
@@ -178,6 +182,8 @@ DoWait (int sec, XBBool countdown, PFV func)
     case XBE_MOUSE_3:
       return XBTrue;
     default:
+      /* check for chatting */
+      Chat_Event(code, data);
       break;
     }
   }
@@ -185,7 +191,7 @@ DoWait (int sec, XBBool countdown, PFV func)
 } /* DoWait */
 
 /*
- *
+ * show intro
  */
 void
 DoIntro (void)
@@ -201,7 +207,7 @@ DoIntro (void)
   rect.y = BLOCK_HEIGHT*5;
   rect.w = BLOCK_WIDTH*3;
   rect.h = BLOCK_HEIGHT;
-    
+
   /* clear pixmap with default bitmap */
   GUI_ClearPixmap ();
   /* standard init text */
@@ -217,12 +223,10 @@ DoIntro (void)
   GUI_SetTimer (FRAME_TIME, XBTrue);
   GUI_SetKeyboardMode (KB_MENU);
   GUI_SetMouseMode (XBTrue);
-
   for (count = 0; count < INTRO_LENGTH; count ++) {
     if (! (count % CHAR_ANIME) ) {
       SND_Play (SND_EXPL, (count/CHAR_ANIME)*3+1);
     }
-    
     /* draw growing poly */
     pfactor = (double)(count+1)/INTRO_LENGTH;
     x = (int) (0.5 + (7.5 - pfactor * 4.5) * BLOCK_WIDTH);
@@ -230,8 +234,8 @@ DoIntro (void)
     w = (int) (0.5 + pfactor *  9.0 * BLOCK_WIDTH);
     h = (int) (0.5 + pfactor * 12.0 * BLOCK_HEIGHT);
     lw = (int) (0.5 + pfactor * 8);
-    GUI_DrawPolygon (x, y, w, h, lw, pointx, SIZE_OF_X, XBTrue);
-    MarkMazeRect (x-lw/2, y-lw/2, w+lw, h+lw);
+       GUI_DrawPolygon (x, y, w, h, lw, pointx, SIZE_OF_X, XBTrue);
+	   MarkMazeRect (x-lw/2, y-lw/2, w+lw, h+lw);
 
     if (count < CHAR_ANIME) {
       CopyExplBlock ( 0, 5, blockB[count]);
@@ -270,7 +274,7 @@ DoIntro (void)
     /* update window from pixmap */
     GUI_FlushPixmap (XBTrue);
     /* clear the redraw map */
-    ClearRedrawMap ();
+      ClearRedrawMap ();
 
     /* check event */
     while (XBE_TIMER != (event = GUI_WaitEvent (&data) ) ) {
@@ -281,13 +285,14 @@ DoIntro (void)
       case XBE_MOUSE_3:
 	if (count < INTRO_LENGTH - 2) {
 	  count = INTRO_LENGTH - 3;
-	} 
+	}
 	break;
       default:
 	break;
       }
     }
   }
+  SND_Play (SND_INTRO,SOUND_MIDDLE_POSITION);
   /* wait for keystroke or timer */
   DoWait (TIMEOUT_GAME_INTRO, XBFalse, NULL);
   /* unload sounds */
@@ -300,10 +305,10 @@ DoIntro (void)
 } /* DoIntro */
 
 /*
- *
+ * redraw screen?
  */
 static void
-LoadUpdateWindow (void) 
+LoadUpdateWindow (void)
 {
   /* shuffle sprites and mark them */
   ShuffleAllSprites ();
@@ -319,7 +324,7 @@ LoadUpdateWindow (void)
   GUI_FlushPixmap (XBTrue);
   /* clear the redraw map */
   ClearRedrawMap();
-} /* ScoreUpdateWindow */
+} /* LoadUpdateWindow */
 
 /*
  * poll events
@@ -329,18 +334,18 @@ LoadPoll (void)
 {
   XBEventCode eCode;
   XBEventData eData;
-
   while (XBE_NONE != (eCode = (GUI_PeekEvent (&eData) ) ) ) {
+    /* check for escape */
     if (XBE_MENU   == eCode &&
 	XBMK_ABORT == eData.value) {
       return XBFalse;
     }
   }
   return XBTrue;
-} /* Poll */
+} /* LoadPoll */
 
 /*
- *
+ * show player sprites in intro
  */
 XBBool
 InitPlayerSprites (int numPlayers, const CFGPlayer *cfgPlayer)
@@ -400,9 +405,7 @@ InitPlayerSprites (int numPlayers, const CFGPlayer *cfgPlayer)
     }
   }
   /* successful ;-) */
-#ifdef DEBUG
   Dbg_Out ("%d sprites loaded in %lu msec\n", numPlayers, Dbg_FinishClock () );
-#endif
   ok = XBTrue;
 Finish:
   /* delete old sprites */
@@ -410,9 +413,9 @@ Finish:
     if (NULL != sprite[i]) {
       DeleteSprite (sprite[i]);
     }
-  }  
+  }
   FinishLevelGraphics ();
-  /* stop sound */ 
+  /* stop sound */
   SND_Stop (SND_WHIRL);
   SND_Unload (SND_WHIRL);
   /* that's all */
@@ -420,7 +423,7 @@ Finish:
 } /* LoadPlayerSprites */
 
 /*
- *
+ * show level intro
  */
 XBBool
 LevelIntro (int numPlayers, const DBRoot *level, int timeOut)
@@ -430,7 +433,6 @@ LevelIntro (int numPlayers, const DBRoot *level, int timeOut)
   int          numInfo;
   const char **info;
   static char  tmp[128];
-
   ResetStatusBar (player_stat, "Press Space or Return", XBFalse);
   /* draw player positions */
   for (player=0, ps = player_stat; player < numPlayers; player++, ps++) {
@@ -441,14 +443,12 @@ LevelIntro (int numPlayers, const DBRoot *level, int timeOut)
   /* Update Window */
   DrawMaze ();
   DrawAllSprites ();
-  /* Get Level info */
   /* draw level title und hint*/
   titleBox[1].text = GetLevelName (level);
   sprintf (tmp, "created by %s", GetLevelAuthor (level));
   titleBox[2].text = tmp;
   titleBox[3].text = GetLevelHint (level);
   DrawIntroTextBoxes (titleBox);
-
   /* player info */
   info = GetPlayerInfo (&numInfo);
   for (i = 0; i < numInfo; i ++) {
@@ -475,19 +475,22 @@ LevelIntro (int numPlayers, const DBRoot *level, int timeOut)
   GUI_FlushPixmap (XBFalse);
   /* clear the redraw map */
   ClearRedrawMap();
+  /* negative timeout, simulate keypress */
+  if (timeOut < 0) {
+    return XBTrue;
+  }
   /* now wait for select */
   Dbg_Out("Gonna wait %i seconds\n", timeOut);
-  return DoWait ((timeOut > 0) ? timeOut : 0, XBTrue, LoadUpdateWindow);
+  return DoWait (timeOut, XBTrue, LoadUpdateWindow);
 } /* LevelIntro */
 
 /*
- *
+ * show winner animation
  */
 static void
 LevelAnimateWinner (void)
 {
   int i;
-
   /* animate winner sprites */
   for (i = 0; i < winnerNum; i ++) {
     SetSpriteMode  (winnerSprite[i], SPM_MAPPED);
@@ -503,18 +506,16 @@ LevelAnimateWinner (void)
 } /* LevelAnimateWinner */
 
 /*
- *
+ * show level end
  */
 XBBool
-LevelEnd (int numPlayers, int lastTeam, const char *msg, XBBool timeOut)
+LevelEnd (int numPlayers, int lastTeam, const char *msg, int timeOut)
 {
   int i;
-
   int player;
   for (player = 0; player <numPlayers; player ++) {
-    ((player_stat[player].sprite)->player).player=player_stat[player].id; 
+    ((player_stat[player].sprite)->player).player=player_stat[player].id;
   }
-
   /* counter for winner animation */
   winnerCount = 0;
   /* which sprites do we animate */
@@ -531,14 +532,18 @@ LevelEnd (int numPlayers, int lastTeam, const char *msg, XBBool timeOut)
   if (lastTeam >= 0) {
     SND_Play (SND_WON, SOUND_MIDDLE_POSITION);
   }
-  /* wait for timout / keystroke */
+  /* negative timeout, just return */
+  if (timeOut < 0) {
+    return XBTrue;
+  }
+  /* zero timeout: wait space, otherwise space or timeout */
   return DoWait (timeOut ? TIMEOUT_LEVEL_END : 0, XBFalse, LevelAnimateWinner);
 } /* LevelEnd */
 
 /*
- *
+ * prepare scoreboard
  */
-void 
+void
 InitScoreBoard (int numPlayers, int numWins)
 {
   int i, j, x, y;
@@ -613,21 +618,23 @@ InitScoreBoard (int numPlayers, int numWins)
       audience[numAudience].yPos = y - OtherRandomNumber (BASE_Y) - 4*BLOCK_HEIGHT/3;
       audience[numAudience].team = OtherRandomNumber (MAX(NUM_DEFAULT_PLAYERS, numPlayers));
       numAudience ++;
-    } 
+    }
   }
-  /* SMPF circle test
-  for(i=0; i< 16; i++) {
+#if 0
+  /* SMPF circle test */
+  for (i=0; i< 16; i++) {
     audience[numAudience].xPos = (int) floor(3*PIXW*cos(M_PI+(2*M_PI * i / 16))/12+PIXW/2-BLOCK_WIDTH/2);
     audience[numAudience].yPos = (int) floor(3*PIXH*sin(M_PI+(2*M_PI * i / 16))/12+PIXH/2-BLOCK_HEIGHT/2);
     audience[numAudience].team = OtherRandomNumber (MAX(NUM_DEFAULT_PLAYERS, numPlayers));
     numAudience ++;
-    } */
+  }
+#endif
 } /* InitScoreBoard */
 
 /*
- *
+ * initialize winner screen
  */
-void 
+void
 InitWinner (int numPlayers)
 {
   int i, x, y;
@@ -643,7 +650,6 @@ InitWinner (int numPlayers)
   case 5:  xStart = 1; xStep = 4; break;
   default: xStart = 1; xStep = 3; break;
   }
-  /* fill map */
   /* audience */
   for (y = 1; y <= yAudience; y ++) {
     for (x = 0; x < MAZE_W; x ++) {
@@ -663,23 +669,13 @@ InitWinner (int numPlayers)
   }
   /* winner */
   for (x = MAZE_W/2-1; x < MAZE_W/2+2; x ++) {
-    /* draw big podests */
-    //mapScoreBoard[x][6] = 0;
-    //mapScoreBoard[x][7] = 0;
-    //mapScoreBoard[x][8] = 1;
-    /* winner position */
     scorePos[0].x = (PIXW - BLOCK_WIDTH)/2;
     scorePos[0].y = 4 * BLOCK_HEIGHT + 6 * BASE_Y;
   }
   /* other players */
   for (i = 1, x = xStart; i < numPlayers; i ++) {
-    /* draw small podest */
-    // mapScoreBoard[x][10] = 4;
-    // mapScoreBoard[x][11] = 5;
-    /* player position */
     scorePos[i].x =  x * BLOCK_WIDTH;;
     scorePos[i].y =  9 * BLOCK_HEIGHT + 4 * BASE_Y;
-    /* next one */
     x += xStep;
   }
   /* set audience positions and sprite */
@@ -691,26 +687,27 @@ InitWinner (int numPlayers)
       audience[numAudience].yPos = y - OtherRandomNumber (BASE_Y) - 4*BLOCK_HEIGHT/3;
       audience[numAudience].team = OtherRandomNumber (MAX(numPlayers,NUM_DEFAULT_PLAYERS));
       numAudience ++;
-    } 
+    }
   }
-  /* SMPF circle test
-  for(i=0; i< 16; i++) {
+  /* SMPF circle test */
+#if 0
+  for (i=0; i< 16; i++) {
     audience[numAudience].xPos = (int) floor(3*PIXW*cos(M_PI+(2*M_PI * i / 16))/12+PIXW/2-BLOCK_WIDTH/2);
     audience[numAudience].yPos = (int) floor(3*PIXH*sin(M_PI+(2*M_PI * i / 16))/12+PIXH/2+BLOCK_HEIGHT);
     audience[numAudience].team = OtherRandomNumber (MAX(NUM_DEFAULT_PLAYERS, numPlayers));
     numAudience ++;
-    } */
+  }
+#endif
 } /* InitScoreBoard */
 
 /*
- *
+ * show audience
  */
 static void
 ShowAudience (int numPlayers, int lastTeam)
 {
   int i;
   BMAudience *ptr;
-
   for (i = 0, ptr = audience; i < numAudience; i ++, ptr ++) {
     /* set animation */
     if (ptr->team < numPlayers) {
@@ -741,14 +738,13 @@ ShowAudience (int numPlayers, int lastTeam)
 } /* ShowAudience */
 
 /*
- *
+ * show laola in audience
  */
 static void
 ShowLaOla (void)
 {
   int i;
   BMAudience *ptr;
-
   for (i = 0, ptr = audience; i < numAudience; i ++, ptr ++) {
     /* set animation */
     ptr->data    = laOlaAnime;
@@ -760,14 +756,13 @@ ShowLaOla (void)
 } /* ShowLaOla */
 
 /*
- *
+ * animate audience
  */
 static void
 AnimateAudience (void)
 {
   int i;
   BMAudience *ptr;
-
   /* create audience sprites */
   for (i = 0, ptr = audience; i < numAudience; i ++, ptr ++) {
     /* set new animation frame */
@@ -777,17 +772,16 @@ AnimateAudience (void)
     if (ptr->phase >= ptr->numData) {
       ptr->phase = 0;
     }
-  }  
+  }
 } /* AnimateAudience */
 
 /*
- *
+ * remove audience sprites
  */
 static void
 HideAudience (void)
 {
   int i;
-
   for (i = 0; i < numAudience; i ++) {
     assert (audience[i].sprite != NULL);
     DeleteSprite (audience[i].sprite);
@@ -795,10 +789,10 @@ HideAudience (void)
 } /* HideAudience */
 
 /*
- *
+ * update score window
  */
 static void
-ScoreUpdateWindow (void) 
+ScoreUpdateWindow (void)
 {
   /* sprite animations */
   AnimateAudience ();
@@ -819,21 +813,20 @@ ScoreUpdateWindow (void)
 } /* ScoreUpdateWindow */
 
 /*
- *
+ * show the scoreboard after level
  */
 XBBool
-ShowScoreBoard (int lastTeam, int maxNumWins, int numPlayers, BMPlayer *playerStat, XBBool timeOut)
+ShowScoreBoard (int lastTeam, int maxNumWins, int numPlayers, BMPlayer *playerStat, int timeOut)
 {
   int       i, j, n=0;
   BMPlayer *ps;
   int       numTrophies = 0;
-  XBBool    result;
+  XBBool    result = XBTrue;
   Sprite   *trophy[MAX_PLAYER*MAX_VICTORIES];
 #ifndef SMPF
   Sprite   *name[MAX_PLAYER];
 #endif
   int      winningTeam = 0;
-
   /* load score board map */
   ResetInfo ();
   ConfigScoreGraphics (graphicsScoreBoard);
@@ -868,20 +861,19 @@ ShowScoreBoard (int lastTeam, int maxNumWins, int numPlayers, BMPlayer *playerSt
       trophy[numTrophies ++] = CreateBombSprite (BB_NORMAL, (6+j)*BLOCK_WIDTH, ps->y + BLOCK_HEIGHT, 0, SPM_MAPPED);
     }
     /* draw player names */
-    name[i] = CreateTextSprite (p_string[i].name, namePos[i].x, namePos[i].y, SCORE_NAME_WIDTH, SCORE_NAME_HEIGHT, 
+    name[i] = CreateTextSprite (p_string[i].name, namePos[i].x, namePos[i].y, SCORE_NAME_WIDTH, SCORE_NAME_HEIGHT,
 				FF_White|FF_Medium|FF_Boxed, SPM_MAPPED);
 #endif
   }
 #ifdef SMPF
   for (j = 0; j < n; j ++) {
-    trophy[numTrophies ++] = CreateBombSprite (BB_NORMAL,(j-n/2)*BLOCK_WIDTH+BLOCK_WIDTH*MAZE_W/2, 
+    trophy[numTrophies ++] = CreateBombSprite (BB_NORMAL,(j-n/2)*BLOCK_WIDTH+BLOCK_WIDTH*MAZE_W/2,
 					       BLOCK_HEIGHT * (MAZE_H-2)+BLOCK_HEIGHT/2, 0, SPM_MAPPED);
-  } 
+  }
 #endif
-
   /* now show audience */
   for (i = 0, ps = playerStat; i <  numPlayers; i ++, ps ++) {
-    if(ps->team == lastTeam) { winningTeam|=1<<i; }
+    if(ps->team == lastTeam) {      winningTeam|=1<<i; }
   }
   Dbg_Out("The players that won are: %i\n", winningTeam);
   ShowAudience (numPlayers, winningTeam);
@@ -893,8 +885,10 @@ ShowScoreBoard (int lastTeam, int maxNumWins, int numPlayers, BMPlayer *playerSt
   GUI_FlushPixmap (XBFalse);
   /* applause */
   SND_Play (SND_APPL, SOUND_MIDDLE_POSITION);
-  /* wait for select key to continue */
-  result = DoWait (timeOut ? TIMEOUT_SCOREBOARD : 0, XBTrue, ScoreUpdateWindow);
+  /* non-negative timeout, wait till keypress or timeout */
+  if (timeOut >= 0) {
+    result = DoWait (timeOut ? TIMEOUT_SCOREBOARD : 0, XBTrue, ScoreUpdateWindow);
+  }
   /* fade out screen */
   DoFade (XBFM_BLACK_OUT, PIXH);
   /* clean up */
@@ -911,12 +905,11 @@ ShowScoreBoard (int lastTeam, int maxNumWins, int numPlayers, BMPlayer *playerSt
   HideAudience ();
   FinishLevelGraphics ();
   ClearRedrawMap ();
-  
   return result;
 } /* ShowScoreBoard */
 
 /*
- *
+ * show winner after game
  */
 void
 ShowWinner (int lastTeam, int numPlayers, BMPlayer *playerStat)
@@ -930,43 +923,41 @@ ShowWinner (int lastTeam, int numPlayers, BMPlayer *playerStat)
   int nTeamPlayers[MAX_HOSTS];
   int teamStart[MAX_HOSTS];
   int nTeamPlayersSet[MAX_HOSTS];
-  int teams=0;
-
-  /* load score board map */
-  for(i=0; i < MAX_HOSTS; i++) {
-    nTeamPlayers[i]=0;
-    nTeamPlayersSet[i]=0;
+  int teams = 0;
+  /* clear team size arrays */
+  for (i=0; i < MAX_HOSTS; i++) {
+    nTeamPlayers[i] = 0;
+    nTeamPlayersSet[i] = 0;
   }
-
+  /* count teams and sizes */
   for (i=0, ps = playerStat; i < numPlayers; ps ++, i++) {
     nTeamPlayers[ps->team]++;
-    if(nTeamPlayers[ps->team]==1) {
+    if (nTeamPlayers[ps->team]==1) {
       teams++;
     }
   }
-  
-  if((numPlayers-nTeamPlayers[lastTeam])>=5) { 
-    xStep=2;
+  /* calculate step length */
+  if ( (numPlayers - nTeamPlayers[lastTeam]) >= 5) {
+    xStep = 2;
   } else {
-    xStep=3;
+    xStep = 3;
   }
-
+  /* output results */
   Dbg_Out("Team configuration:\n");
-  s=0;
-  for(i=0; i < MAX_HOSTS; i++) {
-    if(i== lastTeam) {
-      Dbg_Out("Winning team %i, %i players:\n",i,nTeamPlayers[i]);
+  s = 0;
+  for (i=0; i < MAX_HOSTS; i++) {
+    if (i == lastTeam) {
+      Dbg_Out("Winning team %i, %i players:\n", i, nTeamPlayers[i]);
     } else {
-      Dbg_Out("Loosing team %i, %i players:\n",i,nTeamPlayers[i]);
-      teamStart[i]=s;
-      if(nTeamPlayers[i]>0) {
-	s=s+2*nTeamPlayers[i]-1+xStep;
+      Dbg_Out("Loosing team %i, %i players:\n", i, nTeamPlayers[i]);
+      teamStart[i] = s;
+      if (nTeamPlayers[i] > 0) {
+	s = s + 2 * nTeamPlayers[i] -1 + xStep;
       }
     }
   }
-  s=s-3; // last team so no between space!
-
-  n=nTeamPlayers[lastTeam];
+  s = s-3;
+  n = nTeamPlayers[lastTeam];
   switch (n) {
   case 1:  xStart = 7; xStep = 2; break;
   case 2:  xStart = 6; xStep = 2; break;
@@ -975,18 +966,15 @@ ShowWinner (int lastTeam, int numPlayers, BMPlayer *playerStat)
   case 5:  xStart = 3; xStep = 2; break;
   default: xStart = 3; xStep = 1; break;
   }
- 
   /* winners */
-
 #ifndef SMPF
-  for(x=xStart-1;x<=xStart+xStep*(n-1)+1;x++) {
+  for (x = xStart - 1; x <= xStart + xStep * (n-1) + 1; x++) {
     /* winner position */
     mapScoreBoard[x][6] = 0;
     mapScoreBoard[x][7] = 0;
     mapScoreBoard[x][8] = 1;
   }
 #endif
-
   for (x = xStart, i=0; i<n; i++) {
     /* draw big podests */
     scorePos[i].x = x * BLOCK_WIDTH;
@@ -995,27 +983,26 @@ ShowWinner (int lastTeam, int numPlayers, BMPlayer *playerStat)
 #else
     scorePos[i].y = 4 * BLOCK_HEIGHT + 6 * BASE_Y;
 #endif
-    x+=xStep;
+    x += xStep;
   }
-
 #ifdef SMPF
-  for (i=0, j=0, ps = playerStat; i < numPlayers; ps ++, i++) {
-    if(ps->team!=lastTeam) {
+  for (i = 0, j = 0, ps = playerStat; i < numPlayers; ps ++, i++) {
+    if (ps->team != lastTeam) {
       scorePos[j+n].x = (int) floor(3*PIXW*cos(M_PI+(2*M_PI * j / (numPlayers-n)))/12+PIXW/2-BLOCK_WIDTH/2);
       scorePos[j+n].y = (int) floor(3*PIXH*sin(M_PI+(2*M_PI * j / (numPlayers-n)))/12+PIXH/2+BLOCK_HEIGHT);
       j++;
     }
   }
 #else
-  s=s/2;
-  for (i=0, j=0, ps = playerStat; i < numPlayers; ps ++, i++) {
-    if(ps->team!=lastTeam) {
-      x=7+teamStart[ps->team]-s+nTeamPlayersSet[ps->team]*2;
+  s = s/2;
+  for (i = 0, j = 0, ps = playerStat; i < numPlayers; ps ++, i++) {
+    if (ps->team != lastTeam) {
+      x = 7 + teamStart[ps->team] - s + nTeamPlayersSet[ps->team] * 2;
       Dbg_Out("x = %d\n",x);
       nTeamPlayersSet[ps->team]++;
       mapScoreBoard[x][10] = 4;
       mapScoreBoard[x][11] = 5;
-      if(nTeamPlayersSet[ps->team]<nTeamPlayers[ps->team]) {
+      if (nTeamPlayersSet[ps->team] < nTeamPlayers[ps->team]) {
 	mapScoreBoard[x+1][10] = 4;
 	mapScoreBoard[x+1][11] = 5;
       }
@@ -1025,16 +1012,13 @@ ShowWinner (int lastTeam, int numPlayers, BMPlayer *playerStat)
     }
   }
 #endif
-
   ResetInfo ();
   ConfigScoreGraphics (graphicsScoreBoard);
   ConfigScoreMap (mapScoreBoard);
-  /* draw maze in pximap */
+  /* draw maze in pixmap */
   DrawMaze ();
-      
-
   /* init */
-  for(i=0;i<256;i++) {
+  for (i = 0; i < 256; i++) {
     totalName[i]=0;
   }
   for (i = 0, j = 0, k = 0, ps = playerStat; i <  numPlayers; i ++, ps ++) {
@@ -1044,13 +1028,14 @@ ShowWinner (int lastTeam, int numPlayers, BMPlayer *playerStat)
       ps->y = scorePos[k].y;
       SetSpriteAnime (ps->sprite, SpriteBigWinner);
       /* show winner name */
-      if(k>0) { strcat(totalName," & "); }
+      if (k>0) {
+	strcat(totalName," & ");
+      }
       strcat(totalName,p_string[i].name);
       /* set winning message */
       msg = p_string[i].wingame;
       k++;
     } else {
-      //Dbg_Out("Draw one at %i %u\n", (j+n) , (scorePos[j+n].x)/BLOCK_WIDTH);
       ps->x = scorePos[j+n].x;
       ps->y = scorePos[j+n].y;
       if (ps->in_active) {
@@ -1059,7 +1044,7 @@ ShowWinner (int lastTeam, int numPlayers, BMPlayer *playerStat)
 	SetSpriteAnime (ps->sprite, SpriteDamagedDown);
       } else{
 	SetSpriteAnime (ps->sprite, SpriteLooser);
-      }    
+      }
       j ++;
     }
     /* draw player sprites */
@@ -1067,8 +1052,8 @@ ShowWinner (int lastTeam, int numPlayers, BMPlayer *playerStat)
     SetSpriteMode (ps->sprite, SPM_MAPPED);
   }
 #ifndef SMPF
-  name=CreateTextSprite (totalName, (30-(n*3))*BLOCK_WIDTH/4, 49*BLOCK_HEIGHT/6, n*3*BLOCK_WIDTH/2,  2*BLOCK_HEIGHT/3, 
-			        FF_White|FF_Medium|FF_Boxed, SPM_MAPPED);
+  name = CreateTextSprite (totalName, (30-(n*3))*BLOCK_WIDTH/4, 49*BLOCK_HEIGHT/6, n*3*BLOCK_WIDTH/2,  2*BLOCK_HEIGHT/3,
+				FF_White|FF_Medium|FF_Boxed, SPM_MAPPED);
 #endif
   /* display message */
   assert (msg != NULL);
