@@ -1,7 +1,7 @@
 /*
  * file introdat.c - animation and layouts for intro and inbetween screens
  *
- * $Id: introdat.c,v 1.4 2004/05/14 10:00:34 alfie Exp $
+ * $Id: introdat.c,v 1.5 2004/11/27 02:34:52 iskywalker Exp $
  *
  * Program XBLAST 
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
@@ -130,7 +130,7 @@ IntroTextBox titleBox [] = {
     { 5*BLOCK_WIDTH/2, 67*BLOCK_HEIGHT/6, 10*BLOCK_WIDTH,  2*BLOCK_HEIGHT/3,}
   }, 
   {
-    "chat with rado The lamest at http://irc.xblast-center.com/",
+    "visit http://xblast.sf.net/",
     FF_White | FF_Boxed | FF_Small,
     { 8*BLOCK_WIDTH/2, 73*BLOCK_HEIGHT/6, 7*BLOCK_WIDTH,  2*BLOCK_HEIGHT/3,}
   },
