@@ -1,9 +1,9 @@
 /*
  * file level.h - Setting up level given in database
  *
- * $Id: level.h,v 1.3 2004/05/14 10:00:35 alfie Exp $
+ * $Id: level.h,v 1.5 2004/12/04 06:01:13 lodott Exp $
  *
- * Program XBLAST 
+ * Program XBLAST
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -12,7 +12,7 @@
  * any later version
  *
  * This program is distributed in the hope that it will be entertaining,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
  * Public License for more details.
  *
@@ -31,12 +31,14 @@
 /*
  * global prototypes
  */
+extern XBBool ParseLevel (const DBRoot *level);
 extern XBBool ConfigLevel (const DBRoot *level);
 extern void FinishLevel (void);
 
 extern const char *GetLevelName   (const DBRoot *level);
 extern const char *GetLevelAuthor (const DBRoot *level);
 extern const char *GetLevelHint   (const DBRoot *level);
+extern unsigned GetWarningCount();
 
 #endif
 /*
