@@ -1,10 +1,10 @@
 /*
  * file map.c - handling the level tile map
  *
- * Program XBLAST 
+ * Program XBLAST
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
  *
- * $Id: map.c,v 1.7 2004/06/30 16:06:00 iskywalker Exp $
+ * $Id: map.c,v 1.29 2005/03/07 16:22:52 lodott Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published
@@ -12,7 +12,7 @@
  * any later version
  *
  * This program is distributed in the hope that it will be entertaining,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
  * Public License for more details.
  *
@@ -30,20 +30,21 @@
 #include "info.h"
 #include "str_util.h"
 #include "random.h"
+#include "version.h"
 
 /*
- * local types 
+ * local types
  */
 
 /* extra distribution */
-typedef enum {
-  DEnone = 0, 
-  DEsingle, 
-  DEall, 
-  DEspecial, 
-  DEget, 
+enum BMExtraDistribution {
+  DEnone = 0,
+  DEsingle,
+  DEall,
+  DEspecial,
+  DEget,
   DEdouble
-} BMExtraDistribution;
+} ;
 /* one maze column */
 typedef unsigned short col[MAZE_H];
 /* extra probabilities */
@@ -57,7 +58,7 @@ typedef struct {
 
 /*
  * local variables
- */ 
+ */
 static col  em1[MAZE_W];
 static col  em2[MAZE_W];
 static col *explMaze    = em1;
@@ -70,12 +71,21 @@ static unsigned explDirty;
 static unsigned explOldDirty;
 
 static unsigned redrawMaze[MAZE_H];
-static unsigned redrawStat[STAT_H];
+static unsigned redrawStat[STAT_H+1];
+static Sprite *bombSprites[MAZE_W][MAZE_H];
 
-static BMMapTile maze[MAZE_W][MAZE_H];
+static BMMapTile maze[MAZE_W+1][MAZE_H+1];
 static BMMapTile extra[MAZE_W][MAZE_H];
+static Sprite *bombSprite[MAZE_W][MAZE_H];
 
-static BMExtraDistribution distribExtras;
+static XBBool tilesInitialized = XBFalse;
+static BMBlockTile tile[MAX_BLOCK];
+/* Added by VVL (Chat) 12/11/99 */
+static unsigned redrawChat;
+static unsigned redrawTiles;
+static XBBool mapMode=XBFalse;
+
+static int distribExtras;
 
 static BMExtraProb extraProb;
 
@@ -84,9 +94,10 @@ static DBToInt distribExtraTable[] = {
   { "all",     (int) DEall },
   { "double",  (int) DEdouble },
   { "get",     (int) DEget },
+  { "none",    (int) DEnone },
   { "single",  (int) DEsingle },
   { "special", (int) DEspecial },
-  { NULL,      (int) DEnone },
+  { NULL, -1 },
 };
 
 /*
@@ -101,57 +112,104 @@ CompareTile (const BMBlockTile *a, const BMBlockTile *b)
 	   a->add == b->add);
 } /* CompareTile */
 
+void
+SetXBMapMode(XBBool mode){
+  mapMode=mode;
+}
+
+XBBool
+GetXBMapMode(void){
+  return mapMode;
+}
+
+
 /*
- * public function: ConfigLevelGraphics 
+ * parse level data for graphics
  */
-XBBool 
-ConfigLevelGraphics (const DBSection *section)
+XBBool
+ParseLevelGraphics (const DBSection *section, DBSection *warn)
 {
   int         i;
   const char *s;
   char      **argv[MAX_BLOCK];
   int         argc;
-  BMBlockTile tile[MAX_BLOCK];
 
-  assert (section != NULL);
+  /* check if section exists */
+  if (NULL == section) {
+    Dbg_Out("LEVEL: graphics section is missing!\n");
+    DB_CreateEntryString(warn,atomMissing,"true");
+    return XBFalse;
+  }
+  /* set tile names to NULL if called for first time */
+  if (! tilesInitialized) {
+    for (i=0; i< MAX_BLOCK; i++) {
+      tile[i].name = NULL;
+    }
+    tilesInitialized = XBTrue;
+  }
   /* parse each entry */
   for (i=0; i< MAX_BLOCK; i++) {
     if (! DB_GetEntryString (section, atomArrayBlock00[i], &s) ) {
+      Dbg_Out("LEVEL: critical failure, %s is missing\n", GUI_AtomToString(atomArrayBlock00[i]));
+      DB_CreateEntryString(warn,atomArrayBlock00[i],"missing");
       return XBFalse;
     }
     argv[i] = SplitString (s, &argc);
     assert (NULL != argv[i]);
+    if (NULL != tile[i].name) {
+      free((void *) tile[i].name);
+      tile[i].name=NULL;
+    }
     /* check block type */
     if (1 == argc) {
       /* simple rgb pixmap */
-      tile[i].name = argv[i][0];
+      tile[i].name = DupString(argv[i][0]);
       tile[i].fg   = COLOR_INVALID;
       tile[i].bg   = COLOR_INVALID;
       tile[i].add  = COLOR_INVALID;
     } else if (4 == argc) {
       /* layered ppm pixmap */
-      tile[i].name = argv[i][0];
+      tile[i].name = DupString(argv[i][0]);
       if (COLOR_INVALID == (tile[i].fg  = StringToColor (argv[i][1]) ) ||
 	  COLOR_INVALID == (tile[i].bg  = StringToColor (argv[i][2]) ) ||
 	  COLOR_INVALID == (tile[i].add = StringToColor (argv[i][3]) ) ) {
+	free ((char *) argv[i]);
+	Dbg_Out("LEVEL: critical failure, invalid value for %s\n", GUI_AtomToString(atomArrayBlock00[i]));
+	DB_CreateEntryString(warn,atomArrayBlock00[i],"invalid");
 	return XBFalse;
       }
     } else {
+      free ((char *) argv[i]);
+      Dbg_Out("LEVEL: critical failure, invalid value for %s\n", GUI_AtomToString(atomArrayBlock00[i]));
+      DB_CreateEntryString(warn,atomArrayBlock00[i],"invalid");
       return XBFalse;
     }
-  }  
+    free ((char *) argv[i]);
+  }
+  return XBTrue;
+}
+
+/*
+ * configure after parsing
+ */
+void
+ConfigLevelGraphics (const DBSection *section)
+{
+  int i;
+  assert (section != NULL);
   /* load blocks */
+  Dbg_Out("checking graphics\n");
   for (i=0; i< MAX_BLOCK; i++) {
     if (tile[i].fg != COLOR_INVALID) {
       GUI_LoadBlockCch (i, tile[i].name, tile[i].fg, tile[i].bg, tile[i].add);
     } else {
       GUI_LoadBlockRgb (i, tile[i].name);
     }
-  }  
+  }
   GUI_InitExplosionBlocks ();
   /* set level info */
   if (CompareTile (tile + BTFree, tile + BTBlock) ) {
-    AddLevelInfo ("Walls are invisble");
+    AddLevelInfo ("Walls are invisible");
   }
   if (CompareTile (tile + BTBomb, tile + BTRange) &&
       CompareTile (tile + BTBomb, tile + BTSick) &&
@@ -161,12 +219,8 @@ ConfigLevelGraphics (const DBSection *section)
   if (CompareTile (tile + BTExtra, tile + BTBlock) ) {
     AddLevelInfo ("Some walls are blastable");
   }
-  /* clean up */
-  for (i=0; i< MAX_BLOCK; i++) {
-    free ((char *) argv[i]);
-  }
+  Dbg_Out("checked graphics\n");
   /* that's all */
-  return XBTrue;
 } /* ConfigLevelGraphics */
 
 /*
@@ -190,14 +244,19 @@ ExtraProbString (int val)
   }
 } /* ExtraProbName */
 
-/* 
+/*
  * clean up graphics allocated by level
  */
-void 
+void
 FinishLevelGraphics (void)
 {
   int i;
-
+  for (i = 0; i< MAX_BLOCK; i++) {
+    if (NULL != tile[i].name) {
+      free((void *) tile[i].name);
+      tile[i].name = NULL;
+    }
+  }
   for (i = 0; i< MAX_TILE; i++) {
     GUI_FreeBlock (i);
   }
@@ -205,74 +264,120 @@ FinishLevelGraphics (void)
 } /* UnloadBlocks */
 
 /*
- * configure level map layout
+ * parse level data
  */
 XBBool
-ConfigLevelMap (const DBSection *section)
+ParseLevelMap (const DBSection *section, DBSection *warn)
 {
+  int prob,slowFlame;
   int x, y;
   const char *s;
-  int prob;
 
-  assert (section != NULL);
-
-  /* extra distribution */
-  if (! DB_ConvertEntryInt (section, atomExtraDistribution, (int *) &distribExtras, distribExtraTable) ) {
-    distribExtras = DEnone;
+  /* check if section exists */
+  if (NULL == section) {
+    Dbg_Out("LEVEL: map section is missing!\n");
+    DB_CreateEntryString(warn,atomMissing,"true");
+    return XBFalse;
   }
-  /* extra probalities */
+  /* ExtraDistribution has default */
+  switch (DB_ConvertEntryInt (section, atomExtraDistribution, (int *) &distribExtras, distribExtraTable) ) {
+  case DCR_NoSuchEntry:
+    Dbg_Level("default for %s\n",  DB_SectionEntryString(section,atomExtraDistribution));
+    distribExtras = DEnone;
+    break;
+  case DCR_Failure:
+    Dbg_Out("LEVEL: warning for %s\n", DB_SectionEntryString(section, atomExtraDistribution));
+    distribExtras = DEnone;
+    DB_CreateEntryString(warn,atomExtraDistribution,DB_IntToString(distribExtraTable,distribExtras));
+    break;
+  default:
+    break;
+  }
+  /* ProbBomb has default */
   if (DB_GetEntryInt (section, atomProbBomb, &prob) ) {
     extraProb.bomb = 64 * prob / 100;
   } else {
+    Dbg_Level("default for %s\n", DB_SectionEntryString(section,atomProbBomb));
     extraProb.bomb = 0;
   }
+  /* ProbRange has default */
   if (DB_GetEntryInt (section, atomProbRange, &prob) ) {
     extraProb.range = extraProb.bomb + 64 * prob / 100;
   } else {
+    Dbg_Level("default for %s\n", DB_SectionEntryString(section,atomProbRange));
     extraProb.range = extraProb.bomb;
   }
+  /* ProbVirus has default */
   if (DB_GetEntryInt (section, atomProbVirus, &prob) ) {
     extraProb.ill = extraProb.range + 64 * prob / 100;
   } else {
+    Dbg_Level("default for %s\n", DB_SectionEntryString(section,atomProbVirus));
     extraProb.ill = extraProb.range;
   }
+  /* ProbSpecial has default */
   if (DB_GetEntryInt (section, atomProbSpecial, &prob) ) {
     extraProb.invinc = extraProb.ill + 64 * prob / 100;
   } else {
+    Dbg_Level("default for %s\n", DB_SectionEntryString(section,atomProbSpecial));
     extraProb.invinc = extraProb.ill;
   }
+  /* ProbHidden has default */
   if (DB_GetEntryInt (section, atomProbHidden, &prob) ) {
     extraProb.evil = extraProb.invinc + 64 * prob / 100;
   } else {
+    Dbg_Level("default for %s\n", DB_SectionEntryString(section,atomProbHidden));
     extraProb.evil = extraProb.invinc;
   }
-
+  if (Version_AtLeast(VERSION_JOINT, &Ver_2_10_1)) {
+    /* SlowMotionBurst has default, since 2.10.1 */
+    if (DB_GetEntryInt (section,atomSlowFlame , &prob) ) {
+      slowFlame =prob;
+    } else {
+      Dbg_Level("default for %s\n", DB_SectionEntryString(section,atomSlowFlame));
+      slowFlame = 1;
+    }
+    SetSlowMotionBurst(slowFlame);
+  }
   /* the layout */
   for (y = 0; y < MAZE_H; y ++) {
     if (! DB_GetEntryString (section, atomArrayRow00[y], &s) ) {
+      Dbg_Out("LEVEL: critical failure, missing %s!\n", DB_SectionEntryString(section,atomArrayRow00[y]));
+      DB_CreateEntryString(warn,atomArrayRow00[y],"missing");
       return XBFalse;
     }
     for (x = 0; x < MAZE_W; x ++) {
       switch (s[x]) {
-      case '_': SetMazeBlock (x, y, BTFree);       break;
-      case 'B': SetMazeBlock (x, y, BTBlock);      break;
-      case 'R': SetMazeBlock (x, y, BTBlockRise);  break;
-      case 'X': SetMazeBlock (x, y, BTExtra);      break;
-      case 'b': SetMazeBlock (x, y, BTBomb);       break;
-      case 'r': SetMazeBlock (x, y, BTRange);      break;
-      case 's': SetMazeBlock (x, y, BTSick);       break;
-      case 'q': SetMazeBlock (x, y, BTSpecial);    break;
-      case 'v': SetMazeBlock (x, y, BTVoid);       break;
-      case 'e': SetMazeBlock (x, y, BTEvil);       break;
-      case 'V': SetMazeBlock (x, y, BTBackground); break;
-      default:  
+      case '_':
+      case 'B':
+      case 'R':
+      case 'X':
+      case 'b':
+      case 'r':
+      case 's':
+      case 'q':
+      case 'v':
+      case 'e':
+      case 'V': break;
+      default:
+	Dbg_Out("LEVEL: critical failure, invalid entry %s!\n", DB_SectionEntryString(section,atomArrayRow00[y]));
+	DB_CreateEntryString(warn,atomArrayRow00[y],"invalid");
 	return XBFalse;
       }
-      explMaze[x][y]      = 0;
-      oldExplMaze[x][y]  = 0;
-      redrawMaze[y]      &= ~(1<<x);
     }
   }
+  return XBTrue;
+}
+
+/*
+ * configure level map layout
+ */
+void
+ConfigLevelMap (const DBSection *section)
+{
+  int x,y;
+  const char *s;
+  XBBool okay;
+  assert (section != NULL);
   /* set level info */
   if (NULL != (s = ExtraProbString (extraProb.bomb) ) ) {
     AddExtraInfo ("Bomb extras are %s", s);
@@ -289,8 +394,31 @@ ConfigLevelMap (const DBSection *section)
   if (NULL != (s = ExtraProbString (extraProb.evil - extraProb.invinc) ) ) {
     AddExtraInfo ("Hidden bombs are %s", s);
   }
+  /* the layout */
+  for (y = 0; y < MAZE_H; y ++) {
+    okay = DB_GetEntryString (section, atomArrayRow00[y], &s);
+    assert (okay);
+    for (x = 0; x < MAZE_W; x ++) {
+      switch (s[x]) {
+      case '_': SetMazeBlock (x, y, BTFree);       break;
+      case 'B': SetMazeBlock (x, y, BTBlock);      break;
+      case 'R': SetMazeBlock (x, y, BTBlockRise);  break;
+      case 'X': SetMazeBlock (x, y, BTExtra);      break;
+      case 'b': SetMazeBlock (x, y, BTBomb);       break;
+      case 'r': SetMazeBlock (x, y, BTRange);      break;
+      case 's': SetMazeBlock (x, y, BTSick);       break;
+      case 'q': SetMazeBlock (x, y, BTSpecial);    break;
+      case 'v': SetMazeBlock (x, y, BTVoid);       break;
+      case 'e': SetMazeBlock (x, y, BTEvil);       break;
+      case 'V': SetMazeBlock (x, y, BTBackground); break;
+      default: break;
+      }
+      explMaze[x][y]      = 0;
+      oldExplMaze[x][y]  = 0;
+      redrawMaze[y]      &= ~(1<<x);
+    }
+  }
   /* that's all */
-  return XBTrue;
 } /* ConfigLevelMap */
 
 /*
@@ -333,10 +461,10 @@ ConfigScoreMap (const XBScoreMap data)
   }
 } /* ConfigScoreMap */
 
-/* 
+/*
  *
  */
-void 
+void
 DrawMaze (void)
 {
   int x, y;
@@ -347,21 +475,29 @@ DrawMaze (void)
       GUI_DrawBlock (x, y, maze[x][y]);
     }
   }
+  /* if(mapMode){
+  for (x = 0; x < MAZE_W; x ++) {
+    GUI_DrawBlock (x, MAZE_H, maze[x][MAZE_H]);
+
+  }
+
+
+  }*/
   GUI_FlushBlocks ();
 } /* DrawMaze */
 
-/* 
+/*
  * create list of rectangles to be redrawn
  */
-void 
+void
 SetRedrawRectangles (void)
 {
   int x,y;
 
   for (y = 0; y < MAZE_H; y ++) {
     if ((explDirty | explOldDirty) & (1<<y)) {
-      for (x = 0; x < MAZE_W; x ++) { 
-	if (redrawMaze[y] & (1<<x)) { 
+      for (x = 0; x < MAZE_W; x ++) {
+	if (redrawMaze[y] & (1<<x)) {
 	  GUI_AddMazeRectangle (x, y);
 	} else if (explMaze[x][y] != oldExplMaze[x][y])  {
 	  GUI_AddMazeRectangle (x, y);
@@ -369,51 +505,103 @@ SetRedrawRectangles (void)
 	}
       }
     } else if (redrawMaze[y]) {
-      for (x = 0; x < MAZE_W; x ++) { 
-	if (redrawMaze[y] & (1<<x)) { 
+      for (x = 0; x < MAZE_W; x ++) {
+	if (redrawMaze[y] & (1<<x)) {
 	  GUI_AddMazeRectangle (x, y);
-	} 
+	}
       }
     }
   }
-  
+
   for (y = 0; y < STAT_H; y ++) {
     if (redrawStat[y]) {
-      for (x = 0; x < STAT_W; x++) { 
+      for (x = 0; x < STAT_W; x++) {
 	if (redrawStat[y] & (1<<x)) {
 	  GUI_AddStatRectangle (x, y);
 	}
       }
     }
   }
+  if(redrawTiles){
+    for (x=0; x<STAT_W; x++) {
+      if (redrawTiles & (1<<x)) {
+	GUI_AddTilesRectangle(x, 0);
+      }
+    }
+  }
+  /* Added by VVL (Chat) 12/11/99 : Begin */
+  if (redrawTiles) {
+    for (x=0; x<STAT_W; x++) {
+      if (redrawTiles & (1<<x)) {
+	GUI_AddTilesRectangle(x, 0);
+      }
+    }
+  }
+  /* Added by VVL (Chat) 12/11/99 : Begin */
+  if (redrawChat) {
+    for (x=0; x<STAT_W; x++) {
+      if (redrawChat & (1<<x)) {
+	GUI_AddChatRectangle(x, STAT_H);
+      }
+    }
+  }
+  /* Added by VVL (Chat) 12/11/99 : End */
 } /* SetRedrawRectangles */
 
-/* 
+/*
  * mark region of tiles as to be redrawn
  */
-void 
+void
 MarkMaze (int x1, int y1, int x2, int y2)
 {
   int x, y;
 
 #ifdef SMPF
-  for (y = MAX (y1, 0); y <= MIN (y2, MAZE_H+2); y ++) {
+  for (y = MAX (y1, 0); y <= MIN (y2, MAZE_H+3); y ++) {
 #else
-  for (y = MAX (y1, 0); y <= MIN (y2, MAZE_H+1); y ++) {
+  for (y = MAX (y1, 0); y <= MIN (y2, MAZE_H+2); y ++) {
 #endif
     if (y < MAZE_H) {
       for (x = MAX (x1, 0); x <= MIN (x2, MAZE_W-1); x ++) {
 	redrawMaze[y] |= (1<<x);
       }
-    } else {
-      for (x = MAX (x1, 0); x <= MIN (x2, STAT_W - 1); x++) {
-	redrawStat[y-MAZE_H] |= (1<<x);
+    }
+#ifdef SMPF
+    /* Added by VVL (Chat) 12/11/99 : Begin */
+    else if (y==MAZE_H+3) {
+#else
+    else if (y==MAZE_H+2) {
+#endif
+      for (x=MAX(x1,0); x<=MIN(x2,STAT_W-1); x++) {
+	redrawChat |= (1<<x);
+	//	redrawStat[y-MAZE_H] |= (1<<x);
       }
+      if(MAZE_H==y){
+      for (x=MAX(x1,0); x<=MIN(x2,STAT_W-1); x++) {
+	redrawTiles |= (1<<x);
+      }
+      }
+
+      /* Added by VVL (Chat) 12/11/99 : End */
+    } else {/*
+      if(mapMode){
+	if (y==MAZE_H) {
+	  fprintf(stderr,"mariking map mode \n");
+	  for (x = MAX (x1, 0); x <= MIN (x2, STAT_W - 1); x++) {
+	    redrawTiles |= (1<<x);
+	  }
+	}
+      }
+      else{*/
+	for (x = MAX (x1, 0); x <= MIN (x2, STAT_W - 1); x++) {
+	  redrawStat[y-MAZE_H] |= (1<<x);
+	 }
+	//}
     }
   }
 } /* MarkMaze */
 
-/* 
+/*
  * mark single tile to be redrawn
  */
 void
@@ -426,8 +614,8 @@ MarkMazeTile (int x, int y)
   }
 } /* MarkMazeTile */
 
-/* 
- * mark rectangle (in pixel coordinates) to be redrawn 
+/*
+ * mark rectangle (in pixel coordinates) to be redrawn
  */
 void
 MarkMazeRect (int x, int y, int w, int h)
@@ -442,11 +630,11 @@ MarkMazeRect (int x, int y, int w, int h)
   }
 } /* MarkMazeRect */
 
-/* 
+/*
  *
  */
 XBBool
-SpriteMarked (const Sprite *spr) 
+SpriteMarked (const Sprite *spr)
 {
   int left, right, top, bottom;
   const BMRectangle *r;
@@ -479,7 +667,7 @@ SpriteMarked (const Sprite *spr)
 /*
  *
  */
-void 
+void
 ClearRedrawMap (void)
 {
   col *swap;
@@ -490,21 +678,30 @@ ClearRedrawMap (void)
 
   explOldDirty = explDirty;
   explDirty    = 0;
+  /* Added by VVL (Chat) 12/11/99 */
+  redrawChat=0;
+  redrawTiles=0;
 
   memset (explMaze, 0, MAZE_W*sizeof(col) );
   memset (redrawMaze, 0, sizeof(redrawMaze) );
   memset (redrawStat, 0, sizeof(redrawStat) );
 } /* ClearRedrawMap */
 
-/* 
+/*
  *
  */
-void 
+void
 UpdateMaze (void)
 {
   int x, y;
+  /*if(mapMode){
+    height=MAZE_H+1;
+  }
+  else{
+    height=MAZE_H;
+    }*/
 
-  for (y = 0; y < MAZE_H; y++) {
+  for (y = 0; y <MAZE_H ; y++) {
     if (redrawMaze[y]) {
       /* check if explosions are needed in this row */
       if (explDirty & (1<<y)) {
@@ -533,14 +730,14 @@ UpdateMaze (void)
 /*
  *
  */
-void 
+void
 UpdateExpl (void)
 {
   int x,y;
   for (y = 0; y < MAZE_H; y ++) {
     if (redrawMaze[y]) {
       for (x = 0; x < MAZE_W; x ++) {
-	if ( (redrawMaze[y] & (1<<x)) && 
+	if ( (redrawMaze[y] & (1<<x)) &&
 	     (explMaze[x][y] & 0x10) ) {
 	  GUI_DrawExplosionSprite (x, y, explMaze[x][y] & 0xf);
 	}
@@ -552,35 +749,35 @@ UpdateExpl (void)
 /*
  *
  */
-XBBool 
+XBBool
 CheckMaze (int x, int y)
 {
-  return (maze[x][y] == BTBlock || 
-	  maze[x][y] == BTBlockRise || 
-	  maze[x][y] == BTExtra || 
-	  maze[x][y] == BTExtraOpen  || 
-	  maze[x][y] == BTVoid || 
+  return (maze[x][y] == BTBlock ||
+	  maze[x][y] == BTBlockRise ||
+	  maze[x][y] == BTExtra ||
+	  maze[x][y] == BTExtraOpen  ||
+	  maze[x][y] == BTVoid ||
 	  maze[x][y] < 0);
 
 } /* CheckMaze */
 
-/* 
+/*
  *
  */
-XBBool 
+XBBool
 CheckMazeFree (int x, int y)
 {
-  return (maze[x][y] == BTFree || 
+  return (maze[x][y] == BTFree ||
 	  maze[x][y] == BTBurned );
 } /* CheckMazeFree */
 
-/* 
+/*
  *
  */
-XBBool 
+XBBool
 CheckMazeFree2 (int x, int y)
 {
-  return (maze[x][y] == BTFree || 
+  return (maze[x][y] == BTFree ||
 	  maze[x][y] == BTVoid || // koen
 	  maze[x][y] == BTBurned );
 } /* CheckMazeFree */
@@ -591,45 +788,44 @@ CheckMazeFree2 (int x, int y)
 XBBool
 CheckMazeWall (int x, int y)
 {
-  return (maze[x][y] == BTBlock || 
+  return (maze[x][y] == BTBlock ||
 	  maze[x][y] == BTBlockRise);
 } /* CheckMazeWall */
 
 /* public function check_maze_phantom_wall */
 
- /* Written by Amilhastre */ int 
+ /* Written by Amilhastre */ int
  /* Written by Amilhastre */ CheckMazePhantomWall (int x,
- /* Written by Amilhastre */ 		 int y)
+ /* Written by Amilhastre */		 int y)
  /* Written by Amilhastre */ {
  /* Written by Amilhastre */   return((maze[x][y]==BTBlock) || (maze[x][y]==BTBlockRise) || (maze[x][y]==BTVoid));
  /* Written by Amilhastre */ }
 /* Skywalker go through walls*/
 int   CheckMazeGhost (int ghost,int x,
-  		 int y)
+		 int y)
   {
     if(!ghost) { return CheckMaze(x,y); }
-    return( (maze[x][y]==BTVoid )  /*
-				     original ghost map
+    return( (maze[x][y]==BTVoid )
 	    || (maze[x][y]<0)
-	    || (x<=0)  
+	    || (x<=0)
 	    || (y<=0)
 	    || (x>MAZE_H )
 	    || (y>MAZE_W-4 )
-				   */);
+				  );
   }
-XBBool 
+XBBool
 CheckMazeSolid (int x, int y)
 {
-  return (maze[x][y] == BTBlock || 
-	  maze[x][y] == BTBlockRise || 
-	  maze[x][y] == BTExtra || 
+  return (maze[x][y] == BTBlock ||
+	  maze[x][y] == BTBlockRise ||
+	  maze[x][y] == BTExtra ||
 	  maze[x][y] == BTExtraOpen);
 } /* CheckMazeSolid */
 
 /*
  *
  */
-XBBool 
+XBBool
 CheckMazeOpen (int x, int y)
 {
   return (maze[x][y] == BTFree ||
@@ -637,10 +833,10 @@ CheckMazeOpen (int x, int y)
 	  maze[x][y] == BTVoid);
 } /* CheckMazeOpen */
 
-/* 
+/*
  *
  */
-XBBool 
+XBBool
 CheckMazeExtra (int x, int y)
 {
   return (maze[x][y] == BTExtra);
@@ -691,8 +887,8 @@ BMMapTile
 CheckBonuses (int x,int y )
  {
    return( (maze[x][y]==BTBomb)
- 	 || (maze[x][y]==BTRange) 
- 	 || (maze[x][y]==BTSpecial)
+	 || (maze[x][y]==BTRange)
+	 || (maze[x][y]==BTSpecial)
 	   //  || (maze[x][y]==BTExtra)
 	   );
  }
@@ -704,10 +900,23 @@ CheckBonuses2 (int x,int y )
 	   );
  }
 
-/* 
- * 
+void
+DeleteAllMapBombSprites(void){
+  int x,y;
+  for (x = 0; x < MAZE_W; x++) {
+    for (y = 0; y < MAZE_H; y++) {
+      if(bombSprite[x][y]){
+	DeleteAllBombSprites();
+	bombSprite[x][y]=NULL;
+      }
+    }
+  }
+
+}
+/*
+ *
  */
-void 
+void
 SetMazeBlock (int x, int y, BMMapTile block)
 {
   int rnd;
@@ -720,6 +929,12 @@ SetMazeBlock (int x, int y, BMMapTile block)
 
   switch (block) {
   case BTExtra:
+    if(mapMode){
+      if(bombSprite[x][y]!=NULL){
+	DeleteSprite(bombSprite[x][y]);
+	bombSprite[x][y]=NULL;
+      }
+    }
     rnd = GameRandomNumber (64);
     if (rnd < extraProb.bomb) {
       extra[x][y] = BTBomb;
@@ -737,24 +952,65 @@ SetMazeBlock (int x, int y, BMMapTile block)
     maze[x][y] = block;
     break;
   case BTEvil:
+
+    if(mapMode){
+      if(x==1 && y==13){
+	maze[x][y]  = BTBlock;
+      }
+      else{
+
+      maze[x][y]  = BTFree;
+
+      }
+      if(bombSprite[x][y]==NULL){
+	bombSprite[x][y]= CreateBombSprite (BB_NORMAL,
+				      x*BLOCK_WIDTH, y*BLOCK_HEIGHT,
+				      0, SPM_MAPPED);
+	fprintf(stderr," creating sprite \n");
+	GUI_FlushBlocks ();
+      }
+
+    }
+    else{
     maze[x][y]  = BTFree;
     extra[x][y] = BTFree;
     NewEvilBomb (x, y);
+    }
     break;
   case BTExtraOpen:
+    if(mapMode){
+      if(bombSprite[x][y]!=NULL){
+	DeleteSprite(bombSprite[x][y]);
+	bombSprite[x][y]=NULL;
+      }
+    }
     maze[x][y]  = block;
     break;
   default:
+    if(mapMode){
+      if(bombSprites[x][y]){
+      DeleteSprite( bombSprites[x][y]);
+      bombSprites[x][y]= NULL;
+      }
+    }
     maze[x][y]  = block;
     extra[x][y] = BTFree;
     break;
   }
 } /* SetMazeBlock */
+/*
+ *
+ */
+BMMapTile
+GetMazeBlock (int x, int y){
 
+  return  maze[x][y];
+
+}
 /*
  * get extra on position (x,y)
  */
-int 
+int
 GetExtra (int invincible, int x, int y)
 {
   int extraBlock;
@@ -780,7 +1036,7 @@ GetExtra (int invincible, int x, int y)
   return extraBlock ;
 } /* GetExtra */
 
-/* 
+/*
  * check if special extra are distributed
  */
 XBBool
@@ -812,10 +1068,10 @@ PutExtras (int freeBlocks, BMPosition *distPos, BMMapTile tile, int numExtras)
   return freeBlocks;
 } /* PutExtras */
 
-/* 
+/*
  *
  */
-void 
+void
 DistributeExtras (int bombs, int range, int extras, int specials)
 {
   int x, y;
@@ -824,7 +1080,7 @@ DistributeExtras (int bombs, int range, int extras, int specials)
   int numExtras  = 0;
   static unsigned   distExtra[MAZE_H];
   static BMPosition distPos[MAZE_W*MAZE_H];
-    
+
   /* are there extras to distribute */
   if (bombs + range + extras + specials == 0)  {
     return;
@@ -834,7 +1090,7 @@ DistributeExtras (int bombs, int range, int extras, int specials)
   /* First check for free Blocks */
   for (x = 0; x < MAZE_W; x++) {
     for (y = 0; y < MAZE_H; y++) {
-      if (maze[x][y] == BTBurned || 
+      if (maze[x][y] == BTBurned ||
 	  maze[x][y] == BTFree) {
 	distExtra[y] |= (1 << x);
 	freeBlocks ++;
@@ -856,7 +1112,7 @@ DistributeExtras (int bombs, int range, int extras, int specials)
       }
     }
   }
-    
+
   /* Distribute special extras */
   /* use -1 for direct distribution on get extra */
   switch (distribExtras) {
@@ -883,7 +1139,7 @@ DistributeExtras (int bombs, int range, int extras, int specials)
   }
 } /* DistributeExtras */
 
-/* 
+/*
  *
  */
 void
@@ -896,7 +1152,7 @@ BlastExtraBlock (int x, int y)
     break;
     /* special extras are redistributed in DEget mode */
   case BTSpecial:
-    if (DEget == distribExtras || 
+    if (DEget == distribExtras ||
 	DEdouble == distribExtras ) {
       DistributeExtras (0, 0, -1, 0);
     }
@@ -914,7 +1170,7 @@ BlastExtraBlock (int x, int y)
 /*
  *
  */
-void 
+void
 CopyExplBlock (int x, int y, const int block[CHARH][CHARW])
 {
   int xp, yp;
@@ -931,9 +1187,3 @@ CopyExplBlock (int x, int y, const int block[CHARH][CHARW])
 /*
  * end of file map.c
  */
-
-
-
-
-
-
