@@ -1,9 +1,9 @@
 /*
  * file menu_game.c - menus for setting up game parameters
  *
- * $Id: menu_game.c,v 1.3 2004/05/14 10:00:35 alfie Exp $
- * 
- * Program XBLAST 
+ * $Id: menu_game.c,v 1.13 2005/03/08 16:34:33 iskywalker Exp $
+ *
+ * Program XBLAST
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -12,7 +12,7 @@
  * any later version
  *
  * This program is distributed in the hope that it will be entertaining,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
  * Public License for more details.
  *
@@ -31,10 +31,9 @@
 
 /*
  * local constants
- */ 
+ */
 #define ANIME_LENGTH 4
 
-#define DEBUG_TEAM
 /*
  * local variables
  */
@@ -42,6 +41,7 @@ static XBPlayerHost hostType;
 static XBMenuLevelPar mlp;
 /* game config */
 static CFGGameSetup gameSetup  = {
+  XBFalse,   /* respect the number of lives dictated by level */
   1,         /* number of lives */
   9,         /* number victories */
   30,        /* frame rate */
@@ -49,10 +49,12 @@ static CFGGameSetup gameSetup  = {
   XBTrue,    /* random level order */
   XBTrue,    /* random player positions */
   0,         /* level order */
-  15,        /* info wait time */  
+  15,        /* info wait time */
   XBFalse,   /* demo on DEMOFIX*/
   XBTM_None, /* team mode */
-  XBTrue,   /* XBCC not rated */
+  XBTrue,    /* XBCC not rated */
+  XBFalse,   /* bot */
+  2,         /* number of rec lives (valid if 1. XBTrue) */
 };
 static MENU_ID idButtonCentral;
 static MENU_ID idButtonStart;
@@ -70,10 +72,10 @@ static const XBPlayerControl _gameControl[NUM_LOCAL_PLAYER] = {
 static const char *playerControl[NUM_LOCAL_PLAYER] = {
   "Right Keyboard:",
   "Left Keyboard:",
-  "Joystick 1:", 
-  "Joystick 2:", 
-  "Joystick 3:", 
-  "Joystick 4:", 
+  "Joystick 1:",
+  "Joystick 2:",
+  "Joystick 3:",
+  "Joystick 4:",
 };
 /* local player selection */
 static XBComboEntryList *playerList = NULL;
@@ -115,6 +117,16 @@ static XBComboEntryList levelOrderList[] = { // LRF
   { "Time"   ,   3 , NULL, ATOM_INVALID, },
   { NULL     ,   0 , NULL, ATOM_INVALID, },
 };
+static XBComboEntryList musicList[] = { // LRF
+  { "None",    0, NULL, ATOM_INVALID, },
+  { "Song1",   SND_SNG1 , NULL, ATOM_INVALID, },
+  { "Song2",   SND_SNG2 , NULL, ATOM_INVALID, },
+  { "Song3",   SND_SNG3 , NULL, ATOM_INVALID, },
+  { "Song4",   SND_SNG4 , NULL, ATOM_INVALID, },
+  { "Song5",   SND_SNG5 , NULL, ATOM_INVALID, },
+  { "Song6",   SND_SNG6 , NULL, ATOM_INVALID, },
+  { NULL   ,   0 , NULL, ATOM_INVALID, },
+};
 static XBComboEntryList frameRateList[] = {
   { "10",  10, NULL, ATOM_INVALID, },
   { "12",  12, NULL, ATOM_INVALID, },
@@ -130,7 +142,6 @@ static XBComboEntryList frameRateList[] = {
   { "32",  32, NULL, ATOM_INVALID, },
   { "34",  35, NULL, ATOM_INVALID, },
   { "36",  36, NULL, ATOM_INVALID, },
-#ifdef DEBUG	      		 
   { "40",  40, NULL, ATOM_INVALID, },
   { "45",  45, NULL, ATOM_INVALID, },
   { "50",  50, NULL, ATOM_INVALID, },
@@ -138,7 +149,6 @@ static XBComboEntryList frameRateList[] = {
   { "72",  72, NULL, ATOM_INVALID, },
   { "85",  85, NULL, ATOM_INVALID, },
   { "100",100, NULL, ATOM_INVALID, },
-#endif		      		 
   { NULL,   0, NULL, ATOM_INVALID, },
 };
 #ifdef DEBUG_TEAM
@@ -149,7 +159,7 @@ static XBComboEntryList teamModeList[] = {
 };
 #endif
 static BMSpriteAnimation playerAnime[ANIME_LENGTH] = {
-    SpriteStopDown, SpriteWinner3,  SpriteWinner2, SpriteWinner, 
+    SpriteStopDown, SpriteWinner3,  SpriteWinner2, SpriteWinner,
 };
 
 XBBool autoCentral=XBFalse;
@@ -174,7 +184,7 @@ DeletePlayerConfigList (XBComboEntryList * list)
 } /* DeletePlayerConfigList */
 
 /*
- * Convert player config array to combo list 
+ * Convert player config array to combo list
  */
 static XBComboEntryList *
 CreatePlayerConfigList (void)
@@ -193,7 +203,7 @@ CreatePlayerConfigList (void)
     list[i].atom = atom;
     list[i].text = GetPlayerName (CT_Local, atom);
     gfx = calloc (1, sizeof (CFGPlayerGraphics) );
-    if (RetrievePlayerGraphics (CT_Local, atom, COLOR_INVALID, gfx) ) { 
+    if (RetrievePlayerGraphics (CT_Local, atom, COLOR_INVALID, gfx) ) {
       list[i].data = gfx;
     } else {
       free (gfx);
@@ -232,20 +242,16 @@ ButtonStartLocalGame (void *par)
 }  /* set game config */
 
 #ifdef DEBUG_TEAM
+
 /*
  *
  */
-
-/*
-
 static void
 AssignRandomTeams (XBTeamMode mode, int num, XBPlayerTeam *team)
 {
   int          i, j;
   XBPlayerTeam swap;
-
   assert (NULL != team);
-
   GUI_ErrorMessage ("in assign 0");
   switch (mode) {
   case XBTM_RandomTwo:
@@ -262,8 +268,7 @@ AssignRandomTeams (XBTeamMode mode, int num, XBPlayerTeam *team)
   default:
     return;
   }
-  shuffle teams 
-   KOEN
+  /* shuffle teams */
   GUI_ErrorMessage ("in assign 3");
   for (i = 0; i < num; i ++) {
     GUI_ErrorMessage ("in assign 3.5");
@@ -280,13 +285,11 @@ AssignRandomTeams (XBTeamMode mode, int num, XBPlayerTeam *team)
   for (i = 0; i < num; i ++) {
     GUI_ErrorMessage ("Player %i is of team %i", i, team[i]);
   }
-  }  AssignRandomTeams 
- */
+} /* AssignRandomTeams */
 #endif
 
-
-/* 
- * store game config 
+/*
+ * store game config
  */
 static void
 StoreGameConfig (XBAtom atom)
@@ -306,14 +309,15 @@ StoreGameConfig (XBAtom atom)
   }
   gamePlayers.num = j;
 #ifdef DEBUG_TEAM
-  /* set teams */
-  //  AssignRandomTeams (gameSetup.teamMode, gamePlayers.num, gamePlayers.team);
+  gameSetup.teamMode = XBTM_None;
+  AssignRandomTeams (gameSetup.teamMode, gamePlayers.num, gamePlayers.team);
 #else
   gameSetup.teamMode = XBTM_None;
 #endif
   /* set and store game config */
   StoreGameSetup   (CT_Local, atom, &gameSetup);
   StoreGamePlayers (CT_Local, atom, &gamePlayers);
+  StoreGameConstLocal (CT_Local, atom);
 } /* StoreGameConfig */
 
 /*
@@ -455,7 +459,7 @@ ButtonStartServer (void *par)
 } /* ButtonSelectLevels */
 
 /*
- * check if enough players are selected for playing 
+ * check if enough players are selected for playing
  */
 static void
 PollStartGame (void *par)
@@ -517,7 +521,7 @@ CreatePlayerItems (int x)
   playerList = CreatePlayerConfigList ();
   numJoy = GUI_NumJoysticks ();
   for (i = 0; i < 2 + numJoy; i ++) {
-    MenuAddCombo (x, MENU_ROW (i), 5*CELL_W, playerControl[i], 2*CELL_W, NULL, 
+    MenuAddCombo (x, MENU_ROW (i), 5*CELL_W, playerControl[i], 2*CELL_W, NULL,
 		  (void **) &localPlayerGraphics[i], &_gamePlayers[i], playerList);
     MenuAddPlayer (PLAYER_LEFT(i,numJoy+2), PLAYER_TOP, PLAYER_WIDTH, i, &localPlayerGraphics[i], -ANIME_LENGTH, playerAnime);
   }
@@ -538,14 +542,12 @@ CreateSetupItems (int x, CFGGameSetup *gameSetup, XBBool networkGame)
   MenuAddComboInt  (x, MENU_ROW (3), 5*CELL_W, "Level Order:",     2*CELL_W, &gameSetup->levelOrder, levelOrderList);
   MenuAddComboInt  (x, MENU_ROW (4), 5*CELL_W, "Info screen:",     2*CELL_W, &gameSetup->infoTime, waitInfoList);
   MenuAddComboBool (x, MENU_ROW (5), 5*CELL_W, "Record Demo:",     2*CELL_W, &gameSetup->recordDemo);
-  if(networkGame) {
-    MenuAddComboInt  (x, MENU_ROW (6), 5*CELL_W, "Team Mode:",       2*CELL_W, (int *) &gameSetup->teamMode, teamModeList);
+  MenuAddComboBool (x, MENU_ROW (6), 5*CELL_W, "Bot:", 2*CELL_W, &gameSetup->bot);
+  MenuAddComboBool (x-6*CELL_W, MENU_ROW (5), 5*CELL_W, "Recom. Lives:",     2*CELL_W, &gameSetup->ifRecLives);
+  MenuAddComboInt (x-6*CELL_W, MENU_ROW (6), 5*CELL_W, "Music:",     2*CELL_W, (int *) &gameSetup->Music, musicList);
+  if (networkGame) {
     MenuAddComboBool (x, MENU_ROW (7), 5*CELL_W, "Rated Game:",      2*CELL_W, &gameSetup->rated); // XBCC
   }
-  else{
-    MenuAddComboBool (x, MENU_ROW (6), 5*CELL_W, "Bot:", 2*CELL_W, &gameSetup->bot);
-
-  }
 } /* CreateSetupItems */
 
 static void
@@ -563,30 +565,30 @@ RetrieveGameConfig (XBAtom atom)
   memset (&_gamePlayers, 0, sizeof (_gamePlayers));
   for (i = 0; i < gamePlayers.num; i ++) {
     switch (gamePlayers.control[i]) {
-    case XBPC_RightKeyboard: 
-      _gamePlayers[0] = gamePlayers.player[i]; 
+    case XBPC_RightKeyboard:
+      _gamePlayers[0] = gamePlayers.player[i];
       break;
-    case XBPC_LeftKeyboard:  
-      _gamePlayers[1] = gamePlayers.player[i]; 
+    case XBPC_LeftKeyboard:
+      _gamePlayers[1] = gamePlayers.player[i];
       break;
-    case XBPC_Joystick1:     
+    case XBPC_Joystick1:
       if (numJoy >= 1) {
-	_gamePlayers[2] = gamePlayers.player[i]; 
+	_gamePlayers[2] = gamePlayers.player[i];
       }
       break;
     case XBPC_Joystick2:
       if (numJoy >= 2) {
-	_gamePlayers[3] = gamePlayers.player[i]; 
+	_gamePlayers[3] = gamePlayers.player[i];
       }
       break;
     case XBPC_Joystick3:
       if (numJoy >= 3) {
-	_gamePlayers[4] = gamePlayers.player[i]; 
+	_gamePlayers[4] = gamePlayers.player[i];
       }
       break;
     case XBPC_Joystick4:
       if (numJoy >= 4) {
-	_gamePlayers[5] = gamePlayers.player[i]; 
+	_gamePlayers[5] = gamePlayers.player[i];
       }
       break;
     default:
@@ -594,7 +596,7 @@ RetrieveGameConfig (XBAtom atom)
     }
   }
 } /* RetrieveGameConfig */
-  
+
 /*
  * create local game
  */
@@ -678,7 +680,7 @@ CreateStartNetGameMenu (void *par)
 /*
  * create the options menu
  */
-XBBool 
+XBBool
 CreateJoinNetGameMenu (void *par)
 {
   XBAtom *atom = par;
@@ -696,6 +698,8 @@ CreateJoinNetGameMenu (void *par)
   MenuAddComboBool (8*CELL_W, MENU_ROW (0), 5*CELL_W, "Record Demo:", 2*CELL_W, &gameSetup.recordDemo);
 /* bot */
   MenuAddComboBool (8*CELL_W, MENU_ROW (1), 5*CELL_W, "bot:", 2*CELL_W, &gameSetup.bot);
+  MenuAddComboBool (8*CELL_W, MENU_ROW (2), 5*CELL_W, "Beep at start:", 2*CELL_W, &gameSetup.beep);
+  MenuAddComboInt (8*CELL_W, MENU_ROW (3), 5*CELL_W, "Music:",     2*CELL_W, (int *) &gameSetup.Music, musicList);
   /* Buttons */
   MenuSetAbort    (MenuAddHButton ( 4 * CELL_W/2, MENU_BOTTOM, 5*CELL_W, "Back",        ButtonMainMenu,    NULL) );
   idButtonCentral= MenuAddHButton (17 * CELL_W/2, MENU_ROW(6), 5*CELL_W, "Search Central",ButtonSearchCentral, par);
@@ -716,9 +720,9 @@ CreateJoinNetGameMenu (void *par)
 void setAutoCentral(XBBool set) {
   autoCentral=set;
   setAutoCentral2(autoCentral);
-} 
+}
 
-XBBool 
+XBBool
 CreateCentralGameMenu (void *par)
 {
   return CreateCentralMenu (par);
@@ -727,7 +731,7 @@ CreateCentralGameMenu (void *par)
 /*
  *
  */
-XBPlayerHost 
+XBPlayerHost
 GetHostType (void)
 {
   return hostType;
@@ -736,7 +740,7 @@ GetHostType (void)
 /*
  *
  */
-void 
+void
 SetHostType (XBPlayerHost _hostType)
 {
   hostType = _hostType;
