@@ -1,9 +1,9 @@
 /*
  * file menu_player.c - edit player settings
  *
- * $Id: menu_player.c,v 1.3 2004/05/14 10:00:35 alfie Exp $
+ * $Id: menu_player.c,v 1.11 2005/02/25 14:44:01 iskywalker Exp $
  *
- * Program XBLAST 
+ * Program XBLAST
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -12,7 +12,7 @@
  * any later version
  *
  * This program is distributed in the hope that it will be entertaining,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
  * Public License for more details.
  *
@@ -49,7 +49,7 @@ static XBBool CreateRenamePlayerMenu (void *par);
  * local variables
  */
 static int firstPlayer = 0;
-static XBAtom atomPlayer[NUM_EDIT_PLAYERS];
+static XBAtom atomPlayers[NUM_EDIT_PLAYERS];
 static CFGPlayerGraphics gfxPlayer[NUM_EDIT_PLAYERS];
 static const CFGPlayerGraphics *pGfxPlayer[NUM_EDIT_PLAYERS] = {
   NULL, NULL, NULL, NULL,
@@ -70,6 +70,8 @@ static char msgWinLevel[MSG_LENGTH];
 static char msgWinGame[MSG_LENGTH];
 static char msgLoseLife[MSG_LENGTH];
 static char msgLoseLevel[MSG_LENGTH];
+static char msgLaola[MSG_LENGTH];
+static char msgLoser[MSG_LENGTH];
 static char msgGloat[MSG_LENGTH];
 static char msgWelcome[MSG_LENGTH];
 
@@ -104,6 +106,10 @@ static CFGPlayerID playerID;
 
 XBBool gotPID=XBFalse; // XBCC
 
+/*****************
+ * graphics menu *
+ *****************/
+
 /*
  * create color value list
  */
@@ -180,7 +186,7 @@ ButtonSaveGraphics (void *par)
 } /* ButtonSaveGraphics */
 
 /*
- *
+ * cancel button
  */
 static XBBool
 ButtonEditPlayerMenu (void *par)
@@ -196,19 +202,19 @@ ButtonEditPlayerMenu (void *par)
   return CreateEditPlayerMenu (par);
 } /* ButtonEditPlayerMenu */
 
-/*
- * save player graphics
- */
-static XBBool
-ButtonApplyColors (void *par)
-{
-  XBAtom *atom = par;
+/* /\* */
+/*  * apply colors */
+/*  *\/ */
+/* static XBBool */
+/* ButtonApplyColors (void *par) */
+/* { */
+/*   XBAtom *atom = par; */
 
-  assert (atom != NULL);
-  /* store it */
-  StorePlayerGraphics (CT_Local, *atom, gfxPlayer + 0);
-  return CreatePlayerColorsMenu (par);
-} /* ButtonSaveGraphics */
+/*   assert (atom != NULL); */
+/*   /\* store it *\/ */
+/*   StorePlayerGraphics (CT_Local, *atom, gfxPlayer + 0); */
+/*   return CreatePlayerColorsMenu (par); */
+/* } /\* ButtonApplyColors *\/ */
 
 /*
  * parse color name
@@ -232,7 +238,7 @@ CheckColorName (void *par)
 } /* CheckColorName */
 
 /*
- * edit player messages
+ * draw colors menu
  */
 static XBBool
 CreatePlayerColorsMenu (void *par)
@@ -259,41 +265,44 @@ CreatePlayerColorsMenu (void *par)
   /* build menu */
   MenuClear ();
   /* Title */
-  sprintf (title, "Colors for %s", GetPlayerName (CT_Local, *atom)); 
-  MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, title);
+  sprintf (title, "Colors for %s", GetPlayerName (CT_Local, *atom));
+  MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, title); 
+/* string for rgb value */
+  if (COLOR_INVALID != GUI_ParseColor ("Black")) {
+    MenuAddString (MSG_LEFT, MENU_ROW (5), MSG_WIDTH, "Colorname:", MSG_EDIT, colorName, sizeof (colorName));
+    MenuAddCyclic (CheckColorName, colorName);
+  } 
   /* shape */
   MenuAddComboAtom (5*CELL_W, MENU_ROW (0), 5*CELL_W, "Shape:", 2*CELL_W, &gfxPlayer[0].shape, shapeList);
-  /* colors windows */
-  MenuAddColor (COLOR_LEFT,  MENU_ROW (1), COLOR_WIDTH, "Helmet:",       &gfxPlayer[0].helmet,    &rgb);
-  MenuAddColor (COLOR_RIGHT, MENU_ROW (1), COLOR_WIDTH, "Arms & Legs:",  &gfxPlayer[0].armsLegs,  &rgb);
-  MenuAddColor (COLOR_LEFT,  MENU_ROW (2), COLOR_WIDTH, "Face:",         &gfxPlayer[0].face,      &rgb);
-  MenuAddColor (COLOR_RIGHT, MENU_ROW (2), COLOR_WIDTH, "Hands & Feet:", &gfxPlayer[0].handsFeet, &rgb);
-  MenuAddColor (COLOR_LEFT,  MENU_ROW (3), COLOR_WIDTH, "Body:",         &gfxPlayer[0].body,      &rgb);
-  MenuAddColor (COLOR_RIGHT, MENU_ROW (3), COLOR_WIDTH, "Backpack:",     &gfxPlayer[0].backpack,  &rgb);
   /* set rgb value */
   memset (&rgb, 0, sizeof (rgb));
   MenuAddComboInt ( 2*CELL_W, MENU_ROW (4), 3*CELL_W, "Red:",   3*CELL_W/2, &rgb.red,   colorValueList);
   MenuAddComboInt ( 6*CELL_W, MENU_ROW (4), 3*CELL_W, "Green:", 3*CELL_W/2, &rgb.green, colorValueList);
   MenuAddComboInt (10*CELL_W, MENU_ROW (4), 3*CELL_W, "Blue:",  3*CELL_W/2, &rgb.blue,  colorValueList);
-  /* string for rgb value */
-  if (COLOR_INVALID != GUI_ParseColor ("Black")) {
-    MenuAddString (MSG_LEFT, MENU_ROW (5), MSG_WIDTH, "Colorname:", MSG_EDIT, colorName, sizeof (colorName));
-    MenuAddCyclic (CheckColorName, colorName);
-  }
+  /* colors windows */
+  MenuSelect   (MenuAddColor (COLOR_LEFT,  MENU_ROW (1), COLOR_WIDTH, "Helmet:",       &gfxPlayer[0].helmet,    &rgb) );
+  MenuAddColor (COLOR_RIGHT, MENU_ROW (1), COLOR_WIDTH, "Arms & Legs:",  &gfxPlayer[0].armsLegs,  &rgb);
+  MenuAddColor (COLOR_LEFT,  MENU_ROW (2), COLOR_WIDTH, "Face:",         &gfxPlayer[0].face,      &rgb);
+  MenuAddColor (COLOR_RIGHT, MENU_ROW (2), COLOR_WIDTH, "Hands & Feet:", &gfxPlayer[0].handsFeet, &rgb);
+  MenuAddColor (COLOR_LEFT,  MENU_ROW (3), COLOR_WIDTH, "Body:",         &gfxPlayer[0].body,      &rgb);
+  MenuAddColor (COLOR_RIGHT, MENU_ROW (3), COLOR_WIDTH, "Backpack:",     &gfxPlayer[0].backpack,  &rgb);
   /* sprite */
   for (i = 0; i < 4; i ++) {
     pGfxPlayer[i] = gfxPlayer;
-    MenuAddPlayer (PLAYER_LEFT(i,4), PLAYER_TOP, PLAYER_WIDTH, i, pGfxPlayer + i, 1, allAnime + i);
+    MenuAddConfigPlayer (PLAYER_LEFT(i,4), PLAYER_TOP, PLAYER_WIDTH, i, pGfxPlayer + i, -1, allAnime + i,&rgb);
   }
   /* ok, cancel or cancel */
   MenuSetAbort   (MenuAddHButton ( 3*CELL_W/2, MENU_BOTTOM, 4*CELL_W, "Cancel", ButtonEditPlayerMenu, atom) );
-  MenuSetDefault (MenuAddHButton (11*CELL_W/2, MENU_BOTTOM, 4*CELL_W, "Apply",  ButtonApplyColors,    atom) );
-  (void)          MenuAddHButton (19*CELL_W/2, MENU_BOTTOM, 4*CELL_W, "OK",     ButtonSaveGraphics,   atom);
+  MenuSetDefault (MenuAddHButton (19*CELL_W/2, MENU_BOTTOM, 4*CELL_W, "OK",     ButtonSaveGraphics,   atom) );
   /* link items */
   MenuSetLinks ();
   /* that's all */
   return XBFalse;
-} /* CreatePlayerMessageMenu */
+} /* CreatePlayerColorsMenu */
+
+/*****************
+ * messages menu *
+ *****************/
 
 /*
  * save player messages
@@ -306,18 +315,20 @@ ButtonSaveMessages (void *par)
   assert (atom != NULL);
   /* get player messages */
   playerMsg.msgWinLevel  = (0 != msgWinLevel[0])  ? msgWinLevel  : NULL;
-  playerMsg.msgWinGame   = (0 != msgWinGame[0])   ? msgWinGame   : NULL;;
-  playerMsg.msgLoseLife  = (0 != msgLoseLife[0])  ? msgLoseLife  : NULL;;
-  playerMsg.msgLoseLevel = (0 != msgLoseLevel[0]) ? msgLoseLevel : NULL;;
-  playerMsg.msgGloat     = (0 != msgGloat[0])     ? msgGloat     : NULL;;
-  playerMsg.msgWelcome   = (0 != msgWelcome[0])   ? msgWelcome   : NULL;;
+  playerMsg.msgWinGame   = (0 != msgWinGame[0])   ? msgWinGame   : NULL;
+  playerMsg.msgLoseLife  = (0 != msgLoseLife[0])  ? msgLoseLife  : NULL;
+  playerMsg.msgLoseLevel = (0 != msgLoseLevel[0]) ? msgLoseLevel : NULL;
+  playerMsg.msgGloat     = (0 != msgGloat[0])     ? msgGloat     : NULL;
+  playerMsg.msgLaola     = (0 != msgLaola[0])     ? msgLaola     : NULL;
+  playerMsg.msgLoser     = (0 != msgLoser[0])     ? msgLoser     : NULL;
+  playerMsg.msgWelcome   = (0 != msgWelcome[0])   ? msgWelcome   : NULL;
   /* store it */
   StorePlayerMessages (CT_Local, *atom, &playerMsg);
   return CreateEditPlayerMenu (par);
 } /* ButtonSaveMessages */
 
 /*
- * edit player messages
+ * draw player messages menu
  */
 static XBBool
 CreatePlayerMessageMenu (void *par)
@@ -353,6 +364,16 @@ CreatePlayerMessageMenu (void *par)
   } else {
     msgGloat[0] = 0;
   }
+  if (NULL != playerMsg.msgLaola) {
+    strncpy (msgLaola, playerMsg.msgLaola, MSG_LENGTH);
+  } else {
+    msgLaola[0] = 0;
+  }
+  if (NULL != playerMsg.msgLoser) {
+    strncpy (msgLoser, playerMsg.msgLoser, MSG_LENGTH);
+  } else {
+    msgLoser[0] = 0;
+  }
   if (NULL != playerMsg.msgWelcome) {
     strncpy (msgWelcome, playerMsg.msgWelcome, MSG_LENGTH);
   } else {
@@ -361,7 +382,7 @@ CreatePlayerMessageMenu (void *par)
   /* build menu */
   MenuClear ();
   /* Title */
-  sprintf (title, "Messages for %s", GetPlayerName (CT_Local, *atom)); 
+  sprintf (title, "Messages for %s", GetPlayerName (CT_Local, *atom));
   MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, title);
   /* message windows */
   MenuAddString (MSG_LEFT, MSG_TOP,            MSG_WIDTH, "Welcome:",    MSG_EDIT, msgWelcome,   MSG_LENGTH);
@@ -370,6 +391,8 @@ CreatePlayerMessageMenu (void *par)
   MenuAddString (MSG_LEFT, MSG_TOP + 3*CELL_H, MSG_WIDTH, "Lose Life:",  MSG_EDIT, msgLoseLife,  MSG_LENGTH);
   MenuAddString (MSG_LEFT, MSG_TOP + 4*CELL_H, MSG_WIDTH, "Lose Level:", MSG_EDIT, msgLoseLevel, MSG_LENGTH);
   MenuAddString (MSG_LEFT, MSG_TOP + 5*CELL_H, MSG_WIDTH, "Gloat:",      MSG_EDIT, msgGloat,     MSG_LENGTH);
+  MenuAddString (MSG_LEFT, MSG_TOP + 6*CELL_H, MSG_WIDTH, "Laola:",      MSG_EDIT, msgLaola,     MSG_LENGTH);
+    MenuAddString (MSG_LEFT, MSG_TOP + 7*CELL_H, MSG_WIDTH, "Loser:",      MSG_EDIT, msgLoser,     MSG_LENGTH);
   /* sprite */
   if (RetrievePlayerGraphics (CT_Local, *atom, COLOR_INVALID, gfxPlayer) ) {
     pGfxPlayer[0] = gfxPlayer;
@@ -386,15 +409,17 @@ CreatePlayerMessageMenu (void *par)
   return XBFalse;
 } /* CreatePlayerMessageMenu */
 
+/************************
+ * control options menu *
+ ************************/
 
 /*
- * save player graphics
+ * save control options
  */
 static XBBool
 ButtonSaveMisc (void *par)
 {
   XBAtom *atom = par;
-
   assert (atom != NULL);
   /* store it */
   StorePlayerMisc (CT_Local, *atom, &playerMisc);
@@ -402,7 +427,7 @@ ButtonSaveMisc (void *par)
 } /* ButtonSaveGraphics */
 
 /*
- * create menu for misc. player options
+ * draw menu for misc. player options
  */
 static XBBool
 CreatePlayerMiscMenu (void *par)
@@ -416,7 +441,7 @@ CreatePlayerMiscMenu (void *par)
   /* build menu */
   MenuClear ();
   /* Title */
-  sprintf (title, "Other Options for %s", GetPlayerName (CT_Local, *atom)); 
+  sprintf (title, "Other Options for %s", GetPlayerName (CT_Local, *atom));
   MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, title);
   /* options */
   /* sprite */
@@ -428,9 +453,9 @@ CreatePlayerMiscMenu (void *par)
   MenuAddPlayer (PLAYER_LEFT(0,1), PLAYER_TOP, PLAYER_WIDTH, 0, &pGfxPlayer[0], 1, bigAnime);
   /* options */
   MenuAddComboBool (DLG_LEFT, DLG_ROW (0), DLG_WIDTH, "Use stop key:",                   2*CELL_W, &playerMisc.useStopKey);
-  MenuAddComboInt  (DLG_LEFT, DLG_ROW (1), DLG_WIDTH, "Backtrack for turns (keyboard):", 2*CELL_W, &playerMisc.turnStepKeyboard, 
+  MenuAddComboInt  (DLG_LEFT, DLG_ROW (1), DLG_WIDTH, "Backtrack for turns (keyboard):", 2*CELL_W, &playerMisc.turnStepKeyboard,
 		    turnStepTable);
-  MenuAddComboInt  (DLG_LEFT, DLG_ROW (2), DLG_WIDTH, "Backtrack for turns (joystick):", 2*CELL_W, &playerMisc.turnStepJoystick, 
+  MenuAddComboInt  (DLG_LEFT, DLG_ROW (2), DLG_WIDTH, "Backtrack for turns (joystick):", 2*CELL_W, &playerMisc.turnStepJoystick,
 		    turnStepTable);
   /* ok or cancel */
   MenuSetAbort   (MenuAddHButton ( 5*CELL_W/2, MENU_BOTTOM, 4*CELL_W, "Cancel", CreateEditPlayerMenu, atom) );
@@ -438,8 +463,12 @@ CreatePlayerMiscMenu (void *par)
   return XBFalse;
 } /* CreatePlayerMenu */
 
+/**********************
+ * delete player menu *
+ **********************/
+
 /*
- * callback for deleting player
+ * delete a player
  */
 static XBBool
 ButtonDeletePlayer (void *par)
@@ -452,7 +481,7 @@ ButtonDeletePlayer (void *par)
 } /* ButtonDeletePlayer */
 
 /*
- * delete a single player
+ * draw delete player menu
  */
 static XBBool
 CreateDeletePlayerMenu (void *par)
@@ -462,7 +491,7 @@ CreateDeletePlayerMenu (void *par)
   assert (atom != NULL);
   MenuClear ();
   /* Title */
-  sprintf (title, "Delete Player %s?", GetPlayerName (CT_Local, *atom)); 
+  sprintf (title, "Delete Player %s?", GetPlayerName (CT_Local, *atom));
   MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, title);
   /* edit options */
   MenuSetDefault (MenuAddHButton (MENU_LEFT, MENU_TOP,    MENU_WIDTH, "Yes, delete player!", ButtonDeletePlayer,   atom) );
@@ -481,8 +510,12 @@ CreateDeletePlayerMenu (void *par)
   return XBFalse;
 } /* CreateEditPlayerMenu */
 
+/**********************
+ * rename player menu *
+ **********************/
+
 /*
- * Callback for creating a new player 
+ * rename a player
  */
 static XBBool
 ButtonRenamePlayer (void *par)
@@ -502,145 +535,8 @@ ButtonRenamePlayer (void *par)
   }
 } /* ButtonNewPlayer */
 
-static XBBool
-ButtonAbortRegistration (void *par) {
-  User_Disconnect();
-  return CreatePlayerOptionsMenu (NULL);
-}
-
 /*
- *
- */
-static void
-PollForPID (void *par)
-{
-  XBAtom *atom = par;
-  int    PID;
-
-  PID=User_GetPID();
-  if(PID > -1000) {
-    if(!gotPID) {
-      Dbg_Out("Got PID = %i\n",PID);
-      switch(PID) {
-      case -3:
-	MenuAddLabel  (5*CELL_W/2, MENU_ROW(4), 10*CELL_W, "Error in central, press OK.");
-	break;
-      case -2:
-	MenuAddLabel  (5*CELL_W/2, MENU_ROW(4), 10*CELL_W, "Bad user ID, press OK.");
-	break;
-      case -1:
-	MenuAddLabel  (5*CELL_W/2, MENU_ROW(4), 10*CELL_W, "Wrong password, press OK.");
-	break;
-      default:
-	MenuAddLabel  (5*CELL_W/2, MENU_ROW(4), 10*CELL_W, "Registered, press OK to continue.");
-	break;
-      }
-      MenuSetDefault  (MenuAddHButton ( 8*CELL_W/2, MENU_ROW(7), 7*CELL_W, "OK", ButtonAbortRegistration, par) ); 
-      gotPID=XBTrue;
-      if(PID>0) {
-	if(RetrievePlayerID(CT_Local, *atom, &playerID)) {
-	  playerID.PID=PID;
-	  if (NULL != playerID.pass) {
-	    strncpy (newPass, playerID.pass, 16);
-	  } else {
-	    newPass[0] = 0;
-	  }
-	  playerID.pass  = (0 != newPass[0])  ? newPass  : NULL;
-	  StorePlayerID(CT_Local, *atom, &playerID);
-	  SavePlayerConfig();
-	}
-      }
-      User_SendDisconnect();
-    } 
-  }
-} /* PollForPID */
-
-/*
- * Callback for regestering a player XBCC
- */
-static XBBool
-ButtonRegisterPlayer (void *par)
-{
-  XBAtom *atom = par;
-  CFGCentralSetup central;
-
-  assert (atom != NULL);
-  /* store it */
-  playerID.pass  = (0 != newPass[0])  ? newPass  : NULL;
-  StorePlayerID (CT_Local, *atom, &playerID);
-  /* sprite */
-  if (RetrievePlayerGraphics (CT_Local, *atom, COLOR_INVALID, gfxPlayer) ) {
-    pGfxPlayer[0] = gfxPlayer;
-  } else {
-    pGfxPlayer[0] = NULL;
-  }
-
-  MenuClear ();
-  sprintf (title, "Registering %s", GetPlayerName (CT_Local, *atom)); 
-  MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, title);
-  /* connect to central */
-  RetrieveCentralSetup (&central);
-  if(User_Connect(&central)) {
-    MenuAddLabel  (5*CELL_W/2, MENU_ROW(3), 10*CELL_W, "Please wait...");
-    MenuSetAbort  (MenuAddHButton ( 8*CELL_W/2, MENU_BOTTOM, 7*CELL_W, "Abort registration", ButtonAbortRegistration, par) );
-    MenuAddPlayer (PLAYER_LEFT(0,1), PLAYER_TOP, PLAYER_WIDTH, 0, &pGfxPlayer[0], 1, bigAnime);
-    /* poll function */
-    gotPID=XBFalse;
-    User_SendRegisterPlayer(*atom); // XBCC send player config
-    MenuAddCyclic (PollForPID, par);
-  } else {
-    MenuAddLabel  (5*CELL_W/2, MENU_ROW(3), 10*CELL_W, "Unable to connect to central.");
-    MenuSetAbort   (MenuAddHButton ( 8*CELL_W/2, MENU_BOTTOM, 7*CELL_W, "Damn", CreatePlayerOptionsMenu, NULL) );
-    MenuAddPlayer (PLAYER_LEFT(0,1), PLAYER_TOP, PLAYER_WIDTH, 0, &pGfxPlayer[0], 1, renameAnime);
-  }
-  /* message windows */
-  /* link items */
-  MenuSetLinks ();
-
-  /* that's all */
-  return XBFalse;
-
-  return CreateEditPlayerMenu (par);
-} /* ButtonNewPlayer */
-/*
- * Callback for regestering a player XBCC
- */
-static XBBool
-ButtonUnregisterPlayer (void *par)
-{
-  XBAtom *atom = par;
-  /*  CFGCentralSetup central; DOES NOT SEND TO CENRAL */
-
-  assert (atom != NULL);
-  /* store it */
-  playerID.PID  = -1;
-  playerID.pass = NULL;
-  StorePlayerID (CT_Local, *atom, &playerID);
-  /* sprite */
-  if (RetrievePlayerGraphics (CT_Local, *atom, COLOR_INVALID, gfxPlayer) ) {
-    pGfxPlayer[0] = gfxPlayer;
-  } else {
-    pGfxPlayer[0] = NULL;
-  }
-
-  MenuClear ();
-  sprintf (title, "Unregistered"); 
-  MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, title);
-  /* connect to central */
-  MenuSetDefault  (MenuAddHButton ( 8*CELL_W/2, MENU_BOTTOM, 7*CELL_W, "OK", CreatePlayerOptionsMenu, NULL) );
-  MenuAddPlayer (PLAYER_LEFT(0,1), PLAYER_TOP, PLAYER_WIDTH, 0, &pGfxPlayer[0], 1, bigAnime);
-  /* message windows */
-  /* link items */
-  MenuSetLinks ();
-
-  /* that's all */
-  return XBFalse;
-
-  return CreateEditPlayerMenu (par);
-} /* ButtonNewPlayer */
-
-/*
- * menu for creating a new player
+ * menu for renaming a player
  */
 static XBBool
 CreateRenamePlayerMenu (void *par)
@@ -650,8 +546,8 @@ CreateRenamePlayerMenu (void *par)
   MenuClear ();
   /* Title */
   assert (NULL != atom);
-  sprintf (newName, "%s", GetPlayerName (CT_Local, *atom)); 
-  sprintf (title, "Rename Player %s", GetPlayerName (CT_Local, *atom)); 
+  sprintf (newName, "%s", GetPlayerName (CT_Local, *atom));
+  sprintf (title, "Rename Player %s", GetPlayerName (CT_Local, *atom));
   MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, title);
   /* message windows */
   MenuAddString (DLG_LEFT, DLG_TOP, DLG_WIDTH, "New Name:", 3*CELL_W, newName, sizeof (newName) );
@@ -671,6 +567,107 @@ CreateRenamePlayerMenu (void *par)
   return XBFalse;
 } /* CreateRenamePlayerMenu */
 
+/**************************
+ * unregister player menu *
+ **************************/
+
+/*
+ * abort the registration process
+ */
+static XBBool
+ButtonAbortRegistration (void *par) {
+  User_Disconnect();
+  return CreatePlayerOptionsMenu (NULL);
+} /* ButtonAbortRegistration */
+
+/*
+ * cyclic function checking if PID arrived
+ */
+static void
+PollForPID (void *par)
+{
+  XBAtom *atom = par;
+  int    PID;
+  PID = User_GetPID();
+  if (PID > -1000) {
+    if (!gotPID) {
+      switch(PID) {
+      case -3:
+	MenuAddLabel  (5*CELL_W/2, MENU_ROW(4), 10*CELL_W, "Error in central, press OK.");
+	break;
+      case -2:
+	MenuAddLabel  (5*CELL_W/2, MENU_ROW(4), 10*CELL_W, "Bad user ID, press OK.");
+	break;
+      case -1:
+	MenuAddLabel  (5*CELL_W/2, MENU_ROW(4), 10*CELL_W, "Wrong password, press OK.");
+	break;
+      default:
+	MenuAddLabel  (5*CELL_W/2, MENU_ROW(4), 10*CELL_W, "Registered, press OK to continue.");
+	break;
+      }
+      MenuSetDefault  (MenuAddHButton ( 8*CELL_W/2, MENU_ROW(7), 7*CELL_W, "OK", ButtonAbortRegistration, par) );
+      gotPID = XBTrue;
+      if(PID>0) {
+	if(RetrievePlayerID(CT_Local, *atom, &playerID)) {
+	  playerID.PID=PID;
+	  if (NULL != playerID.pass) {
+	    strncpy (newPass, playerID.pass, 16);
+	  } else {
+	    newPass[0] = 0;
+	  }
+	  playerID.pass  = (0 != newPass[0])  ? newPass  : NULL;
+	  StorePlayerID(CT_Local, *atom, &playerID);
+	  SavePlayerConfig();
+	}
+      }
+      User_SendDisconnect();
+    }
+  }
+} /* PollForPID */
+
+/*
+ * register a player
+ */
+static XBBool
+ButtonRegisterPlayer (void *par)
+{
+  XBAtom *atom = par;
+  CFGCentralSetup central;
+
+  assert (atom != NULL);
+  /* store it */
+  playerID.pass  = (0 != newPass[0])  ? newPass  : NULL;
+  StorePlayerID (CT_Local, *atom, &playerID);
+  /* sprite */
+  if (RetrievePlayerGraphics (CT_Local, *atom, COLOR_INVALID, gfxPlayer) ) {
+    pGfxPlayer[0] = gfxPlayer;
+  } else {
+    pGfxPlayer[0] = NULL;
+  }
+
+  MenuClear ();
+  sprintf (title, "Registering %s", GetPlayerName (CT_Local, *atom));
+  MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, title);
+  /* connect to central */
+  RetrieveCentralSetup (&central);
+  if (User_Connect(&central)) {
+    MenuAddLabel  (5*CELL_W/2, MENU_ROW(3), 10*CELL_W, "Please wait...");
+    MenuSetAbort  (MenuAddHButton ( 8*CELL_W/2, MENU_BOTTOM, 7*CELL_W, "Abort registration", ButtonAbortRegistration, par) );
+    MenuAddPlayer (PLAYER_LEFT(0,1), PLAYER_TOP, PLAYER_WIDTH, 0, &pGfxPlayer[0], 1, bigAnime);
+    /* poll function */
+    gotPID = XBFalse;
+    User_SendRegisterPlayer(*atom);
+    MenuAddCyclic (PollForPID, par);
+  } else {
+    MenuAddLabel  (5*CELL_W/2, MENU_ROW(3), 10*CELL_W, "Unable to connect to central.");
+    MenuSetAbort   (MenuAddHButton ( 8*CELL_W/2, MENU_BOTTOM, 7*CELL_W, "Damn", CreatePlayerOptionsMenu, NULL) );
+    MenuAddPlayer (PLAYER_LEFT(0,1), PLAYER_TOP, PLAYER_WIDTH, 0, &pGfxPlayer[0], 1, renameAnime);
+  }
+  /* link items */
+  MenuSetLinks ();
+  /* that's all */
+  return XBFalse;
+} /* ButtonRegisterPlayer */
 
 /*
  * menu for registering a player XBCC
@@ -691,7 +688,7 @@ CreateRegisterPlayerMenu (void *par)
   }
   /* build menu */
   MenuClear ();
-  sprintf (title, "Register Player %s", GetPlayerName (CT_Local, *atom)); 
+  sprintf (title, "Register Player %s", GetPlayerName (CT_Local, *atom));
   MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, title);
   /* message windows */
   MenuAddString (DLG_LEFT, DLG_TOP, DLG_WIDTH, "Password:", 3*CELL_W, newPass, sizeof(newPass) );
@@ -710,8 +707,50 @@ CreateRegisterPlayerMenu (void *par)
   /* that's all */
   return XBFalse;
 } /* CreateRenamePlayerMenu */
+
+/**************************
+ * unregister player menu *
+ **************************/
+
 /*
- * menu for registering a player XBCC
+ * button to unregister a player
+ */
+static XBBool
+ButtonUnregisterPlayer (void *par)
+{
+  XBAtom *atom = par;
+  /*  CFGCentralSetup central; DOES NOT SEND TO CENRAL */
+
+  assert (atom != NULL);
+  /* store it */
+  playerID.PID  = -1;
+  playerID.pass = NULL;
+  StorePlayerID (CT_Local, *atom, &playerID);
+  /* sprite */
+  if (RetrievePlayerGraphics (CT_Local, *atom, COLOR_INVALID, gfxPlayer) ) {
+    pGfxPlayer[0] = gfxPlayer;
+  } else {
+    pGfxPlayer[0] = NULL;
+  }
+
+  MenuClear ();
+  sprintf (title, "Unregistered (locally)");
+  MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, title);
+  /* connect to central */
+  MenuSetDefault  (MenuAddHButton ( 8*CELL_W/2, MENU_BOTTOM, 7*CELL_W, "OK", CreatePlayerOptionsMenu, NULL) );
+  MenuAddPlayer (PLAYER_LEFT(0,1), PLAYER_TOP, PLAYER_WIDTH, 0, &pGfxPlayer[0], 1, bigAnime);
+  /* message windows */
+  /* link items */
+  MenuSetLinks ();
+
+  /* that's all */
+  return XBFalse;
+
+  return CreateEditPlayerMenu (par);
+} /* ButtonUnregisterPlayer */
+
+/*
+ * draw menu for unregistering a player
  */
 static XBBool
 CreateUnregisterPlayerMenu (void *par)
@@ -729,7 +768,7 @@ CreateUnregisterPlayerMenu (void *par)
   }
   /* build menu */
   MenuClear ();
-  sprintf (title, "Unregister Player %s", GetPlayerName (CT_Local, *atom)); 
+  sprintf (title, "Unregister Player %s", GetPlayerName (CT_Local, *atom));
   MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, title);
   /* ok and cancel button */
   MenuSetAbort   (MenuAddHButton ( 7*CELL_W/2, MENU_BOTTOM, 4*CELL_W, "Cancel", CreatePlayerOptionsMenu, NULL) );
@@ -747,8 +786,12 @@ CreateUnregisterPlayerMenu (void *par)
   return XBFalse;
 } /* CreateRenamePlayerMenu */
 
+/********************
+ * edit player menu *
+ ********************/
+
 /*
- * edit a single player
+ * draw edit player menu
  */
 static XBBool
 CreateEditPlayerMenu (void *par)
@@ -758,7 +801,7 @@ CreateEditPlayerMenu (void *par)
   assert (atom != NULL);
   MenuClear ();
   /* Title */
-  sprintf (title, "Edit Player %s", GetPlayerName (CT_Local, *atom)); 
+  sprintf (title, "Edit Player %s", GetPlayerName (CT_Local, *atom));
   MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, title);
   /* edit options */
   MenuAddHButton (MENU_LEFT, MENU_ROW (0), MENU_WIDTH, "Edit Graphics",   CreatePlayerColorsMenu,  atom);
@@ -783,8 +826,12 @@ CreateEditPlayerMenu (void *par)
   return XBFalse;
 } /* CreateEditPlayerMenu */
 
+/***********************
+ * add new player menu *
+ ***********************/
+
 /*
- * Callback for creating a new player 
+ * button to create a new player
  */
 static XBBool
 ButtonNewPlayer (void *par)
@@ -801,7 +848,7 @@ ButtonNewPlayer (void *par)
 } /* ButtonNewPlayer */
 
 /*
- * menu for creating a new player
+ * draw new player menu
  */
 static XBBool
 CreateNewPlayerMenu (void *par)
@@ -822,10 +869,14 @@ CreateNewPlayerMenu (void *par)
   return XBFalse;
 } /* CreateNewPlayerMenu */
 
+/***********************
+ * player options menu *
+ ***********************/
+
 /*
- * get new player list
+ * get next player list
  */
-static XBBool 
+static XBBool
 ButtonPlayerForward (void *par)
 {
   int newFirst = firstPlayer + NUM_EDIT_PLAYERS;
@@ -836,9 +887,9 @@ ButtonPlayerForward (void *par)
 } /* ButtonPlayerForward */
 
 /*
- * get new player list
+ * get previous player list
  */
-static XBBool 
+static XBBool
 ButtonPlayerBackward (void *par)
 {
   int newFirst = firstPlayer - NUM_EDIT_PLAYERS;
@@ -851,7 +902,7 @@ ButtonPlayerBackward (void *par)
 } /* ButtonPlayerForward */
 
 /*
- * player main menu
+ * draw player main menu
  */
 XBBool
 CreatePlayerOptionsMenu (void *par)
@@ -865,25 +916,25 @@ CreatePlayerOptionsMenu (void *par)
   MenuClear ();
   /* Title */
   MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, "Player Options Menu");
-  /* Buttons */				   
+  /* Buttons */
   row = MENU_TOP;
   MenuAddHButton (MENU_LEFT, row, MENU_WIDTH, "Create New Player", CreateNewPlayerMenu, NULL);
-  row += CELL_H;  	     		   
+  row += CELL_H;
   /* List players */
   numPlayers = GetNumPlayerConfigs (CT_Local);
   for (player = firstPlayer, button = 0;
-       player < numPlayers && button < NUM_EDIT_PLAYERS; 
+       player < numPlayers && button < NUM_EDIT_PLAYERS;
        player ++, button ++) {
-    atomPlayer[button] = GetPlayerAtom (CT_Local, player);
+    atomPlayers[button] = GetPlayerAtom (CT_Local, player);
     odd = (1 == (button % 2));
     /* button */
-    MenuAddHButton (MENU_LEFT + odd * MENU_WIDTH/2, row, MENU_WIDTH/2, GetPlayerName (CT_Local, atomPlayer[button]), 
-		    CreateEditPlayerMenu, atomPlayer + button);
+    MenuAddHButton (MENU_LEFT + odd * MENU_WIDTH/2, row, MENU_WIDTH/2, GetPlayerName (CT_Local, atomPlayers[button]),
+		    CreateEditPlayerMenu, atomPlayers + button);
     if (odd) {
-      row += CELL_H;  	
+      row += CELL_H;
     }
     /* sprite */
-    if (RetrievePlayerGraphics (CT_Local, atomPlayer[button], COLOR_INVALID, gfxPlayer + button) ) {
+    if (RetrievePlayerGraphics (CT_Local, atomPlayers[button], COLOR_INVALID, gfxPlayer + button) ) {
       pGfxPlayer[button] = gfxPlayer + button;
     } else {
       pGfxPlayer[button] = NULL;
@@ -892,7 +943,7 @@ CreatePlayerOptionsMenu (void *par)
     MenuAddPlayer (PLAYER_LEFT(button,NUM_EDIT_PLAYERS), PLAYER_TOP, PLAYER_WIDTH, button, &pGfxPlayer[button], 1, playerAnime);
   }
   /* forward backward buttons */
-  row = MENU_TOP + (1 + NUM_EDIT_PLAYERS/2)* CELL_H;  	     		   
+  row = MENU_TOP + (1 + NUM_EDIT_PLAYERS/2)* CELL_H;
   id = MenuAddHButton (MENU_LEFT,                row, MENU_WIDTH/2, "Back", ButtonPlayerBackward, par);
   MenuSetActive (id, (firstPlayer > 0) );
   id = MenuAddHButton (MENU_LEFT + MENU_WIDTH/2, row, MENU_WIDTH/2, "More", ButtonPlayerForward,  par);
