@@ -1,9 +1,9 @@
 /*
  * file mi_tool.c - toolkit for xblast menus
  *
- * $Id: mi_tool.c,v 1.4 2004/05/14 10:00:35 alfie Exp $
+ * $Id: mi_tool.c,v 1.30 2005/03/07 14:43:29 lodott Exp $
  *
- * Program XBLAST 
+ * Program XBLAST
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -12,7 +12,7 @@
  * any later version
  *
  * This program is distributed in the hope that it will be entertaining,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
  * Public License for more details.
  *
@@ -21,7 +21,9 @@
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 #include "mi_tool.h"
+#include "atom.h"
 
+#include  "server.h"
 #include "map.h"
 #include "mi_button.h"
 #include "mi_color.h"
@@ -37,17 +39,22 @@
 #include "mi_string.h"
 #include "mi_tag.h"
 #include "mi_toggle.h"
+#include "menu_edit.h"
+#include "menu_game.h"
+#include "status.h"
 
 /*
  * local variables
  */
-static XBMenuItem *itemFirst 	   = NULL;
-static XBMenuItem *itemLast  	   = NULL;
-static XBMenuItem *itemFocus 	   = NULL;
+static XBMenuItem *itemFirst	   = NULL;
+static XBMenuItem *itemLast	   = NULL;
+static XBMenuItem *itemFocus	   = NULL;
 static XBBool      execFlag        = XBTrue;
 static XBBool      fadeFlag        = XBFalse;
 static XBMenuItem *defaultItem     = NULL;
 static XBMenuItem *abortItem       = NULL;
+static XBBool      mapMode         = 0;
+static int         setPressed      = 0;
 static XBComboEntryList yesNoTable[] = {
   { "no",  0, NULL},
   { "yes", 1, NULL},
@@ -81,43 +88,54 @@ DeleteMenuItem (XBMenuItem *item)
   free (item);
 } /* DeleteMenuItem */
 
+
+/*****************
+ * mouse buttons *
+ *****************/
+
 /*
- * clear total menu
+ * store press status of mouse
  */
 void
-MenuClear (void)
+SetPressed(XBBool mode)
 {
-  XBMenuItem *item_next;
-  while (itemFirst != NULL) {
-    item_next = itemFirst->next;
-    DeleteMenuItem (itemFirst);
-    itemFirst = item_next;
-  }
-  itemLast        = itemFocus = NULL;
-  fadeFlag    	  = XBTrue;
-  defaultItem     = NULL;
-  abortItem       = NULL;
-  MenuResetBase ();
-  MenuClearMap ();
-} /* MenuClear */
+  setPressed = mode;
+} /* SetPressed */
 
 /*
- * set directional links
+ * get press status of mouse
  */
-void 
-MenuSetLinks (void)
+XBBool
+GetPressed(void)
 {
-  XBMenuItem *ptr;
+  return setPressed;
+} /* GetPressed */
 
-  for (ptr = itemFirst; ptr != NULL; ptr = ptr->next) {
-    if (NULL != ptr->mouse) {
-      ptr->left  = MenuFindLeftItem (ptr);
-      ptr->right = MenuFindRightItem (ptr);
-      ptr->up    = MenuFindUpperItem (ptr);
-      ptr->down  = MenuFindLowerItem (ptr);
-    }
-  }
-} /* MenuSetLinks */
+/***************
+ * editor mode *
+ ***************/
+
+/*
+ * set editor mode
+ */
+void
+SetXBEditMapMode(XBBool mode)
+{
+  mapMode = mode;
+} /* SetXBEditMapMode */
+
+/*
+ * get editor mode
+ */
+XBBool
+GetXBEditMapMode(void)
+{
+  return mapMode;
+} /* GetXBEditMapMode */
+
+/***************
+ * local tools *
+ ***************/
 
 /*
  * find item by id
@@ -135,6 +153,69 @@ FindItem (MENU_ID id)
   return NULL;
 } /* FindItem */
 
+/**************
+ * menu tools *
+ **************/
+
+/*
+ * an menu item was selected (i.e Space was pressed)
+ */
+static void
+SelectItem (XBMenuItem *item)
+{
+  if (item != NULL &&
+      item->select != NULL) {
+    (*item->select) (item);
+  }
+} /* SelectItem */
+
+/*
+ * clear all menu items
+ */
+void
+MenuClear (void)
+{
+  XBMenuItem *item_next;
+  while (itemFirst != NULL) {
+    item_next = itemFirst->next;
+    DeleteMenuItem (itemFirst);
+    itemFirst = item_next;
+  }
+  itemLast        = itemFocus = NULL;
+  fadeFlag	  = XBTrue;
+  defaultItem     = NULL;
+  abortItem       = NULL;
+  MenuResetBase ();
+  MenuClearMap ();
+} /* MenuClear */
+
+/*
+ * set directional links between items for arrow keys
+ */
+void
+MenuSetLinks (void)
+{
+  XBMenuItem *ptr;
+
+  for (ptr = itemFirst; ptr != NULL; ptr = ptr->next) {
+    if (NULL != ptr->mouse) {
+      ptr->left  = MenuFindLeftItem (ptr);
+      ptr->right = MenuFindRightItem (ptr);
+      ptr->up    = MenuFindUpperItem (ptr);
+      ptr->down  = MenuFindLowerItem (ptr);
+    }
+  }
+} /* MenuSetLinks */
+
+/*
+ * select item
+ */
+void
+MenuSelect (MENU_ID id)
+{
+  SelectItem(FindItem(id));
+} /* MenuSelect */
+
 /*
  * set default item
  */
@@ -162,7 +243,7 @@ MenuSetAbort (MENU_ID id)
 /*
  * async execution of function
  */
-void 
+void
 MenuExecFunc (MIC_button func, void *data)
 {
   execFlag = XBTrue;
@@ -192,9 +273,9 @@ MenuSetActive (MENU_ID id, XBBool active)
 } /* MenuSetActive */
 
 /*
- * welches item hat den focus
+ * get item id with focus
  */
-MENU_ID 
+MENU_ID
 MenuGetFocus (void)
 {
   return (NULL != itemFocus) ? itemFocus->id : 0;
@@ -207,7 +288,7 @@ MenuGetFocus (void)
  *------------------------------------------------------------------------*/
 
 /*
- * add an item to the menu
+ * add a generic item to the menu
  */
 static MENU_ID
 MenuAdd (XBMenuItem *item)
@@ -218,8 +299,9 @@ MenuAdd (XBMenuItem *item)
   if (itemLast != NULL) {
     itemLast->next = item;
     item->prev      = itemLast;
-  } 
+  }
   itemLast = item;
+  itemLast->next=NULL;
   if (itemFocus == NULL && item->focus != NULL) {
     item->flags |= MIF_FOCUS;
     itemFocus    = item;
@@ -247,8 +329,8 @@ MenuAddVButton (int x, int y, int h, const char *text, MIC_button func, void *fu
 } /* MenuAddVButton */
 
 /*
- * a checkbox item to menu
- */ 
+ * add a checkbox item to menu
+ */
 MENU_ID
 MenuAddToggle (int x, int y, int w, const char *text, XBBool *pState)
 {
@@ -256,7 +338,7 @@ MenuAddToggle (int x, int y, int w, const char *text, XBBool *pState)
 } /* MenuAddToggle */
 
 /*
- * a label/title to menu
+ * add a label/title to menu
  */
 MENU_ID
 MenuAddLabel (int x, int y, int w, const char *text)
@@ -264,22 +346,26 @@ MenuAddLabel (int x, int y, int w, const char *text)
   return MenuAdd (MenuCreateLabel (x, y, w, text) );
 } /* MenuAddLabel */
 
-
+/*
+ * add label variant 1 to menu
+ */
 MENU_ID
 MenuAddLabel1 (int x, int y, int w, const char *text)
 {
   return MenuAdd (MenuCreateLabel1 (x, y, w, text) );
-} /* MenuAddLabel */
+} /* MenuAddLabel1 */
 
+/*
+ * add label variant 2 to menu
+ */
 MENU_ID
 MenuAddLabel2 (int x, int y, int w, const char *text)
 {
   return MenuAdd (MenuCreateLabel2 (x, y, w, text) );
-} /* MenuAddLabel */
+} /* MenuAddLabel2 */
 
 /*
-/*
- * a simple combobox to menu
+ * add a combobox to menu, check only integer keys
  */
 MENU_ID
 MenuAddComboInt (int x, int y, int w_text, const char *text, int w, int *value, XBComboEntryList *table)
@@ -288,7 +374,7 @@ MenuAddComboInt (int x, int y, int w_text, const char *text, int w, int *value,
 } /* MenuAddComboInt */
 
 /*
- * a simple combobox to menu
+ * a simple yes/no combobox to menu
  */
 MENU_ID
 MenuAddComboBool (int x, int y, int w_text, const char *text, int w, XBBool *value)
@@ -297,7 +383,7 @@ MenuAddComboBool (int x, int y, int w_text, const char *text, int w, XBBool *val
 } /* MenuAddComboInt */
 
 /*
- * a simple combobox to menu
+ * add a combobox to menu, check only data entry
  */
 MENU_ID
 MenuAddComboData (int x, int y, int w_text, const char *text, int w, void **data, XBComboEntryList *table)
@@ -306,7 +392,7 @@ MenuAddComboData (int x, int y, int w_text, const char *text, int w, void **data
 } /* MenuAddComboData */
 
 /*
- * a simple combobox to menu
+ * add a combobox to menu, check only atom entry
  */
 MENU_ID
 MenuAddComboAtom (int x, int y, int w_text, const char *text, int w, XBAtom *atom, XBComboEntryList *table)
@@ -315,7 +401,7 @@ MenuAddComboAtom (int x, int y, int w_text, const char *text, int w, XBAtom *ato
 } /* MenuAddComboAtom */
 
 /*
- * a simple combobox to menu
+ * add a combobox to menu, check all entries
  */
 MENU_ID
 MenuAddCombo (int x, int y, int w_text, const char *text, int w, int *value, void **data, XBAtom *atom, XBComboEntryList *table)
@@ -324,16 +410,24 @@ MenuAddCombo (int x, int y, int w_text, const char *text, int w, int *value, voi
 } /* MenuAddCombo */
 
 /*
- *
+ * add a player graphics to menu
  */
 MENU_ID
 MenuAddPlayer (int x, int y, int w, int sprite, const CFGPlayerGraphics **cfg, int n_anime, BMSpriteAnimation *anime)
 {
   return MenuAdd (MenuCreatePlayer (x, y, w, sprite, cfg, n_anime, anime) );
 } /* MenuAddPlayer */
+/*
+ * add a player graphics to menu
+ */
+MENU_ID
+MenuAddConfigPlayer (int x, int y, int w, int sprite, const CFGPlayerGraphics **cfg, int n_anime, BMSpriteAnimation *anime, XBRgbValue *pRgb )
+{
+  return MenuAdd (MenuCreateConfigPlayer (x, y, w, sprite, cfg, n_anime, anime, pRgb) );
+} /* MenuAddPlayer */
 
 /*
- *
+ * add an item for string input to menu
  */
 MENU_ID
 MenuAddString (int x, int y, int w_text, const char *text, int w, char *buffer, size_t len)
@@ -342,7 +436,7 @@ MenuAddString (int x, int y, int w_text, const char *text, int w, char *buffer,
 } /* MenuAddString */
 
 /*
- *
+ * add an item for changing a color to menu
  */
 MENU_ID
 MenuAddColor (int x, int y, int w, const char *text, XBColor *color, XBRgbValue *pRgb)
@@ -351,7 +445,7 @@ MenuAddColor (int x, int y, int w, const char *text, XBColor *color, XBRgbValue
 } /* MenuAddColor */
 
 /*
- *
+ * add an item for assigning a key to menu
  */
 MENU_ID
 MenuAddKeysym (int x, int y, int w, const char *text, XBAtom *pKey)
@@ -360,7 +454,7 @@ MenuAddKeysym (int x, int y, int w, const char *text, XBAtom *pKey)
 } /* MenuAddKeysym */
 
 /*
- *
+ * add an invisble item for a poll function
  */
 MENU_ID
 MenuAddCyclic (MIC_cyclic func, void *par)
@@ -369,16 +463,16 @@ MenuAddCyclic (MIC_cyclic func, void *par)
 } /* MenuAddCyclic */
 
 /*
- *
+ * add an item for integer input to menu
  */
-MENU_ID 
+MENU_ID
 MenuAddInteger (int x, int y, int w_text, const char *text, int w, int *pValue, int min, int max)
 {
   return MenuAdd (MenuCreateInteger (x, y, w_text, text, w, pValue, min, max));
 } /* MenuAddInteger */
 
 /*
- * name tag for players
+ * add a name tag for players
  */
 MENU_ID
 MenuAddTag (int x, int y, int w, const char **pText)
@@ -386,6 +480,28 @@ MenuAddTag (int x, int y, int w, const char **pText)
   return MenuAdd (MenuCreateTag (x, y, w, pText));
 } /* MenuAddTag */
 
+/*
+ * add integer tag (counter)
+ */
+MENU_ID
+MenuAddIntTag (int x, int y, int w, int *pNr)
+{
+  return MenuAdd (MenuCreateIntTag (x, y, w, pNr));
+} /* MenuAddIntTag */
+
+/*******************************************
+ * host items for server/client wait menus *
+ *******************************************/
+
+/*
+ * add a generic host button
+ */
+MENU_ID
+MenuAddHost (int x, int y, int w, unsigned client, const char **pText, XBHSFocusFunc focusFunc, XBHSChangeFunc chgFunc, XBHSUpdateFunc upFunc)
+{
+  return MenuAdd (MenuCreateHost (x, y, w, client, pText, focusFunc, chgFunc, upFunc) );
+} /* MenuAddHost */
+
 /*
  * host button (server)
  */
@@ -395,16 +511,6 @@ MenuAddServer (int x, int y, int w, const char **pText)
   return MenuAdd (MenuCreateServer (x, y, w, pText) );
 } /* MenuAddServer */
 
-/*
- * team button (server)
- */
-MENU_ID
-MenuAddTeam (int x, int y, int w, XBTeamState *pTeam, int server)
-{
-  return MenuAdd (MenuCreateTeam (x, y, w, pTeam,server) );
-} /* MenuAddServer */
-
-
 /*
  * host button (client)
  */
@@ -412,7 +518,7 @@ MENU_ID
 MenuAddClient (int x, int y, int w, const char **pText, XBHostState *pState, const int *pPing)
 {
   return MenuAdd (MenuCreateClient (x, y, w, pText, pState, pPing) );
-} /* MenuAddServer */
+} /* MenuAddClient */
 
 /*
  * host button (peer)
@@ -421,7 +527,38 @@ MENU_ID
 MenuAddPeer (int x, int y, int w, const char **pText, XBHostState *pState, const int *pPing)
 {
   return MenuAdd (MenuCreatePeer (x, y, w, pText, pState, pPing) );
-} /* MenuAddServer */
+} /* MenuAddPeer */
+
+/*******************************************
+ * team items for server/client wait menus *
+ *******************************************/
+
+/*
+ * team button (generic)
+ */
+MENU_ID
+MenuAddTeam (int x, int y, int w, unsigned id, unsigned player, XBTSFocusFunc focusFunc, XBTSChangeFunc chgFunc, XBTSUpdateFunc upFunc)
+{
+  return MenuAdd (MenuCreateTeam (x, y, w, id, player, focusFunc, chgFunc, upFunc) );
+} /* MenuAddTeam */
+
+/*
+ * team button (server)
+ */
+MENU_ID
+MenuAddServerTeam (int x, int y, int w, XBTeamState *pTeam)
+{
+  return MenuAdd (MenuCreateServerTeam (x, y, w, pTeam) );
+} /* MenuAddServerTeam */
+
+/*
+ * team button (peeer)
+ */
+MENU_ID
+MenuAddPeerTeam (int x, int y, int w, XBTeamState *pTeam)
+{
+  return MenuAdd (MenuCreatePeerTeam (x, y, w, pTeam) );
+} /* MenuAddPeerTeam */
 
 /*
  * add statitics header to menu
@@ -517,7 +654,7 @@ MenuAddInfoEntry (int x, int y, int w, const XBCentralInfo *stat, MIC_button fun
 
 /*------------------------------------------------------------------------
  *
- * Event hanlding
+ * Event handling
  *
  *------------------------------------------------------------------------*/
 
@@ -572,27 +709,27 @@ MoveFocus (XBMenuKey dir)
     XBMenuItem *newFocus;
     switch (dir) {
       /* arrow keys */
-    case XBMK_LEFT:  
-      newFocus = itemFocus->left;  
-      while (newFocus->flags & MIF_DEACTIVATED) {
+    case XBMK_LEFT:
+      newFocus = itemFocus->left;
+      while (newFocus && newFocus->flags & MIF_DEACTIVATED) {
 	newFocus = newFocus->left;
       }
       break;
-    case XBMK_RIGHT: 
-      newFocus = itemFocus->right; 
-      while (newFocus->flags & MIF_DEACTIVATED) {
+    case XBMK_RIGHT:
+      newFocus = itemFocus->right;
+      while (newFocus && newFocus->flags & MIF_DEACTIVATED) {
 	newFocus = newFocus->right;
       }
       break;
-    case XBMK_UP:    
-      newFocus = itemFocus->up;    
-      while (newFocus->flags & MIF_DEACTIVATED) {
+    case XBMK_UP:
+      newFocus = itemFocus->up;
+      while (newFocus && newFocus->flags & MIF_DEACTIVATED) {
 	newFocus = newFocus->up;
       }
       break;
-    case XBMK_DOWN:  
-      newFocus = itemFocus->down;  
-      while (newFocus->flags & MIF_DEACTIVATED) {
+    case XBMK_DOWN:
+      newFocus = itemFocus->down;
+      while (newFocus && newFocus->flags & MIF_DEACTIVATED) {
 	newFocus = newFocus->down;
       }
       break;
@@ -600,7 +737,7 @@ MoveFocus (XBMenuKey dir)
     case XBMK_NEXT:
       newFocus = itemFocus->next;
       while (newFocus != NULL) {
-	if (newFocus->focus != NULL && 
+	if (newFocus->focus != NULL &&
 	    ! (newFocus->flags & MIF_DEACTIVATED) ) {
 	  break;
 	}
@@ -611,14 +748,14 @@ MoveFocus (XBMenuKey dir)
     case XBMK_PREV:
       newFocus = itemFocus->prev;
       while (newFocus != NULL) {
-	if (newFocus->focus != NULL && 
+	if (newFocus->focus != NULL &&
 	    ! (newFocus->flags & MIF_DEACTIVATED) ) {
 	  break;
 	}
 	newFocus = newFocus->prev;
       }
       break;
-    default:         
+    default:
       return;
     }
     if (NULL != newFocus) {
@@ -632,26 +769,14 @@ MoveFocus (XBMenuKey dir)
 } /* MoveFocus */
 
 /*
- * an menu item was selected (i.e Space was pressed)
- */
-static void
-SelectItem (XBMenuItem *item)
-{
-  if (item != NULL &&
-      item->select != NULL) {
-    (*item->select) (item);
-  }
-} /* SelectItem */
-
-/*
- *
+ * set mouse to position
  */
 static void
 SetMousePosition (int x, int y)
 {
   XBMenuItem *newItem = MenuGetMouseItem (x, y);
   /* set new focus if needed */
-  if (NULL != newItem && 
+  if (NULL != newItem &&
       ! (newItem->flags & MIF_DEACTIVATED) &&
       newItem != itemFocus) {
     /* take back old focus */
@@ -669,7 +794,7 @@ SetMousePosition (int x, int y)
 } /* SetMousePosition */
 
 /*
- *
+ * determine item under mouse
  */
 static void
 MouseItem (XBEventCode button, int x, int y)
@@ -681,6 +806,91 @@ MouseItem (XBEventCode button, int x, int y)
   }
 } /* MouseItem */
 
+/*
+ * edit events
+ */
+static XBBool
+EditEvent(XBEventCode event, XBEventData data)
+{
+  if (! mapMode) {
+    return XBFalse;
+  }
+  switch (event) {
+  case XBE_RMOUSE_2:
+    SetOldBlock();
+  case XBE_RMOUSE_1:
+  case XBE_RMOUSE_3:
+    setPressed = 0;
+    break;
+  case XBE_MOUSE_2:
+    SetToBlockFree();
+  case XBE_MOUSE_1:
+  case XBE_MOUSE_3:
+    setPressed = 1;
+    fprintf(stderr,"mouse 1/3 pressed on  %i %i \n",data.pos.x, data.pos.y);
+    SetEditMapBlock(data.pos.x,data.pos.y);
+    return XBFalse; /* pass mouse buttons on to default */
+    /* mouse motion event */
+  case XBE_MOUSE_MOVE:
+    if (setPressed) {
+      SetEditMapBlock(data.pos.x,data.pos.y);
+    }
+    UpdateMaze();
+    return XBFalse; /* pass mouse move to default */
+  default:
+    return XBFalse;
+  }
+  return XBTrue;
+} /* EditEvent */
+
+/*
+ * default events
+ */
+static XBBool
+DefaultEvent(XBEventCode event, XBEventData data)
+{
+  switch (event) {
+    /* menu keys */
+  case  XBE_MENU:
+    switch (data.value) {
+    case XBMK_PREV:
+    case XBMK_NEXT:
+    case XBMK_LEFT:
+    case XBMK_RIGHT:
+    case XBMK_UP:
+    case XBMK_DOWN:
+      MoveFocus (data.value);
+      break;
+    case XBMK_SELECT:
+      SelectItem (itemFocus);
+      break;
+    case XBMK_ABORT:
+      SelectItem (abortItem);
+      break;
+    case XBMK_DEFAULT:
+      SelectItem (defaultItem);
+      break;
+    default:
+      break;
+    }
+    break;
+    /* mouse events */
+  case XBE_MOUSE_1:
+  case XBE_MOUSE_2:
+  case XBE_MOUSE_3:
+    SetMousePosition (data.pos.x, data.pos.y);
+    MouseItem (event, data.pos.x, data.pos.y);
+    break;
+    /* mouse motion event */
+  case XBE_MOUSE_MOVE:
+    SetMousePosition (data.pos.x, data.pos.y);
+    break;
+  default:
+    return XBFalse;
+  }
+  return XBTrue;
+} /* DefaultEvent */
+
 /*
  * event handling for menus
  */
@@ -714,43 +924,9 @@ MenuEventLoop (void)
     }
     /* get event from gui */
     while (XBE_TIMER != (event = GUI_WaitEvent (&data) ) ) {
-      switch (event) {
-	/* Keyboard events */
-      case  XBE_MENU:
-	switch (data.value) {
-	case XBMK_PREV:
-	case XBMK_NEXT:
-	case XBMK_LEFT:
-	case XBMK_RIGHT:
-	case XBMK_UP:   
-	case XBMK_DOWN:    
-	  MoveFocus (data.value);  
-	  break;
-	case XBMK_SELECT:  
-	  SelectItem (itemFocus);          
-	  break;
-	case XBMK_ABORT:   
-	  SelectItem (abortItem);          
-	  break;
-	case XBMK_DEFAULT: 
-	  SelectItem (defaultItem);          
-	  break;
-	default:
-	  break;
-	}
-	break;
-	/* mouse button events */
-      case XBE_MOUSE_1:
-      case XBE_MOUSE_2:
-      case XBE_MOUSE_3:
-	SetMousePosition (data.pos.x, data.pos.y);
-	MouseItem (event, data.pos.x, data.pos.y);
-	break;
-	/* mouse motion event */
-      case XBE_MOUSE_MOVE:
-	SetMousePosition (data.pos.x, data.pos.y);
-	break;
-      default:
+      if (! Chat_Event(event, data) &&
+	  ! EditEvent(event, data) &&
+	  ! DefaultEvent(event, data) ) {
 	continue;
       }
       result = MenuExecButton ();
@@ -766,7 +942,23 @@ MenuEventLoop (void)
   MenuUnloadTiles ();
   return XBTrue;
 } /* MenuEventLoop */
+/*
+ * set default item
+ */
+void
+MenuDeleteItemById (MENU_ID id)
+{
+  XBMenuItem *itemTemp;
+  XBMenuItem *item_next;
+  itemTemp = FindItem (id);
+  if (itemTemp == NULL) {
+    return;
+  }
+  item_next = itemTemp->next;
+  DeleteMenuItem(itemTemp);
+  itemTemp = item_next;
+} /* MenuDeleteItemById */
 
 /*
  * end of file mi_tool.c
- */ 
+ */
