@@ -1,7 +1,7 @@
 /*
  * file scramble.h - scrambling blocks
  *
- * $Id: scramble.h,v 1.3 2004/05/14 10:00:35 alfie Exp $
+ * $Id: scramble.h,v 1.5 2004/11/29 14:44:49 lodott Exp $
  *
  * Program XBLAST 
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
@@ -28,7 +28,7 @@
 /*
  * global prototypes
  */
-extern XBBool ConfigLevelScramble (const DBSection *sectionDraw, const DBSection *sectionDel);
+extern XBBool ParseLevelScramble (const DBSection *sectionDraw, const DBSection *sectionDel, DBSection *warn1, DBSection *warn2);
 extern void DoScramble (int gameTime);
  
 extern int getScrambleTimes(int p_time); // XBCC
