@@ -1,7 +1,7 @@
 /*
  * file shrink.c - level shrink and scrambling blocks
  *
- * $Id: shrink.c,v 1.4 2004/06/30 16:06:00 iskywalker Exp $
+ * $Id: shrink.c,v 1.14 2005/01/11 22:44:41 lodott Exp $
  *
  * Program XBLAST 
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
@@ -72,51 +72,22 @@ typedef struct {
   BMPosition *blocks;
 } ScrambleStruct; 
 
-/*
- * type definitions
- */
-typedef enum {
-  ST_Void,
-  ST_Spiral,
-  ST_SpeedSpiral,
-  ST_SpiralPlus,
-  ST_Spiral3,
-  ST_Spiral23,
-  ST_SpiralLego,
-  ST_EarlySpiral,
-  ST_Compound,
-  ST_CompoundF,
-  ST_Compound2F,
-  ST_LazyCompoundF,
-  ST_CompoundSolid,
-  ST_SavageCompound,
-  ST_CompoundExtra,
-  ST_Down,
-  ST_DownF,
-  ST_Quad,
-  ST_ConstrictWave,
-  ST_OutwardSpiral,
-  /* no new shrink after this line */
-  /* EPFL */
-  ST_Horiz,
-  ST_Circle,
-  ST_Move,
-  ST_IstySpiral3,
-  ST_IstyCompound2F,
-  ST_Diag,
-  ST_OutwardExtra,
-  ST_Spiral5,
-  /* EPFL */
-  NUM_ST
-} XBShrinkType;
+
 
 /*
- * local variables
+ * local variables - retrieved from level data
+ */
+static int shrinkType;
+static char* stVoidName = "No Shrink";
+
+/*
+ * other local variables
  */
 static SND_Id shrinkSound     	     	= SND_SPIRAL;
 static XBBool soundFlipflop    	     	= XBTrue;
 static XBBool playShrink       	     	= XBTrue;
 static ShrinkGeneric *shrink_ptr    	= NULL;
+static ShrinkGeneric *shrink_ptr1    	= NULL;
 static ShrinkGeneric *shrink_data   	= NULL;
 static ShrinkGeneric shrink_data_none[] = {
   /* terminator */
@@ -152,7 +123,8 @@ static DBToInt shrinkTable[] = {
   { "spiral5",        ST_Spiral5 },
   { "spiralLego",     ST_SpiralLego },
   { "spiralPlus",     ST_SpiralPlus },
-  { NULL, ST_Void },
+  { "void",           ST_Void },
+  { NULL, -1 },
 };
 /* shrink info text */
 static const char *shrinkName[NUM_ST] = {
@@ -244,8 +216,9 @@ DoShrink (int g_time)
   if (g_time == shrink_ptr->time - SHRINK_WARN_OFFSET) {
     SND_Play (SND_WARN, SOUND_MIDDLE_POSITION);
   }
-
+  //  fprintf(stderr," time %i %s \n",g_time,GetShrinkName(shrinkType));
   while ( (g_time) == shrink_ptr->time ) {
+    // fprintf(stderr," %i %i \n",shrink_ptr->x, shrink_ptr->y);
     /* set block */
     SetMazeBlock(shrink_ptr->x, shrink_ptr->y, shrink_ptr->block);
 
@@ -269,6 +242,20 @@ DoShrink (int g_time)
   playShrink = XBTrue;
 } /* DoShrink */
 
+void
+DoShrinkMapEdit (int g_time)
+{
+  shrink_ptr=shrink_ptr1;
+  if (g_time == shrink_ptr->time - SHRINK_WARN_OFFSET) {
+    SND_Play (SND_WARN, SOUND_MIDDLE_POSITION);
+  }
+  while ( (g_time) != shrink_ptr->time && shrink_ptr->time!=0&& shrink_ptr->time<g_time  ) {
+       /* set block */
+    SetMazeBlock(shrink_ptr->x, shrink_ptr->y, shrink_ptr->block);
+    shrink_ptr ++;
+  }
+  playShrink = XBTrue;
+} /* DoShrink */
 /*
  *  Generic shrink function 
  */
@@ -389,6 +376,7 @@ CreateGenericShrink (shri_data *data, shri_xoff_data *xoffdata, int startlevel,
   acelm++;
   SortShrinkArray (shrink_data, acelm);
   shrink_ptr = shrink_data;
+  shrink_ptr1 = shrink_ptr;
 } /* CreateShrinkGeneric */
 
 /*
@@ -477,22 +465,44 @@ CreateFancyVerticalShrink (int startlevel, int endlevel, int inc, int starttime,
 
 /* Now the actual shrink functions */
 
+
 /*
- * 
+ * parse shrink data in level data
+ */
+XBBool
+ParseLevelShrink (const DBSection *section, DBSection *warn) {
+  /* check existence of section */
+  if (NULL == section) {
+    Dbg_Out("LEVEL: shrink section is missing!\n");
+    DB_CreateEntryString(warn,atomMissing,"true");
+    return XBFalse;
+  }
+  /* type has default */
+  switch (DB_ConvertEntryInt (section, atomType, (int *) &shrinkType, shrinkTable) ) {
+  case DCR_NoSuchEntry:
+    Dbg_Level("default for %s\n",  DB_SectionEntryString(section,atomType));
+    shrinkType = ST_Void;
+    break;
+  case DCR_Failure:
+    Dbg_Out("LEVEL: warning %s\n", DB_SectionEntryString(section, atomType));
+    shrinkType = ST_Void;
+    DB_CreateEntryString(warn,atomType,DB_IntToString(shrinkTable,shrinkType));
+    break;
+  default:
+    break;
+  }
+  return XBTrue;
+} /* ParseLevelShrink */
+
+/*
+ * configure selected shrink
  */
 XBBool
 ConfigLevelShrink (const DBSection *section)
 {
-  XBShrinkType shrinkType;
   const char *s;
 
   assert (section != NULL);
-
-  if (! DB_ConvertEntryInt (section, atomType, (int *) &shrinkType, shrinkTable) ) {
-    shrinkType = ST_Void;
-  fprintf(stderr,"UNKNOWN SHRINK\n");
-  }
-  fprintf(stderr,"SHRINK = %d\n",shrinkType);
   switch (shrinkType) {
   case ST_Void:           shrink_ptr = shrink_data_none;                                             	     break;
   case ST_Spiral:         CreateSpiralShrink (1, 2,  GAME_TIME/2, 4, &style_rise_2);     	 	     break;
@@ -531,6 +541,7 @@ ConfigLevelShrink (const DBSection *section)
   if (NULL != (s = shrinkName[shrinkType]) ) {
     AddLevelInfo (s);
   }
+  shrink_ptr1 = shrink_ptr;
   return XBTrue;
 } /* ConfigLevelShrink */
 
@@ -545,6 +556,19 @@ FinishLevelShrink (void)
     shrink_data = NULL;
   }
 } /* FinishShrink */
+/*
+ * return the name of the shrink type
+ */
+const char*
+GetShrinkName(XBShrinkType type) {
+  int i;
+  if (type == ST_Void)
+    return stVoidName;
+  for (i = 0; i < NUM_ST; ++i)
+    if (shrinkTable[i].value == type)
+      return shrinkTable[i].key;
+  return stVoidName;
+}
 
 /*
  * end of file shrink.c
