@@ -1,7 +1,7 @@
 /*
  * file shrink.h - level shrink and scrambling blocks
  *
- * $Id: shrink.h,v 1.3 2004/05/14 10:00:35 alfie Exp $
+ * $Id: shrink.h,v 1.7 2004/11/29 14:44:49 lodott Exp $
  *
  * Program XBLAST 
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
@@ -33,6 +33,8 @@
 extern XBBool ConfigLevelShrink (const DBSection *);
 extern void FinishLevelShrink (void);
 extern void DoShrink (int g_time);
+extern void DoShrinkMapEdit (int g_time);
+extern XBBool ParseLevelShrink (const DBSection *section, DBSection *) ;
 
 /* generic shrink data */
 typedef struct {
@@ -43,7 +45,45 @@ typedef struct {
 extern ShrinkGeneric *GetShrinkPtr ();
 /* XBCC */
 extern int getShrinkTimes(int p_time);
+/*
+ * type definitions
+ */
+typedef enum {
+  ST_Void,
+  ST_Spiral,
+  ST_SpeedSpiral,
+  ST_SpiralPlus,
+  ST_Spiral3,
+  ST_Spiral23,
+  ST_SpiralLego,
+  ST_EarlySpiral,
+  ST_Compound,
+  ST_CompoundF,
+  ST_Compound2F,
+  ST_LazyCompoundF,
+  ST_CompoundSolid,
+  ST_SavageCompound,
+  ST_CompoundExtra,
+  ST_Down,
+  ST_DownF,
+  ST_Quad,
+  ST_ConstrictWave,
+  ST_OutwardSpiral,
+  /* no new shrink after this line */
+  /* EPFL */
+  ST_Horiz,
+  ST_Circle,
+  ST_Move,
+  ST_IstySpiral3,
+  ST_IstyCompound2F,
+  ST_Diag,
+  ST_OutwardExtra,
+  ST_Spiral5,
+  /* EPFL */
+  NUM_ST
+} XBShrinkType;
 
+extern const char* GetShrinkName(XBShrinkType type);
 #endif
 /*
  * end of file shrink.h
