@@ -35,6 +35,8 @@
 #define TEXT_ATTR_NUMBER (FF_Large|FF_White|FF_Boxed)
 #define TEXT_ATTR_SMALLNUMBER (FF_Medium|FF_White|FF_Boxed) // mab
 #define TEXT_ATTR_SMALLNUMBER1 (FF_Small |FF_White|FF_Boxed)
+/* Added by VVL (Chat) 12/11/99 */
+#define FF_Scroll      0x40
 
 #define MAX_MSG           8
 #define MESSAGE_TIME      TIME_STEP
@@ -72,6 +74,19 @@ static const int numPos[MAX_PLAYER] = {
 /* variables for message queue */
 static const char *defaultMessage;
 static int         msgRestore;
+/* Added by VVL (Chat) 12/11/99 */
+static int chat_restore;
+/* Added by VVL (Chat) 12/11/99 */
+static int num_chat = 0;
+/* Added by VVL (Chat) 12/11/99 */
+static int first_chat;
+/* Added by VVL (Chat) 12/11/99 */
+static char chat_queue[MAX_MSG][CHAT_LEN+128];
+/* Added by VVL (Chat) 12/11/99 : Begin */
+static int chat_delay[MAX_MSG+1] = {
+  -1, 200, 80, 40, 20, 15, 10, 7, 7
+};
+/* Added by VVL (Chat) 12/11/99 : End */
 static int         numMsg = 0;
 static int         firstMsg;
 static const char *msgQueue[MAX_MSG];
@@ -85,7 +100,28 @@ static const int textRight = 13;
 static BMRectangle  statusBox = {
   77*STAT_WIDTH/12, 79*STAT_HEIGHT/6,
   87*STAT_WIDTH/12,  2*STAT_HEIGHT/3,
+};/* Added by VVL (Chat) 12/11/99 : Begin */
+static BMRectangle get_box = {
+  2*STAT_WIDTH/12, 
+#ifdef SMPF
+  93*STAT_HEIGHT/6,
+#else
+  87*STAT_HEIGHT/6,
+  
+#endif
+  116*STAT_WIDTH/12,  2*STAT_HEIGHT/3,
 };
+static BMRectangle chat_box = {
+  121*STAT_WIDTH/12, 
+#ifdef SMPF
+  93*STAT_HEIGHT/6,
+#else
+  87*STAT_HEIGHT/6,
+
+#endif
+  116*STAT_WIDTH/12,  2*STAT_HEIGHT/3,
+};
+/* Added by VVL (Chat) 12/11/99 : End */
 
 /* status board number boxes */
 #ifdef SMPF
@@ -197,8 +233,9 @@ ClearStatusBar (int tile1, int tile2)
   for (x = 0; x < MAZE_W; x ++) {
     GUI_DrawBlock (x, MAZE_H,   tile1);
     GUI_DrawBlock (x, MAZE_H+1, tile2);
-#ifdef SMPF
     GUI_DrawBlock (x, MAZE_H+2, tile2);
+#ifdef SMPF
+    GUI_DrawBlock (x, MAZE_H+3, tile2);
 #endif
   }
   MarkMazeRect (0, PIXH, PIXW, SCOREH);
@@ -207,21 +244,21 @@ ClearStatusBar (int tile1, int tile2)
 /*
  *
  */
-#ifdef SMPF
+//#ifdef SMPF
 static void
 SetScoreBlock (int x, int y, int block)
 {
   GUI_DrawScoreBlock (x, y, block);
   MarkMazeTile (x, MAZE_H+y);
 } /* SetScoreBlock */
-#else
+/*#else
 static void
 SetScoreBlock (int x, int block)
 {
   GUI_DrawScoreBlock (x, block);
   MarkMazeTile (x, MAZE_H);
-} /* SetScoreBlock */
-#endif
+  } *//* SetScoreBlock */
+//#endif
 /*
  *
  */
@@ -232,6 +269,18 @@ SetStatusText (const char *msg)
   MarkMaze (textLeft, MAZE_H, textRight, MAZE_H);
 } /* SetStatusText */
 
+/* Added by VVL (Chat) 12/11/99 : Begin */
+/**/
+/* local function set chat text */
+void 
+SetChatText (char *msg)
+{
+  GUI_DrawTextbox(msg, TEXT_ATTR_NORMAL | FF_Scroll, &chat_box);
+  MarkMaze(10, MAZE_H+STAT_H-1, 20, MAZE_H+STAT_H);
+}
+/* Added by VVL (Chat) 12/11/99 : End */
+
+
 /*
  *
  */
@@ -248,7 +297,7 @@ SetStatusNumber (int pos, int value)
   numString[1] = '\0';
   GUI_DrawSimpleTextbox (numString, TEXT_ATTR_NUMBER, statusNumberBox + pos);
 #ifdef SMPF
-  fprintf(stderr," y1 %i pos %i pos  20 %i value %i\n",y,pos,pos%20,value);
+  //  fprintf(stderr," y1 %i pos %i pos  20 %i value %i\n",y,pos,pos%20,value);
   MarkMazeTile (pos % 20, MAZE_H+y); // SMPF - very carefull here!!!!!
 #else
   MarkMazeTile (pos, MAZE_H);
@@ -270,7 +319,7 @@ SetStatusNumber2 (int pos, int value)
   numString[1] = '\0';
   GUI_DrawSimpleTextbox (numString, TEXT_ATTR_SMALLNUMBER1, statusNumberBox2 + (pos));
 #ifdef SMPF
-  fprintf(stderr," y2 %i pos %i pos  20 %i value %i\n",y,pos,pos%20,value);
+  //  fprintf(stderr," y2 %i pos %i pos  20 %i value %i\n",y,pos,pos%20,value);
   MarkMazeTile (pos % 20, MAZE_H+y); // SMPF - very carefull here!!!!!
 #else
   MarkMazeTile (pos, MAZE_H);
@@ -306,8 +355,12 @@ InitStatusBar (int numPlayers)
 #ifdef SMPF
     SetScoreBlock (p, 0, SBVoid); // SMPF - 2D shit here see above..
     SetScoreBlock (p, 1, SBVoid); // SMPF - 2D shit here see above..
+    SetScoreBlock (p, 2, SBVoid); // SMPF - 2D shit here see above..
+    SetScoreBlock (p, 3, SBVoid); // SMPF - 2D shit here see above..
 #else
-    SetScoreBlock (p, SBVoid);
+    SetScoreBlock (p,0, SBVoid);
+    SetScoreBlock (p,1, SBVoid);
+    SetScoreBlock (p,2, SBVoid);
 #endif
   }
 
@@ -323,8 +376,8 @@ InitStatusBar (int numPlayers)
     SetScoreBlock   (facePos[p]+1, (p < 6 ? 0 : 1),  SBTextRight);
     SetStatusNumber (numPos[p], (p < 6 ? 0 : 1),  numUsed[p]);
 #else
-    SetScoreBlock   (facePos[p], faceUsed[p]);
-    SetScoreBlock   (numPos[p],  SBTextRight);
+    SetScoreBlock   (facePos[p],0, faceUsed[p]);
+    SetScoreBlock   (numPos[p],0,  SBTextRight);
     SetStatusNumber (numPos[p],  numUsed[p]);
 #endif
   }
@@ -336,11 +389,11 @@ InitStatusBar (int numPlayers)
   }
   SetScoreBlock (textRight, 0, SBTextRight);
 #else
-  SetScoreBlock (textLeft,  SBTextLeft);
+  SetScoreBlock (textLeft,0,  SBTextLeft);
   for (p = textLeft + 1; p < textRight; p ++) {
-    SetScoreBlock (p, SBTextMid);
+    SetScoreBlock (p,0, SBTextMid);
   }
-  SetScoreBlock (textRight, SBTextRight);
+  SetScoreBlock (textRight,0, SBTextRight);
 #endif
   /* set queue to default message */
   defaultMessage = "";
@@ -373,7 +426,7 @@ ResetStatusBar (const BMPlayer *ps, const char *msg, XBBool flag)
     SetStatusNumber (numPos[p], (p < 6 ? 0 : 1),  numUsed[p]);
     SetStatusNumber2 (numPos[p], (p < 6 ? 0 : 1), ps[p].victories);
 #else
-    SetScoreBlock   (facePos[p], faceUsed[p]);
+    SetScoreBlock   (facePos[p],0, faceUsed[p]);
     SetStatusNumber (numPos[p],  numUsed[p]);
     SetStatusNumber2 (numPos[p], ps[p].victories);
 #endif
@@ -460,7 +513,7 @@ UpdateStatusBar (const BMPlayer *ps, int game_time)
 #ifdef SMPF
       SetScoreBlock (facePos[p], (p < 6 ? 0 : 1), faceUsed[p]);
 #else
-      SetScoreBlock (facePos[p], faceUsed[p]);
+      SetScoreBlock (facePos[p],0, faceUsed[p]);
 #endif
     }
     /* check player lives */
@@ -499,6 +552,26 @@ UpdateStatusBar (const BMPlayer *ps, int game_time)
       }
     }
   }
+/* Added by VVL (Chat) 12/11/99 */
+  /* check chat */
+  /* if any chat message is in the queue */
+  if (num_chat) {
+    chat_restore ++;
+    /* check if message is outdated */
+    if (chat_restore >= chat_delay[num_chat]) {
+      /* return to default message if last queued one */
+      if (--num_chat) {
+	/* set next message */
+	first_chat = (first_chat+1) % MAX_MSG;
+	SetChatText(chat_queue[first_chat]);
+	chat_restore = 0;
+      } else {
+	/* return to default message if last queued one */
+	SetChatText("");
+      }
+    }
+  }
+/* Added by VVL (Chat) 12/11/99 : End */
 } /* UpdateStatusBar */
 
 /*
@@ -534,6 +607,55 @@ SetMessage (const char *msg, XBBool perm)
   }
 } /* SetMessage */
 
+
+/* Added by VVL (Chat) 12/11/99 : Begin */
+/**/
+/* public function set_get
+ */
+
+void
+SetGet (char *msg, int perm)
+{
+
+    GUI_DrawTextbox(msg, TEXT_ATTR_NORMAL | FF_Scroll, &get_box);
+    MarkMaze(0, MAZE_H+STAT_H-1, 20, MAZE_H+STAT_H);
+}
+/* Added by VVL (Chat) 12/11/99 : End */
+
+/* Added by VVL (Chat) 12/11/99 : Begin */
+/**/
+/* public function set_chat
+*/
+void
+SetChat (char *msg, int perm)
+{
+  int m;
+
+  if (NULL != msg) {
+    if (perm) {
+      SetChatText(msg);
+      num_chat = 0;
+    } else {
+      /* if no other message exists set to start of queue */
+      if (0 == num_chat) {
+	m = first_chat = 0;
+	chat_restore = 0;
+	SetChatText(msg);
+      } else {
+	m = (first_chat + num_chat) % MAX_MSG;
+      }
+      strncpy (chat_queue[m],msg,CHAT_LEN+128);
+      num_chat ++;
+      /* check for queue overflow */
+      if (num_chat > MAX_MSG) {
+	/* remove first message */
+	num_chat --;
+	first_chat = (first_chat+1) % MAX_MSG;
+      }
+    }
+  }
+}
+/* Added by VVL (Chat) 12/11/99 : End */
 /*
  * 
  */
