@@ -1,7 +1,7 @@
 /*
  * file status.h
  *
- * $Id: status.h,v 1.3 2004/05/14 10:00:35 alfie Exp $
+ * $Id: status.h,v 1.4 2004/10/19 17:59:19 iskywalker Exp $
  *
  * Program XBLAST 
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
@@ -31,6 +31,9 @@ extern void ResetStatusBar (const BMPlayer *ps,  const char *msg, XBBool flag);
 extern void UpdateStatusBar (const BMPlayer *ps, int g_time);
 extern void SetMessage (const char *msg, XBBool perm);
 extern void ResetMessage (void);
+extern void SetGet (char *msg, int perm);
+extern void SetChat (char *msg, int perm);
+
 
 #endif
 /*
