@@ -9,8 +9,7 @@
 
 #ifndef _TIMEVAL_H
 #define _TIMEVAL_H
-
-#ifdef WIN32
+#ifdef W32
 
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
