@@ -1,9 +1,9 @@
 /*
  * File util.c - file and directory i/o
  *
- * $Id: util.c,v 1.7 2004/07/07 16:43:13 iskywalker Exp $
+ * $Id: util.c,v 1.13 2005/03/12 17:37:31 iskywalker Exp $
  *
- * Program XBLAST 
+ * Program XBLAST
  * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -12,7 +12,7 @@
  * any later version
  *
  * This program is distributed in the hope that it will be entertaining,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
  * Public License for more details.
  *
@@ -21,13 +21,12 @@
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 #include "util.h"
-#ifdef WINMS32
+#if defined(__MINGW32__) || defined(WMS)
 #include <windows.h>
 #include <direct.h>
 #include <math.h>
 #endif
 #include "str_util.h"
-#include "xblast.h"
 
 /*
  * local macros
@@ -48,6 +47,8 @@ static char *pathList[NUM_XBLAST_PATH] = {
   ".",
   "/",
 };
+
+/* shared buffer for complete filename */
 static char zeile [1024];
 
 /*
@@ -67,8 +68,7 @@ InitPaths (void)
   } else {
     sprintf (userPath, "%s/.xblast_tnt", home);
   }
-#ifdef WINMS32
-
+#if defined(__MINGW32__) || defined(WMS)
   mkdir (userPath);
   /* config subdirs */
   sprintf (tmp, "%s/config", userPath);
@@ -103,7 +103,6 @@ FileOpen (const char *path, const char *name, const char *ext, const char *mode)
 {
   FILE       *fp;
   int         i;
-
   /* sanity check */
   assert (NULL != path);
   assert (NULL != name);
@@ -122,7 +121,7 @@ FileOpen (const char *path, const char *name, const char *ext, const char *mode)
       if (NULL != (fp = fopen (zeile, mode) ) ) {
 	/* file opened succesfully, use fclose to close it */
 	return fp;
-      } 
+      }
     }
   }
   /* sorry file opening failed */
@@ -130,8 +129,12 @@ FileOpen (const char *path, const char *name, const char *ext, const char *mode)
   return NULL;
 } /* FileOpen */
 
+/**********************
+ * creating dir lists *
+ **********************/
+
 /*
- *
+ * add a file to list
  */
 static void
 AddToFileList (XBDir **pList, const char *name, size_t len, time_t mtime)
@@ -141,11 +144,9 @@ AddToFileList (XBDir **pList, const char *name, size_t len, time_t mtime)
   XBDir *prev = NULL;
   int    cmp;
   char   tmp[256];
-
   assert (pList != NULL);
   assert (name != NULL);
   assert (len < 256 - 1);
-
   prev = NULL;
   strncpy (tmp, name, len);
   tmp[len] = 0;
@@ -176,41 +177,47 @@ AddToFileList (XBDir **pList, const char *name, size_t len, time_t mtime)
 } /* AddToFileList */
 
 /*
- *
+ * create file list for dir and extension, recurse optional
  */
 XBDir *
-CreateFileList (const char *path, const char *ext)
+CreateFileList (const char *path, const char *ext, XBBool rec)
 {
   int            i;
   XBDir         *list = NULL;
-#ifdef WINMS32
+  XBDir         *list1 = NULL;
+  XBDir  *ptr;
+#if defined(__MINGW32__) || defined(WMS)
   WIN32_FIND_DATA find_data;
+  WIN32_FIND_DATA find_data1;
   HANDLE         dp = NULL;
+  HANDLE         dp1 = NULL;
   size_t         lenExt;
   size_t         lenName;
   struct _stat   buf;
   char          *pFile;
   static char    dirName[1024];
+  static char    dirName1[1024];
   static char    fileName[1024];
-      char dirToOpen[256];
-      XBBool testFile = XBTrue;
+  static char    fileName1[1024];
+  char dirToOpen[256];
+  XBBool testFile = XBTrue;
 #else
   DIR           *dp;
+  DIR           *dp1;
   struct dirent *dirp;
   size_t         lenExt;
   size_t         lenName;
   struct stat    buf;
   char          *pFile;
   static char    dirName[1024];
+  static char    dirName1[1024];
   static char    fileName[1024];
+  static char    fileName1[1024];
 #endif
-
   /* sanity check */
   assert (NULL != path);
   assert (NULL != ext);
   assert ('.'  != ext[0]);
-  //assert ('.'  != path[0]);
-  // assert ('/'  != path[0]);
   /* set load paths if needed */
   if (! initPath) {
     InitPaths ();
@@ -219,94 +226,140 @@ CreateFileList (const char *path, const char *ext)
   lenExt = strlen (ext);
   for (i=0; i<NUM_XBLAST_PATH; i++) {
     if (NULL != pathList[i]) {
+      Dbg_File("trying path %s...\n", pathList[i]);
       /* create path to search */
       strcpy (dirName, pathList[i]);
       strcat (dirName, "/");
       strcat (dirName, path);
       /* prepare full path for file */
       strcpy (fileName, dirName);
-      strcat (fileName, "/"); 
+      strcat (fileName, "/");
       pFile = fileName + strlen (fileName);
       /* open directory to read */
-#ifdef WINMS32
-	  strcpy(dirToOpen, dirName);
+#if defined(__MINGW32__) || defined(WMS)
+      /* windows specific */
+      strcpy(dirToOpen, dirName);
       strcat(dirToOpen, "/*");
+      /* try to open dir */
       if (INVALID_HANDLE_VALUE == (dp = FindFirstFile(dirToOpen, &find_data))) {
-	    continue;
+	continue;
       }
-	  while (testFile)
-      {
-	    /* hidden file ? */
-	    if (find_data.cFileName[0] == '.') {
-            testFile = FindNextFile(dp, &find_data);
-	        continue;
+      Dbg_File("opening dir %s\n", dirName);
+      while (testFile) {
+	/* ignore hidden files */
+	if (find_data.cFileName[0] == '.') {
+	  testFile = FindNextFile(dp, &find_data);
+	  continue;
+	}
+	/* ignore files with wrong extension */
+	lenName = strlen (find_data.cFileName);
+	if (lenName <= lenExt) {
+	  testFile = FindNextFile(dp, &find_data);
+	  continue;
+	}
+	/* check for directory */
+	if (0  != strcmp (find_data.cFileName + lenName - lenExt, ext) ) {
+	  /* if recursive check if directory */
+	  if (0 != rec && find_data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
+	    sprintf(dirName1,"%s/%s/*",dirName,find_data.cFileName);
+	    /* try to open */
+	    if (INVALID_HANDLE_VALUE != (dp1 = FindFirstFile(dirName1, &find_data1)) ) {
+	      /* cut the \/* */
+	      dirName1[strlen(dirName1)-2]='\0';
+	      /* create list */
+	      list1 = CreateFileList (dirName1, ext, rec);
+	      if (list1 != NULL) {
+		for (ptr = list1; NULL != ptr; ptr = ptr->next) {
+		  sprintf(fileName1,"%s/%s",find_data.cFileName,ptr->name);
+		  AddToFileList (&list,fileName1 ,strlen(fileName1) ,ptr->mtime);
+		}
+		DeleteFileList(list1);
+		list1 = NULL;
+	      }
+	      testFile = FindNextFile(dp, &find_data);
+	      continue;
 	    }
-	    
-        /* Check file extension */
-	    lenName = strlen (find_data.cFileName);
-	    if (lenName <= lenExt || 0  != strcmp (find_data.cFileName + lenName - lenExt, ext) ) {
-            testFile = FindNextFile(dp, &find_data);
-	        continue;
-	    }
-	
-        /* Determine file modification time */
-	    strcpy (pFile, find_data.cFileName);
-
-	    if (0 != _stat (fileName, &buf) ) {
-           testFile = FindNextFile(dp, &find_data);
-	        continue;
- 	    }
-	
-        AddToFileList (&list, find_data.cFileName, lenName - lenExt -1, buf.st_mtime);
-
-        testFile = FindNextFile(dp, &find_data);
+	  }
+	  testFile = FindNextFile(dp, &find_data);
+	  continue;
+	}
+	/* ignore files without modification time */
+	strcpy (pFile, find_data.cFileName);
+	if (0 != _stat (fileName, &buf) ) {
+	  testFile = FindNextFile(dp, &find_data);
+	  continue;
+	}
+	AddToFileList (&list, find_data.cFileName, lenName - lenExt -1, buf.st_mtime);
+	testFile = FindNextFile(dp, &find_data);
       }
-
       /* close directory */
       FindClose(dp);
-    }
-  }
-  return list;
-
 #else
+      /* try to open */
       if (NULL == (dp = opendir (dirName) ) ) {
 	continue;
       }
+      Dbg_File("opening dir %s\n", dirName);
+      /* read files */
       while (NULL != (dirp = readdir(dp) ) ) {
-	/* hidden file ? */
+	/* ignroe hidden files */
 	if (dirp->d_name[0] == '.') {
 	  continue;
 	}
-	/* Check file extension */
+	/* ignore files with wrong extension */
 	lenName = strlen (dirp->d_name);
-	if (lenName <= lenExt ||
-	    0  != strcmp (dirp->d_name + lenName - lenExt, ext) ) {
+	if (lenName <= lenExt) {
 	  continue;
 	}
-	/* Determine file modification time */
+	/* check name for sanity */
+	if (0  != strcmp (dirp->d_name + lenName - lenExt, ext) ) {
+	  /* if recursive check if directory */
+	  if(0 != rec ){
+	    /* it might be a directory */
+	    sprintf(dirName1,"%s/%s",dirName,dirp->d_name);
+	    if ( NULL != (dp1 = opendir (dirName1) ) ) {
+	      /* create a list of the dir / subdirs */
+	      list1 = CreateFileList (dirName1, ext, rec);
+	      /* list not empty */
+	      if (list1 != NULL){
+		/* save list in sub list */
+		for (ptr = list1; NULL != ptr; ptr = ptr->next) {
+		  /* list name is filename from the path GAME_DATADIR/level */
+		  sprintf(fileName1,"%s/%s",dirp->d_name,ptr->name);
+		  /* add to main list */
+		  AddToFileList (&list, fileName1, strlen(fileName1) ,ptr->mtime);
+		}
+		DeleteFileList(list1);
+		list1 = NULL;
+	      }
+	      continue;
+	    }
+	  }
+	  continue;
+	}
+	/* ignore files without modification time */
 	strcpy (pFile, dirp->d_name);
 	if (0 != stat (fileName, &buf) ) {
 	  continue;
 	}
+	/* add the file */
 	AddToFileList (&list, dirp->d_name, lenName - lenExt -1, buf.st_mtime);
       }
       /* close directory */
       closedir (dp);
+#endif
     }
   }
   return list;
-  
-#endif
 } /* CreateFileList */
 
 /*
- *
+ * delete a file list
  */
 void
 DeleteFileList (XBDir *list)
 {
   XBDir *next;
-
   while (NULL != list) {
     next = list->next;
     /* delete data */
@@ -319,6 +372,9 @@ DeleteFileList (XBDir *list)
   }
 } /* DeleteFileList */
 
+/******************
+ * loading images *
+ ******************/
 
 /*
  * swaps bit order of pbm data to xbm format
@@ -329,11 +385,9 @@ PbmSwapBits (unsigned char *pbm, int width, int height)
   int             nbytes;
   static XBBool   convTableInit = XBFalse;
   static unsigned convTable[256];
-
   if (! convTableInit) {
     unsigned i, j;
     unsigned *ptr;
-
     for (i = 0, ptr = convTable; i < 256; i++, ptr++) {
       *ptr = 0;
       for (j = 0; j < 8; j ++) {
@@ -344,7 +398,6 @@ PbmSwapBits (unsigned char *pbm, int width, int height)
     }
     convTableInit = XBTrue;
   }
-
   for (nbytes = (width*height + 7) / 8; nbytes > 0; nbytes --, pbm ++) {
     *pbm = convTable[*pbm];
   }
@@ -364,12 +417,10 @@ PbmShrink (unsigned char *pbm, int *width, int *height)
   unsigned char *dst;
   unsigned char *pFirst;
   unsigned char *pSecond;
-
   if (! mergeTableInit) {
     unsigned i, j;
     unsigned *hi;
     unsigned *lo;
-
     for (i = 0, hi = hiMergeTable, lo = loMergeTable; i < 256; i++, hi++, lo++) {
       *hi = 0;
       *lo = 0;
@@ -391,16 +442,16 @@ PbmShrink (unsigned char *pbm, int *width, int *height)
     pFirst  = pbm    + 2*y*(*width/8);
     pSecond = pFirst + (*width/8);
     for (x = 0; x < (hWidth/8); x ++) {
-      dst[0] = (loMergeTable[pFirst[0]]  | 
-		loMergeTable[pSecond[0]] | 
-		hiMergeTable[pFirst[1]]  | 
+      dst[0] = (loMergeTable[pFirst[0]]  |
+		loMergeTable[pSecond[0]] |
+		hiMergeTable[pFirst[1]]  |
 		hiMergeTable[pSecond[1]]);
       dst ++;
       pFirst  += 2;
       pSecond += 2;
     }
     if (0 != hWidth % 8) {
-      dst[0] = (loMergeTable[pFirst[0]]  | 
+      dst[0] = (loMergeTable[pFirst[0]]  |
 		loMergeTable[pSecond[0]]);
       dst ++;
     }
@@ -420,14 +471,12 @@ ReadPbmFile (const char *path, const char *filename, int *width, int *height)
   FILE *         fp  = NULL;
   unsigned char *pbm = NULL;
   int            pbmSize;
-
   assert (width != NULL);
   assert (height != NULL);
-
+  /* try to open file name */
   if (NULL == (fp = FileOpen (path, filename, "pbm", "rb") ) ) {
     return NULL;
   }
-
   /* read header */
   if (3 != fscanf(fp, "%s%d%d%*d%*c", zeile, width, height) ) {
     fprintf(stderr, "Failed to read ppm header\n");
@@ -450,9 +499,7 @@ ReadPbmFile (const char *path, const char *filename, int *width, int *height)
 #endif
   /* close file */
   fclose (fp);
-
   return pbm;
-
   /* error handling */
 Error:
   if (fp != NULL) {
@@ -478,13 +525,10 @@ ShrinkPpm (unsigned char *ppm, int *width, int *height)
   unsigned char *pFirst;
   unsigned char *pSecond;
   unsigned char *dst;
-
   assert (NULL !=  width);
   assert (NULL !=  height);
-
   hWidth  = *width  / 2;
   hHeight = *height / 2;
-
   dst = ppm;
   for (y = 0; y < hHeight; y ++) {
     pFirst  = ppm + 3*2*y*(*width);
@@ -519,31 +563,26 @@ ReadPpmFile (const char *path, const char *filename, int *width, int *height)
   FILE *         fp  = NULL;
   unsigned char *ppm = NULL;
   int            nPixel;
-
   assert (width != NULL);
   assert (height != NULL);
-
+  /* try to open for reading */
   if (NULL == (fp = FileOpen (path, filename, "ppm", "rb") ) ) {
     goto Error;
   }
-
   /* read header */
   if (3 != fscanf(fp, "%s%d%d%*d%*c", zeile, width, height)) {
     fprintf(stderr, "Failed to read ppm header\n");
     goto Error;
   }
   nPixel = 3*(*width)*(*height);
-
   /* alloc data */
   if (NULL == (ppm = malloc (nPixel * sizeof(char) ) ) )  {
     goto Error;
   }
-    
   /* read data */
   if (nPixel != fread (ppm, sizeof(char), nPixel, fp) ) {
     goto Error;
   }
-  
   /* close file */
   fclose (fp);
 #ifdef MINI_XBLAST
@@ -552,7 +591,6 @@ ReadPpmFile (const char *path, const char *filename, int *width, int *height)
 #endif
   /* that's all */
   return ppm;
-
   /* error handling */
 Error:
   if (fp != NULL) {
@@ -578,13 +616,10 @@ ShrinkEpm (unsigned char *epm, int *width, int *height, int depth)
   unsigned char *pFirst;
   unsigned char *pSecond;
   unsigned char *dst;
-
   assert (NULL !=  width);
   assert (NULL !=  height);
-
   hWidth  = *width  / 2;
   hHeight = *height / 2;
-
   dst = epm;
   for (y = 0; y < depth*hHeight; y ++) {
     pFirst  = epm    + 2*y*(*width);
@@ -615,16 +650,14 @@ ReadEpmFile (const char *path, const char *filename, int *width, int *height, in
   unsigned char *epm = NULL;
   unsigned char *buf = NULL;
   int            nPixel;
-
   assert (width  != NULL);
   assert (height != NULL);
   assert (depth  != NULL);
-
+  /* try to open for reading */
   if (NULL == (fp = FileOpen (path, filename, "epm", "rb") ) ) {
     fprintf(stderr, "failed to open file \"%s/%s.%s\".\n", path,filename, "epm");
     return NULL;
   }
-
   /* read header */
   if (4 != fscanf (fp, "%s%d%d%*d%d%*c", zeile, width, height, depth) ) {
     fprintf(stderr, "Failed to read epm header\n");
@@ -636,7 +669,7 @@ ReadEpmFile (const char *path, const char *filename, int *width, int *height, in
   if (NULL == (epm = malloc (nPixel * sizeof(char) ) ) ) {
     goto Error;
   }
-  /* check magic */  
+  /* check magic */
   if (0 == strcmp (zeile, "PX") ) {
     /* uncompressed data */
     /* read data */
@@ -678,9 +711,7 @@ ReadEpmFile (const char *path, const char *filename, int *width, int *height, in
 #endif
   /* close file */
   fclose (fp);
-
   return epm;
-  
   /* Error handling */
 Error:
   if (fp != NULL) {
@@ -691,7 +722,7 @@ Error:
   }
   if (buf != NULL) {
     free (buf);
-  } 
+  }
   return NULL;
 } /* ReadEpmFile */
 
@@ -708,7 +739,6 @@ ReadRawFile (const char *path, const char *filename, size_t *len)
   size_t       lastRead;
   char        *ptr;
   size_t       i;
-
   /* sanity check */
   assert (NULL != path);
   assert (NULL != filename);
@@ -748,10 +778,8 @@ ReadRawFile (const char *path, const char *filename, size_t *len)
   for (i = 0; i < *len; i ++) {
     buf[i] += 0x80;
   }
-
   /* that's all */
   return buf;
-
  Error:
   if (NULL != buf) {
     free (buf);
