@@ -19,7 +19,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: x11_event.h,v 1.3 2004/05/14 10:00:36 alfie Exp $
+ * $Id: x11_event.h,v 1.4 2004/08/16 22:14:16 iskywalker Exp $
  */
 #ifndef _X11_EVENT_H
 #define _X11_EVENT_H
@@ -30,7 +30,7 @@
  * constants
  */
 #define EVENT_MASK_NORMAL (ExposureMask | StructureNotifyMask | KeyPressMask | KeyReleaseMask)
-#define EVENT_MASK_MOUSE  (EVENT_MASK_NORMAL | ButtonPressMask | PointerMotionMask)
+#define EVENT_MASK_MOUSE  (EVENT_MASK_NORMAL | ButtonPressMask |ButtonReleaseMask| PointerMotionMask)
 
 /*
  * prototypes
