@@ -10,7 +10,7 @@
  * Author: Norbert Nicolay, e-mail: nicolay@ikp.uni-koeln.de
  *         July 30th 1996
  *
- * $Id: xbsndsrv.c,v 1.3 2004/05/28 12:15:10 iskywalker Exp $
+ * $Id: xbsndsrv.c,v 1.18 2005/03/08 17:27:15 lodott Exp $
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -33,6 +33,7 @@
 
 #define _SOUND_C_
 
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -42,11 +43,30 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
+
+#ifdef __sun__
+/* format for pids on sun */
+#define PID_FORMAT "%lu"
+/* needed for bzero(), open()*/
+#include <strings.h>
+#include <fcntl.h>
+/* sun-specific audio includes */
+#ifdef HAVE_SYS_AUDIO_H
+#include <sys/audio.h>
+#else
+#include <sys/audioio.h>
+#include <sys/stropts.h>
+#endif
+#else
+/* format for pids, non-sun */
+#define PID_FORMAT "%u"
+/* non-sun audio includes */
 #ifdef __FreeBSD__
 #include <machine/soundcard.h>
 #else
 #include <linux/soundcard.h>
 #endif
+#endif
 #include <signal.h>
 #include "snd.h"
 
@@ -79,7 +99,7 @@
 #define SND_ACK_OK      0
 #define SND_ACK_ERROR   1
 
-static const char rcs_id[] = "$Id: xbsndsrv.c,v 1.3 2004/05/28 12:15:10 iskywalker Exp $";
+static const char rcs_id[] = "$Id: xbsndsrv.c,v 1.18 2005/03/08 17:27:15 lodott Exp $";
 
 typedef unsigned char u8;
 typedef short s16;
@@ -125,6 +145,7 @@ static struct _sound_name {
   {SND_OUCH,     "xb_ouch.raw", NULL, 0, FALSE, FALSE}, /* player lost life */
   {SND_INTRO,    "xb_intro.raw", NULL, 0, FALSE, FALSE},/* intro fanfare */
   {SND_APPL,     "xb_appl.raw", NULL, 0, FALSE, FALSE}, /* applause */
+  {SND_APPL2,     "xb_app2.raw", NULL, 0, FALSE, FALSE}, /* applause */
   {SND_BUTT,     "xb_butt.raw", NULL, 0, FALSE, TRUE},  /* triggered button */
   {SND_SHOOT,    "xb_shoot.raw", NULL, 0, FALSE, FALSE},/* using rem. ctrl. */
   {SND_INVIS,    "xb_nvis.raw", NULL, 0, FALSE, FALSE}, /* player invisible */
@@ -154,6 +175,12 @@ static struct _sound_name {
   {SND_LIFE,     "xblife.raw", NULL, 0, FALSE, TRUE},   /* extra life */
   {SND_NEWCLOAK, "xbcloakx.raw", NULL, 0, FALSE, TRUE}, /* new cloak extra */
   {SND_BOMBMORPH,"xb_bombmorph.raw", NULL, 0, FALSE, TRUE }, /* bomb morph */
+  {SND_STEP1,     "xbstep1.raw", NULL, 0,  FALSE, TRUE },  /* Backgr. song #1 */
+  {SND_STEP2,     "xbstep2.raw", NULL, 0, FALSE, TRUE },    /* Backgr. song #2 */
+  {SND_STEP3,     "xbstep3.raw", NULL, 0, FALSE, TRUE },    /* Backgr. song #3 */
+  {SND_STEP4,     "xbstep4.raw", NULL, 0, FALSE, TRUE },   /* Backgr. song #4 */
+  {SND_STEP5,     "xbstep5.raw", NULL, 0, FALSE, TRUE },    /* Backgr. song #5 */
+  {SND_STEP6,     "xbstep6.raw", NULL, 0, FALSE, TRUE },    /* Backgr. song #6 */
   {SND_SNG1,     "xbsng1.raw", NULL, 0, TRUE, FALSE},   /* Backgr. song #1 */
   {SND_SNG2,     "xbsng2.raw", NULL, 0, TRUE, FALSE},   /* Backgr. song #2 */
   {SND_SNG3,     "xbsng3.raw", NULL, 0, TRUE, FALSE},   /* Backgr. song #3 */
@@ -163,13 +190,20 @@ static struct _sound_name {
   {SND_MAX,     NULL, NULL, 0}              
 };
 
-#define SUBSIZE          2048
-#define FRAGSIZE         0x0004000a
-
+#ifdef __sun__
+#define SOUND_DEVICE "/dev/audio"
+#define SAMPLE_CHANNELS     2
+#define SAMPLE_SIZE         16
+#define SUN_AUDIO_REF 0
+#else
 #define SOUND_DEVICE "/dev/dsp"
-#define SAMPLE_RATE     22050
 #define SAMPLE_CHANNELS     1
 #define SAMPLE_SIZE         8
+#endif
+
+#define SAMPLE_RATE     22050
+#define SUBSIZE          2048
+#define FRAGSIZE         0x0004000a
 
 static s16 sumbuff[SUBSIZE];
 static u8  playbuff[SUBSIZE];
@@ -235,6 +269,48 @@ init_dsp(dsp)
 int dsp;
 #endif
 {
+
+#ifdef __sun__
+  struct audio_info au;
+
+  bzero(&au,sizeof(struct audio_prinfo));
+
+  fprintf(stderr,"Solaris audio device initialisation beginning.\n");
+
+  if (ioctl(dsp,AUDIO_GETINFO,&au)<0)
+    {
+      fprintf(stderr,"XBlast sound server: "
+              "could not read audio device settings\n");
+    }
+  au.play.sample_rate = sample_rate;
+  au.play.precision = sample_size;
+  au.play.channels = sample_channels;
+  if (sample_channels==0)
+    au.play.channels = 1;
+  au.play.buffer_size = fragsize;
+  au.play.encoding = AUDIO_ENCODING_LINEAR; /* ULAW, ALAW, LINEAR */
+  au.play.balance = AUDIO_MID_BALANCE;
+  au.play.pause=0;
+  au.record.pause=1;
+  au.output_muted=0;
+  if (ioctl(dsp,AUDIO_SETINFO,&au)<0)
+    {
+      fprintf(stderr,"XBlast sound server: "
+              "could not read audio device settings\n");
+    }
+  if (ioctl(dsp,AUDIO_GETINFO,&au)<0)
+    {
+      fprintf(stderr,"XBlast sound server: "
+              "could not read audio device settings\n");
+    }
+  
+  fprintf(stderr,"Audio device initialised: %d channels, %d bits, %dHz.\n",
+         au.play.channels,au.play.precision,au.play.sample_rate);
+  fprintf(stderr,"  (wanted %d channels, %d bits, %dHz)\n",
+          sample_channels,sample_size,sample_rate);
+
+#else
+
   if (ioctl(dsp, SNDCTL_DSP_SETFRAGMENT, &fragsize) < 0)
     {
       fprintf(stderr, "XBlast sound server: could not set fragment size %8x on sound device\n", fragsize);
@@ -255,6 +331,7 @@ int dsp;
     {
       fprintf(stderr, "XBlast sound server: could not get block size of sound device\n");
     }
+#endif
 }
 
 
@@ -278,11 +355,17 @@ int dsp;
     {
       *s++ = 128;
     }
+
+#ifdef __sun__
+  ioctl(dsp,I_FLUSH,NULL);
+  fprintf(stderr,"\nsync\n");
+#else
   /* resync sound device to correct any channel flipping */
   write(dsp, playbuff, SUBSIZE);
   write(dsp, playbuff, SUBSIZE);
   write(dsp, playbuff, SUBSIZE);
   (void) ioctl(dsp, SNDCTL_DSP_SYNC, NULL);
+#endif
 }
 
 /*
@@ -314,6 +397,7 @@ int number;
 	{
 	  sprintf(fname, "%s/%s/%s", path_list[i], "sounds", 
 		  sound_name[number].name);
+	  //	  fprintf(stderr," opening %s \n",fname);
 	  if ((f = open(fname, O_RDONLY)) >= 0)
 	    {
 	      int sound_size;
@@ -398,6 +482,48 @@ int id;
     }
 }
 
+int RunningInstances(void){
+  FILE *ptr;
+  static char userPath[1024];
+  char tmp[1024];
+  unsigned int pid;
+
+  char *home;
+  /* set private xblast path */
+  home = getenv ("HOME");
+  if (NULL == home) {
+    strcpy (userPath, "./user/xbsndsrv.pid");
+  } else {
+    sprintf (userPath, "%s/.xblast_tnt/xbsndsrv.pid", home);
+  }
+  ptr=fopen(userPath ,"r+");
+  if(ptr==NULL){
+    ptr=fopen(userPath ,"w+");
+    sprintf(tmp, PID_FORMAT, getpid());
+    fwrite(tmp,1,strlen(tmp),ptr);
+    fclose(ptr);
+    return 0;
+
+  }
+  fscanf(ptr, PID_FORMAT, &pid);
+  fprintf(stderr," "PID_FORMAT" \n",pid);
+  if(kill(pid,0)==-1 || pid<=0)
+    {
+      fseek(ptr,0,SEEK_SET);
+      sprintf(tmp, PID_FORMAT, getpid());
+      fwrite(tmp,1,strlen(tmp),ptr);
+      fclose(ptr);
+      return 0;
+    }
+  else{
+    return 1;
+  }
+    
+
+
+}
+
+
 /*
  * main function
  */
@@ -416,6 +542,15 @@ char **argv;
   int did_sync = FALSE;
   int ack_val;
 
+#ifdef __sun__
+  int samples_to_play=0;
+  int samples_played=0;
+#endif
+
+  if(RunningInstances()){
+    fprintf(stderr,"xbsndsrv already running\n");
+    return 1;
+  }
   /*
    * open and prepare sound device
    */
@@ -446,7 +581,11 @@ char **argv;
 
   if (mono_mode == TRUE)
     {
+#ifdef __sun__
+      sample_channels = 1;
+#else
       sample_channels = 0;
+#endif
     }
 
   init_dsp(dsp);
@@ -455,7 +594,7 @@ char **argv;
    */
   signal(SIGINT, server_abort);
 
-  fprintf(stderr, "XBlast sound server $Revision: 1.3 $ running in %s mode.\n",
+  fprintf(stderr, "XBlast sound server $Revision: 1.18 $ running in %s mode.\n",
 	  (mono_mode == TRUE) ? "mono" : "stereo");
 
 
@@ -472,7 +611,11 @@ char **argv;
       {
 	l++;
 	/* do something similar to the "place a sample" loop */
-	d5 = d2++ - 128;
+#ifdef __sun__
+        d5 = d2++ - SUN_AUDIO_REF;
+#else
+        d5 = d2++ - 128;
+#endif
 	d1 = (d2 * (d3 + 1)) >> 4;
 	d4 = (d2 * (d3 - MAX_SOUND_POSITION + 1)) >> 4;
 	d2 += d1;
@@ -559,7 +702,11 @@ char **argv;
 		  
 		  for (i = 0; i < SUBSIZE && xs->length > 0; i++, xs->length--)
 		    {
-		      *s++ += ((s16) *xs->samples++) - 128;
+#ifdef __sun__
+                      *s++ += ((s16) *xs->samples++) - SUN_AUDIO_REF;
+#else
+                      *s++ += ((s16) *xs->samples++) - 128;
+#endif
 		    }
 
 #if defined(SERVER_STATISTICS)
@@ -638,7 +785,11 @@ char **argv;
 			{
 			  /* calc. position of mono sounds and add to sumup buffer */
 			  int pos    = xs->position;
-			  s16 sample = ((s16) *xs->samples++) - 128;
+#ifdef __sun__
+                          s16 sample = ((s16) *xs->samples++) - SUN_AUDIO_REF;
+#else
+                          s16 sample = ((s16) *xs->samples++) - 128;
+#endif
 			  s16 sr, sl;
 		      
 			  sr = (sample*(pos+1)) >> 4;
@@ -649,13 +800,20 @@ char **argv;
 			}
 		      else
 			{
-			  *s++ += ((s16) *xs->samples++) - 128;
+#ifdef __sun__
+                          *s++ += ((s16) *xs->samples++) - SUN_AUDIO_REF;
+#else
+                          *s++ += ((s16) *xs->samples++) - 128;
+#endif
 			}
 		    }
 #if defined(SERVER_STATISTICS)
 		  total_samples += (double) i;
 #endif
 		  /* repeat a sound if this is required */
+#ifdef __sun__
+                  samples_to_play = i;
+#endif
 		  if (xs->length <= 0)
 		    {
 		      if (xs->repeat == TRUE)
@@ -673,25 +831,48 @@ char **argv;
 		}
 	      xs++;
 	    }
-	  
-	  /* correct clipping */
-	  {
-	    register int i;
-	    register s16 *s = sumbuff;
-	    for (i = 0; i < SUBSIZE; i++)
-	      {
-		if (*s > 255)
-		  {
-		    *s = 255;
-		  }
-		else if (*s < 0)
-		  {
-		    *s = 0;
-		  }
-		s++;
-	      }
-	  }
-      
+
+          /* correct clipping */
+          switch(sample_size)
+            {
+            case 8:
+              {
+                register int i;
+                register s16 *s = sumbuff;
+                for (i = 0; i < SUBSIZE; i++)
+                  {
+                    if (*s > 255)
+                      {
+                        *s = 255;
+                      }
+                    else if (*s < 0)
+                      {
+                        *s = 0;
+                      }
+                    s++;
+                  }
+              }
+              break;
+            case 16:
+              {
+                register int i;
+                register s16 *s = sumbuff;
+                for (i = 0; i < SUBSIZE; i++)
+                  {
+                    if (*s > 8191)
+                      {
+                        *s = 8191;
+                      }
+                    (*s) = (*s)<<5;
+                    if (*s < -8191)
+                      {
+                        *s = -8191;
+                      }
+                    s++;
+                  }
+              }       
+            }
+
 	  /* copy sum buffer to playback buffer and play it */
 	  {
 	    register u8 *d = playbuff;
@@ -704,16 +885,79 @@ char **argv;
 	      }
 	    
 	    /* play buffer */
-	    write(dsp, playbuff, SUBSIZE);
-	  }
+
+#ifdef __sun__
+            switch(sample_size)
+              {
+              case 8:
+                write(dsp, playbuff, samples_to_play);
+                samples_played+=samples_to_play;
+                break;
+              case 16:
+                write(dsp, sumbuff, samples_to_play*2);
+                samples_played+=samples_to_play>>1;
+                break;
+              }
+
+            /* Wait for output to mostly finish playing */
+            {
+              struct audio_info au;
+
+              /* ioctl(dsp,AUDIO_GETINFO,&au);
+                 au.play.pause=0;
+                 au.play.error=0;
+                 ioctl(dsp,AUDIO_SETINFO,&au); */
+              ioctl(dsp,AUDIO_GETINFO,&au);
+
+              /* 
+                 fprintf(stderr,
+                 "\rsamples_played = %10d, played = %10d, error=%d",
+                 samples_played,au.play.samples,au.play.error);
+                 
+                 fflush(stdout); 
+              */
+
+              while((samples_played-au.play.samples)>(SAMPLE_RATE>>4))
+                {
+                  
+                  if (ioctl(dsp,AUDIO_GETINFO,&au)<0)
+                    {
+                      fprintf(stderr,"XBlast sound server: "
+                              "could not read audio device settings\n");
+                    }
+                  /* if (au.play.error) break; */
+                }
+            }
+
+#else
+            switch(sample_size)
+              {
+              case 8:
+	        write(dsp, playbuff, SUBSIZE);
+                break;
+              case 16:
+                write(dsp, sumbuff, SUBSIZE*2);
+                break;
+              }
+
+#endif
+          }
+
 	}
 #if 1
       /* resync sound device to correct any channel flipping */
       if (do_sync == TRUE && did_sync == FALSE)
 	{
-	  (void) ioctl(dsp, SNDCTL_DSP_SYNC, NULL);
-	  did_sync = TRUE;
-	}
+
+#ifdef __sun__
+          /* ioctl(dsp,AUDIO_DRAIN,NULL); */
+          ioctl(dsp,I_FLUSH,NULL);
+          fprintf(stderr,"\nsync\n");
+#else
+          (void) ioctl(dsp, SNDCTL_DSP_SYNC, NULL);
+#endif
+          did_sync = TRUE;
+        }
 #endif
 
       /* check for new commands in input pipe */
@@ -726,6 +970,7 @@ char **argv;
 
 	while(1)
 	  {
+	    
 	    tv.tv_sec = tv.tv_usec = 0;
 	    FD_ZERO(&rs);
 	    FD_SET(0, &rs);
@@ -733,6 +978,10 @@ char **argv;
 	    if (select(1, &rs, NULL, NULL, &tv) > 0 && FD_ISSET(0, &rs))
 	      {
 		n = read(0, command_buff, 8);
+		if ((n == 0) || (n < 0 && errno != EINTR && errno != EAGAIN)) {
+		  fprintf(stderr, "Parent was killed, bailing out ...\n");
+		  exit(0);
+		}
 		cmd = command_buff;
 		/* there are commands in the pipe */
 		while (n > 0)
